Package: basksim
Type: Package
Title: Simulation-Based Calculation of Basket Trial Operating
        Characteristics
Version: 2.2.0
Authors@R: c(
    person("Lukas", "Baumann", , "baumann@imbi.uni-heidelberg.de", 
           role = c("aut", "cre"),
	         comment = c(ORCID = "0000-0001-7931-7470")),
    person("Lukas D", "Sauer", ,"sauer@imbi.uni-heidelberg.de", 
           role = c("aut"),
		       comment = c(ORCID = "0000-0002-1340-9994")),
		person("Sabrina", "Schmitt", ,"sabrina.schmitt@uni-wuerzburg.de", 
		       role = c("aut"),
		       comment = c(ORCID = "0009-0008-2719-6513"))
  )
Description: Provides a unified syntax for the simulation-based comparison of 
  different single-stage basket trial designs with a binary endpoint and equal 
  sample sizes in all baskets. Methods include the designs by 
	Baumann et al. (2025) <doi:10.1080/19466315.2024.2402275>, 
	Schmitt and Baumann (2025) <doi:10.1080/19466315.2025.2486231>,
	Fujikawa et al. (2020) <doi:10.1002/bimj.201800404>,
	Berry et al. (2020) <doi:10.1177/1740774513497539>, and
	Neuenschwander et al. (2016) <doi:10.1002/pst.1730>. For the 
	latter two designs, the functions are mostly 
	wrappers for functions provided by the package 'bhmbasket'.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: arrangements, bhmbasket (>= 1.0.0), doFuture, extraDistr,
        foreach, HDInterval, progressr, purrr, stats
Suggests: covr, testthat (>= 3.0.0), ggplot2
Config/testthat/edition: 3
URL: https://github.com/lbau7/basksim
BugReports: https://github.com/lbau7/basksim/issues
NeedsCompilation: no
Packaged: 2026-05-11 21:18:48 UTC; or415
Author: Lukas Baumann [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7931-7470>),
  Lukas D Sauer [aut] (ORCID: <https://orcid.org/0000-0002-1340-9994>),
  Sabrina Schmitt [aut] (ORCID: <https://orcid.org/0009-0008-2719-6513>)
Maintainer: Lukas Baumann <baumann@imbi.uni-heidelberg.de>
Repository: CRAN
Date/Publication: 2026-05-12 07:30:02 UTC
Built: R 4.5.2; ; 2026-05-12 10:06:09 UTC; unix

Package: basksim
Type: Package
Title: Simulation-Based Calculation of Basket Trial Operating
        Characteristics
Version: 2.2.0
Authors@R: c(
    person("Lukas", "Baumann", , "baumann@imbi.uni-heidelberg.de", 
           role = c("aut", "cre"),
	         comment = c(ORCID = "0000-0001-7931-7470")),
    person("Lukas D", "Sauer", ,"sauer@imbi.uni-heidelberg.de", 
           role = c("aut"),
		       comment = c(ORCID = "0000-0002-1340-9994")),
		person("Sabrina", "Schmitt", ,"sabrina.schmitt@uni-wuerzburg.de", 
		       role = c("aut"),
		       comment = c(ORCID = "0009-0008-2719-6513"))
  )
Description: Provides a unified syntax for the simulation-based comparison of 
  different single-stage basket trial designs with a binary endpoint and equal 
  sample sizes in all baskets. Methods include the designs by 
	Baumann et al. (2025) <doi:10.1080/19466315.2024.2402275>, 
	Schmitt and Baumann (2025) <doi:10.1080/19466315.2025.2486231>,
	Fujikawa et al. (2020) <doi:10.1002/bimj.201800404>,
	Berry et al. (2020) <doi:10.1177/1740774513497539>, and
	Neuenschwander et al. (2016) <doi:10.1002/pst.1730>. For the 
	latter two designs, the functions are mostly 
	wrappers for functions provided by the package 'bhmbasket'.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: arrangements, bhmbasket (>= 1.0.0), doFuture, extraDistr,
        foreach, HDInterval, progressr, purrr, stats
Suggests: covr, testthat (>= 3.0.0), ggplot2
Config/testthat/edition: 3
URL: https://github.com/lbau7/basksim
BugReports: https://github.com/lbau7/basksim/issues
NeedsCompilation: no
Packaged: 2026-05-11 21:18:48 UTC; or415
Author: Lukas Baumann [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7931-7470>),
  Lukas D Sauer [aut] (ORCID: <https://orcid.org/0000-0002-1340-9994>),
  Sabrina Schmitt [aut] (ORCID: <https://orcid.org/0009-0008-2719-6513>)
Maintainer: Lukas Baumann <baumann@imbi.uni-heidelberg.de>
Repository: CRAN
Date/Publication: 2026-05-12 07:30:02 UTC
Built: R 4.6.0; ; 2026-05-12 10:21:11 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(adjust_lambda,bhm)
S3method(adjust_lambda,default)
S3method(adjust_lambda,exnex)
S3method(geom_borrow,fujikawa)
S3method(geom_posterior,fujikawa)
S3method(geom_prior,fujikawa)
S3method(get_details,app)
S3method(get_details,bhm)
S3method(get_details,binomial)
S3method(get_details,cpp)
S3method(get_details,cppglobal)
S3method(get_details,cpplim)
S3method(get_details,exnex)
S3method(get_details,fujikawa)
S3method(get_details,jsdglobal)
S3method(get_details,mml)
S3method(get_details,mmlglobal)
S3method(get_evaluation,app)
S3method(get_evaluation,bhm)
S3method(get_evaluation,cpp)
S3method(get_evaluation,cpplim)
S3method(get_evaluation,exnex)
S3method(get_evaluation,fujikawa)
S3method(get_results,app)
S3method(get_results,bhm)
S3method(get_results,cpp)
S3method(get_results,cppglobal)
S3method(get_results,cpplim)
S3method(get_results,exnex)
S3method(get_results,fujikawa)
S3method(get_results,jsdglobal)
S3method(get_results,mml)
S3method(get_results,mmlglobal)
export(adjust_lambda)
export(ecd)
export(geom_borrow)
export(geom_posterior)
export(geom_prior)
export(get_data)
export(get_details)
export(get_evaluation)
export(get_results)
export(get_scenarios)
export(opt_design)
export(setup_app)
export(setup_bhm)
export(setup_binomial)
export(setup_cpp)
export(setup_cppglobal)
export(setup_cpplim)
export(setup_exnex)
export(setup_fujikawa)
export(setup_jsdglobal)
export(setup_mml)
export(setup_mmlglobal)
export(toer)
importFrom(doFuture,"%dofuture%")

<!-- README.md is generated from README.Rmd. Please edit that file -->

# basksim

<!-- badges: start -->

[![R-CMD-check](https://github.com/lbau7/basksim/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/lbau7/basksim/actions/workflows/R-CMD-check.yaml)
[![codecov](https://codecov.io/gh/lbau7/basksim/branch/master/graph/badge.svg?token=AVO4V52BTH)](https://app.codecov.io/gh/lbau7/basksim)
[![CRAN
status](https://www.r-pkg.org/badges/version/basksim)](https://CRAN.R-project.org/package=basksim)
<!-- badges: end -->

## Overview

`basksim` calculates the operating characteristics of different basket
trial designs based on simulation.

## Installation

Install the development version with:

``` r
# install.packages("devtools")
pak::pak("lbau7/basksim")
```

## Usage

With `basksim` you can calculate the operating characteristics such as
rejection probabilities and mean squared error of single-stage basket
trials with different designs.

At first, you have to create a design-object using a setup-function. For
example to create a design-object for Fujikawa’s design (Fujikawa et
al., 2020):

``` r
library(basksim)
design <- setup_fujikawa(k = 3, shape1 = 1, shape2 = 1, p0 = 0.2)
```

`k` is the number of baskets, `shape1` and `shape2` are the shape
parameters of the Beta-prior of the response probabilities of each
baskets and `p0` is the response probability that defines the null
hypothesis.

Use `get_details` to estimate several important operating
characteristics:

``` r
set.seed(123)
get_details(
  design = design,
  n = c(15, 20, 25),
  p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95,
  epsilon = 1.5,
  tau = 0,
  iter = 5000
)

# $Rejection_Probabilities
# [1] 0.4226 0.9824 0.9874
# 
# $FWER
# [1] 0.4226
# 
# $EWP
# [1] 0.999
# 
# $Mean
# [1] 0.2992626 0.4823250 0.4836304
# 
# $MSE
# [1] 0.020532553 0.007330251 0.006862607
# 
# $Lower_CL
# [1] 0.1517281 0.3407342 0.3440962
# 
# $Upper_CL
# [1] 0.4574680 0.6241900 0.6234426
# 
# $ECD
# [1] 2.5472
# 
# $Rejection_Probabilities_SE
# [1] 0.006985832 0.001859583 0.001577418
# 
# $FWER_SE
# [1] 0.006985832
# 
# $EWP_SE
# [1] 0.0004469899
# 
# $ECD_SE
# [1] 0.007147353
```

#' Adjust Lambda
#'
#' @template design
#' @template dotdotdot
#'
#' @details The default method for \code{adjust_lambda} uses a combination
#' of \code{\link{uniroot}} and grid search and calls \code{\link{toer}}
#' in every iteration. For methods implemented in the \code{bhmbasket} package
#' there are separate methods that are computationally more efficient.
#'
#' @return A list containing the greatest estimated value for \code{lambda} with
#' \code{prec_digits} decimal places which controls the family wise error rate
#' at level \code{alpha} (one-sided) and the estimated family wise error rate
#' for the estimated \code{lambda}.
#' @export
#'
#' @examples
#' \donttest{
#' design <- setup_cpp(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' adjust_lambda(design = design, n = 20, alpha = 0.05,
#'   design_params = list(tune_a = 1, tune_b = 1), iter = 1000)
#'
#' # Unequal sample sizes
#' adjust_lambda(design = design, n = c(15, 20, 25), alpha = 0.05,
#'    design_params = list(tune_a = 1, tune_b = 1), iter = 1000)
#' }
adjust_lambda <- function(design, ...) {
  UseMethod("adjust_lambda", design)
}

#' Adjust Lambda
#'
#' @template design
#' @template n
#' @template p1_toer
#' @template alpha
#' @template design_params
#' @template iter
#' @template prec_digits
#' @template dotdotdot
#' @template data
#'
#' @details It is recommended to use \code{data} and then use the same simulated
#' data set for all further calculations. If \code{data = NULL} then
#' new data are generated in each step of the algorithm, so \code{lambda} doesn't
#' necessarily protect the family wise error rate for different simulated data
#' due to Monte Carlo simulation error.
#'
#' @return A list containing the greatest estimated value for \code{lambda} with
#' \code{prec_digits} decimal places which controls the family wise error rate
#' at level \code{alpha} (one-sided) and the estimated family wise error rate
#' for the estimated \code{lambda}.
#' @export
#'
#' @examples
#' \donttest{
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' adjust_lambda(design = design, n = 20, alpha = 0.05,
#'   design_params = list(epsilon = 2, tau = 0), iter = 1000)
#'
#' # Unequal sample sizes
#' adjust_lambda(design = design, n = c(15, 20, 25), alpha = 0.05,
#'   design_params = list(epsilon = 2, tau = 0), iter = 1000)
#' }
adjust_lambda.default <- function(design, n, p1 = NULL, alpha = 0.05,
                          design_params = list(), iter = 1000, prec_digits = 3,
                          data = NULL, ...) {
  if (is.null(p1)) p1 <- rep(design$p0, design$k)
  upper_lim <- 1 - 10^(-prec_digits)
  root_fun <- function(x) do.call(toer, args = c(design = list(design),
    n = list(n), p1 = list(p1), lambda = x, design_params, iter = iter,
    data = list(data),
    ...)) - alpha

  # Use uniroot to find lambda close to the smallest lambda that protects
  # the significance level at alpha
  uni_root <- stats::uniroot(root_fun, interval = c(0.5, upper_lim),
    tol = 10^(-prec_digits))

  if (uni_root$f.root > 0 ) {
    # If rejection prob is greater than alpha after uniroot, round lambda up
    root <- ceiling(uni_root$root * 10^(prec_digits)) / 10^(prec_digits)
    val <- root_fun(root)
    if (val > 0) {
      # If rejection prob is still above alpha, increase lambda
      while (val > 0) {
        root <- root + 10^(-prec_digits)
        val <- root_fun(root)
      }
    }
  } else {
    # If rejection prob is less than alpha after uniroot, round lambda down
    root <- floor(uni_root$root * 10^(prec_digits)) / 10^(prec_digits)
    val <- root_fun(root)
    if (val > 0) {
      # If the rejection prob is greater now than alpha with the rounded-down
      # lambda, then round lambda up
      root <- ceiling(uni_root$root * 10^(prec_digits)) / 10^(prec_digits)
      val <- root_fun(root)
    } else {
      # If the rejection prob is still below alpha, decrease lambda
      repeat {
        root_old <- root
        val_old <- val
        root <- root - 10^(-prec_digits)
        val <- root_fun(root)
        if (val > 0) {
          root <- root_old
          val <- val_old
          break
        }
      }
    }
  }
  list(
    lambda = root,
    toer = val + alpha
  )
}

#' Adjust Lambda for the EXNEX Design
#'
#' @template design
#' @template n
#' @template p1
#' @template alpha
#' @template design_params
#' @template iter
#' @template n_mcmc
#' @template prec_digits
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the greatest estimated value for \code{lambda} with
#' \code{prec_digits} decimal places which controls the family wise error rate
#' at level \code{alpha} (one-sided) and the estimated family wise error rate
#' for the estimated \code{lambda}.
#' @export
#'
#' @examples
#' design <- setup_exnex(k = 3, p0 = 0.2)
#' \donttest{
#' # Equal sample sizes
#' adjust_lambda(design = design, n = 15,
#'   design_params = list(tau_scale = 1, w_j = 0.5),
#'   iter = 100, n_mcmc = 5000)
#'
#' # Unequal sample sizes
#' adjust_lambda(design = design, n = c(15, 20, 25),
#'   design_params = list(tau_scale = 1, w_j = 0.5),
#'   iter = 100, n_mcmc = 5000)
#'   }
adjust_lambda.exnex <- function(design, n, p1 = NULL, alpha = 0.05,
                                design_params = list(), iter = 1000,
                                n_mcmc = 10000, prec_digits = 3, data = NULL,
                                ...) {
  if (is.null(p1)) p1 <- rep(design$p0, design$k)
  data <- check_data_bhmbasket(data = data, design = design, n = n, p = p1,
    iter = iter)
  levels <- seq(1 - alpha - 0.1, 0.999, by = 10^(-prec_digits))

  prior_parameters <- do.call(bhmbasket::setPriorParametersExNex,
    args = c(mu_mean = design$mu_mean, mu_sd = design$mu_sd,
      mu_j = list(rep(design$basket_mean, design$k)),
      tau_j = list(rep(design$basket_sd, design$k)), design_params))

  analyses <- suppressMessages(do.call(bhmbasket::performAnalyses,
    list(
      scenario_list = data,
      evidence_levels = levels,
      method_names = "exnex",
      prior_parameters_list = prior_parameters,
      n_mcmc_iterations = n_mcmc
    )))

  br <- paste0("c(", paste0("x[", 1:design$k, "] > ", design$p0,
    collapse = ", "), ")")
  alphabhm <- function(x) {
    res <- bhmbasket::getGoDecisions(
      analyses_list = analyses,
      cohort_names = paste("p", 1:design$k, sep = "_"),
      evidence_levels = rep(x, design$k),
      boundary_rules = str2lang(br)
    )
    mean(res$scenario_1$decisions_list$exnex[, 1])
  }

  fwers <- sapply(levels, alphabhm)
  ind <- which.max(fwers <= alpha)

  list(
    lambda = levels[ind],
    toer = fwers[ind]
  )
}

#' Adjust Lambda for the BHM Design
#'
#' @template design
#' @template n
#' @template p1
#' @template alpha
#' @template design_params
#' @template iter
#' @template n_mcmc
#' @template prec_digits
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the greatest estimated value for \code{lambda} with
#' \code{prec_digits} decimal places which controls the family wise error rate
#' at level \code{alpha} (one-sided) and the estimated family wise error rate
#' for the estimated \code{lambda}.
#' @export
#'
#' @examples
#' design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
#' \donttest{
#' # Equal sample sizes
#' adjust_lambda(design = design, n = 15, design_params = list(tau_scale = 1),
#'   iter = 100, n_mcmc = 5000)
#'
#' # Unequal sample sizes
#' adjust_lambda(design = design, n = c(15, 20, 25),
#'   design_params = list(tau_scale = 1),
#'   iter = 100, n_mcmc = 5000)
#' }
adjust_lambda.bhm <- function(design, n, p1 = NULL, alpha = 0.05,
                              design_params = list(), iter = 1000,
                              n_mcmc = 10000, prec_digits = 3, data = NULL,
                              ...) {
  if (is.null(p1)) p1 <- rep(design$p0, design$k)
  data <- check_data_bhmbasket(data = data, design = design, n = n, p = p1,
    iter = iter)
  levels <- seq(1 - alpha - 0.1, 0.999, by = 10^(-prec_digits))

  prior_parameters <- do.call(bhmbasket::setPriorParametersBerry,
    args = c(mu_mean = design$mu_mean, mu_sd = design$mu_sd, design_params))

  analyses <- suppressMessages(do.call(bhmbasket::performAnalyses,
    list(
      scenario_list = data,
      evidence_levels = levels,
      method_names = "berry",
      target_rates = rep(design$p_target, design$k),
      prior_parameters_list = prior_parameters,
      n_mcmc_iterations = n_mcmc
    )))

  br <- paste0("c(", paste0("x[", 1:design$k, "] > ", design$p0,
    collapse = ", "), ")")
  alphabhm <- function(x) {
    res <- bhmbasket::getGoDecisions(
      analyses_list = analyses,
      cohort_names = paste("p", 1:design$k, sep = "_"),
      evidence_levels = rep(x, design$k),
      boundary_rules = str2lang(br)
    )
    mean(res$scenario_1$decisions_list$berry[, 1])
  }

  fwers <- sapply(levels, alphabhm)
  ind <- which.max(fwers <= alpha)

  list(
    lambda = levels[ind],
    toer = fwers[ind]
  )
}

# Beta Borrowing for Limited CPP Design
beta_borrow_cpplim <- function(design, n, r, weights, alpha_0 = alpha_0){

  shape_noprior <- matrix(c(r, n - r), nrow = 2, byrow = TRUE)
  weight_mat <- get_weight_mat_pp(design, n, r, weights)

  # Compute posterior shapes
  shape1post <- apply(alpha_0*weight_mat, 1, function(x) sum(shape_noprior[1, ] * x)) +
    design$shape1
  shape2post <- apply(alpha_0*weight_mat, 1, function(x) sum(shape_noprior[2, ] * x)) +
    design$shape2
  rbind(shape1post, shape2post)

}




# Analyzing Results for limited CPP Design
ana_cpplim <- function(design, n, r, lambda, weights, alpha_0){
  shape_post <- beta_borrow_cpplim(design = design, n = n, r = r,
                                   weights = weights, alpha_0 = alpha_0)
  post_prob <- post_beta(shape = shape_post, p0 = design$p0)
  ifelse(post_prob >= lambda, 1, 0)
}

test_that("beta_borrow_cpplim works", {
  # compare with CPP (alpha_0 = 1)
  design_cpplim <- setup_app(k = 3, p0 = 0.2)
  design_cpp <- setup_cpp(k = 3, p0 = 0.2)

  n <- 15
  r <- rep(10,3)

  weights <- get_weights_cpp(n = n, tune_a = 1, tune_b = 1)
  alpha_0 <- get_alpha_0_app(design = design_cpplim, n = n)

  expect_equal(beta_borrow_cpplim(design = design_cpplim, n = n, r = r,
                                  weights = weights, alpha_0 = alpha_0),
               beta_borrow_pp(design = design_cpp, n = n, r = r,
                               weights = weights))


})


test_that("ana_cpplim works", {
  # compare with CPP (alpha_0 = 1)
  design_cpplim <- setup_app(k = 3, p0 = 0.2)
  design_cpp <- setup_cpp(k = 3, p0 = 0.2)

  n <- 15
  r <- rep(10,3)
  lambda <- 0.987

  weights <- get_weights_cpp(n = n, tune_a = 1, tune_b = 1)
  alpha_0 <- get_alpha_0_app(design = design_cpplim, n = n)

  expect_equal(ana_cpplim(design = design_cpplim, n = n, r = r, lambda = lambda,
                       weights = weights, alpha_0 = alpha_0),
               ana_pp(design = design_cpp, n = n, r = r, lambda = lambda,
                       weights = weights))

})

# Weight matrix with JSD weights
get_weights_jsd <- function(design, n, epsilon, tau, logbase, ...) {
  # the format of the weights depends on whether the sample sizes of the
  # individual baskets are all the same
  if(length(unique(n)) == 1 || length(n) == 1){

    n <- ifelse(length(n) == 1, n, n[1])

    shape1_post <- design$shape1 + c(0:n)
    shape2_post <- design$shape2 + c(n:0)
    n_sum <- n + 1

    p <- function(x) stats::dbeta(x, shape1_post[i], shape2_post[i])
    q <- function(x) stats::dbeta(x, shape1_post[j], shape2_post[j])
    m <- function(x) 0.5 * (p(x) + q(x))
    f <- function(x) p(x) * log(p(x) / m(x), base = logbase)
    g <- function(x) q(x) * log(q(x) / m(x), base = logbase)

    jsd_mat <- matrix(0, nrow = n_sum, ncol = n_sum)
    for (i in 1:n_sum) {
      for (j in i:n_sum) {
        if (i == j) {
          next
        } else {
          kl_f <- stats::integrate(f, 0, 1)$value
          kl_g <- stats::integrate(g, 0, 1)$value
          jsd_mat[i, j] <- 0.5 * kl_f + 0.5 * kl_g
        }
      }
    }
    jsd_mat <- jsd_mat + t(jsd_mat)
    weight_mat <- (1 - jsd_mat)^epsilon
    weight_mat[weight_mat <= tau] <- 0

    weight_mat

  } else {

    # find all possible combinations
    unique_combs <- unique(t(apply(arrangements::combinations(k = 2, v = n),1,sort)))

    weights_list <- list()

    for(l in 1:dim(unique_combs)[1]){
      dim_act <- unique_combs[l,]
      n1 <- dim_act[1]
      n2 <- dim_act[2]

      jsd_mat <- matrix(0, nrow = n1+1, ncol = n2+1)

      shape1_post1 <- design$shape1 + c(0:n1)
      shape1_post2 <- design$shape1 + c(0:n2)

      shape2_post1 <- design$shape2 + c(n1:0)
      shape2_post2 <- design$shape2 + c(n2:0)

      p <- function(x) stats::dbeta(x, shape1_post1[i], shape2_post1[i])
      q <- function(x) stats::dbeta(x, shape1_post2[j], shape2_post2[j])
      m <- function(x) 0.5 * (p(x) + q(x))
      f <- function(x) p(x) * log(p(x) / m(x), base = logbase)
      g <- function(x) q(x) * log(q(x) / m(x), base = logbase)

      for (i in 1:(n1+1) ) {
        for (j in 1:(n2+1)) {

          kl_f <- stats::integrate(f, 0, 1)$value
          kl_g <- stats::integrate(g, 0, 1)$value
          jsd_mat[i, j] <- 0.5 * kl_f + 0.5 * kl_g

        }
      }

      weight_mat <- (1 - jsd_mat)^epsilon
      weight_mat[weight_mat <= tau] <- 0

      weights_list[[l]] <- weight_mat

    }

    weights_list

  }
}

# Weight matrix with MML weights
get_weights_mml <- function(design, n, ...) {
  n_sum <- n + 1
  mat <- matrix(0, nrow = n_sum, ncol = n_sum)
  r <- 0:n
  for (i in 1:n_sum) {
    for (j in 1:n_sum) {
      f <- function(delta) -extraDistr::dbbinom(
        x = r[i],
        size = n,
        alpha = design$shape1 + delta * r[j],
        beta = design$shape2 + delta * (n - r[j])
      )

      l <- stats::optim(0.5, fn = f, lower = 0, upper = 1,
                        method = "Brent")$par
      mat[i, j] <- ifelse(l <= 6.474096e-09, 0, l)
    }
  }
  mat <- (mat + t(mat)) / 2
  diag(mat) <- 1
  mat
}


# Weight matrix with CPP weights
get_weights_cpp <- function(n, tune_a = 1, tune_b = 1, ...) {
  # The format of the weights depends on whether the sample sizes of the
  # individual baskets are all the same
  if(length(unique(n)) == 1 || length(n) == 1){

    n <- ifelse(length(n) == 1, n, n[1])

    n_sum <- n + 1
    weight_mat <- matrix(0, nrow = n_sum, ncol = n_sum)
    r1 <- r2 <- 0:n

    g <- function(s, a, b) {
      1 / (1 + exp(a + b * log(s)))
    }

    for (i in 1:n_sum) {
      for (j in i:n_sum) {
        if (i == j) {
          next
        } else {
          vec1 <- rep(0:1, c(n - r1[i], r1[i]))
          vec2 <- rep(0:1, c(n - r2[j], r2[j]))
          ks <- suppressWarnings(stats::ks.test(vec1, vec2)$statistic)
          s <- n^(1/4) * ks
          weight_mat[i, j] <- g(s = s, a = tune_a, b = tune_b)
        }
      }
    }
    weight_mat <- weight_mat + t(weight_mat)
    diag(weight_mat) <- 1
    weight_mat

  } else {
    # find all possible combinations
    unique_combs <- unique(t(apply(arrangements::combinations(k = 2, v = n),1,sort)))

    g <- function(s, a, b) {
      1 / (1 + exp(a + b * log(s)))
    }

    weights_list <- list()

    for(l in 1:dim(unique_combs)[1]){
      dim_act <- unique_combs[l,]
      n1 <- dim_act[1]
      n2 <- dim_act[2]

      weight_mat <- matrix(0, nrow = n1+1, ncol = n2+1)
      r1 <- 0:n1
      r2 <- 0:n2

      for(i in 1:length(r1)){
        for(j in 1:length(r2)){
          vec1 <- rep(0:1, c(n1 - r1[i], r1[i]))
          vec2 <- rep(0:1, c(n2 - r2[j], r2[j]))
          ks <- suppressWarnings(stats::ks.test(vec1, vec2)$statistic)
          s <- max(n1,n2)^(1/4) * ks
          weight_mat[i, j] <- g(s = s, a = tune_a, b = tune_b)
        }
      }
      weights_list[[l]] <- weight_mat
    }

    weights_list

  }
}

test_that("get_weight_mat_pp works", {
  design <- setup_cpp(k = 4, p0 = 0.2)

  n_int <- 5
  n_vec <- c(5,5,6,7)
  r <- c(4,5,5,3)

  weights_int <- get_weights_cpp(n = n_int, tune_a = 1, tune_b = 1)
  weights_vec <- get_weights_cpp(n = n_vec, tune_a = 1, tune_b = 1)

  expect_equal(get_weight_mat_pp(design = design, n = n_int, r = r, weights = weights_int)[1:2,1:2],
               get_weight_mat_pp(design = design, n = n_vec, r = r, weights = weights_vec)[1:2,1:2])

})


test_that("beta_borrow_pp works", {

  design <- setup_cpp(k = 3, p0 = 0.2)
  weights <- get_weights_cpp(n = c(4,6,8), tune_a = 1, tune_b = 1)

  expect_equal(rbind(shape1post = c(5.284732, 9.560643, 9.728394),
                     shape2post = c(5.826573, 6.601986, 6.78523)),
               beta_borrow_pp(design = design, n = c(4,6,8), r = c(1,4,5), weights = weights),
               tolerance = 1e-6)

})

test_that("beta_borrow_pp works", {
  design <- setup_cpp(k = 4, p0 = 0.2)
  n <- 20
  r <- c(10, 12, 14, 16)

  weight_mat <- get_weights_cpp(n = 20, a = 1, b = 1)
  res1 <- beta_borrow_pp(design = design, n = 20, r = r, weights = weight_mat)
  res2 <- val_borrow_cpp(design = design, n = 20, r = r, a = 1, b = 1)
  res3 <- beta_borrow_pp(design = design, n = 20, r = rev(r),
    weights = weight_mat)

  expect_equal(unname(res1), unname(res2))
  expect_equal(res1, res3[, 4:1])
})

test_that("get_weights_cpp works", {
  wmat <- get_weights_cpp(n = 5, tune_a = 1, tune_b = 1)
  w1 <- 1 / (1 + exp(1 + 1 * log(5^(1/4) * abs(0/5 - 1/5))))
  w2 <- 1 / (1 + exp(1 + 1 * log(5^(1/4) * abs(0/5 - 2/5))))

  expect_equal(wmat[1, 2], w1)
  expect_equal(wmat[1, 3], w2)
  expect_true(isSymmetric(wmat))
  expect_equal(dim(wmat), c(6, 6))
})

test_that("get_weights_mml works", {
  design <- setup_mml(k = 4, p0 = 0.2, shape1 = 1, shape2 = 1)
  wmat <- get_weights_mml(design = design, n = 20)

  expect_true(isSymmetric(wmat))
  expect_true(all(wmat >= 0) & all(wmat <= 1))
  expect_equal(dim(wmat), c(21, 21))
})

test_that("get_weights_jsd works", {
  design <- setup_jsdglobal(k = 4, p0 = 0.15, shape1 = 1, shape2 = 1)
  wmat <- get_weights_jsd(design = design, n = 20, epsilon = 1.5, tau = 0,
    logbase = 2)

  shape <- rbind(1 + c(12, 7), 1 + (20 - c(12, 7)))
  w1 <- jsd_global(shape, epsilon = 1.5)

  expect_true(isSymmetric(wmat))
  expect_true(all(wmat >= 0) & all(wmat <= 1))
  expect_equal(wmat[13, 8], w1)
  expect_equal(dim(wmat), c(21, 21))
})

#' Evaluate a Basket Trial
#'
#' @template design
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates and,
#' for some methods, the posterior probabilities that the estimated response rates are above
#' a specified threshold p0.
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_evaluation(design = design, n = 20, r = c(10, 15, 5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25),
#'    r = c(10, 15, 17), lambda = 0.95, epsilon = 2,
#'    tau = 0, iter = 100)
#'
get_evaluation <- function(design, ...) {
  UseMethod("get_evaluation", design)
}





#' Evaluate a BHM Basket Trial
#'
#' @template design_bhm
#' @template n
#' @template r
#' @template lambda
#' @template level
#' @template tau_bhm
#' @template n_mcmc
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates.
#' @export
#'
#' @examples
#' design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
#'
#' get_evaluation(design = design, n = c(20, 20, 20), r = c(10, 15, 5),
#'   lambda = 0.95, tau_scale = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
#'   lambda = 0.95, tau_scale = 1, iter = 100)
get_evaluation.bhm <- function(design, n, r, lambda, level = 0.95,
                            tau_scale, n_mcmc = 10000, ...) {

  # n must be passed in the correct form
  if((length(n) < design$k & length(n) != 1) | length(n) > design$k){
    stop("n must either have length 1 or k")
  }

  analyses <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = bhmbasket::createTrial(n_subjects = n, n_responders = r),
    evidence_levels = c(lambda, 1 - level),
    method_names = "berry",
    target_rates = rep(design$p_target, design$k),
    prior_parameters_list = bhmbasket::setPriorParametersBerry(
      mu_mean = design$mu_mean,
      mu_sd = design$mu_sd,
      tau_scale = tau_scale
    ),
    n_mcmc_iterations = n_mcmc
  ))

  est <- bhmbasket::getEstimates(analyses, point_estimator = "mean",
                                 alpha_level = (1 - level))$berry
  list(
    Estimates = unname(est$scenario_1[,"Mean"])
  )
}




#' Evaluate a Basket Trial with the EXNEX Design
#'
#' @template design_exnex
#' @template n
#' @template r
#' @template lambda
#' @template level
#' @template tau_exnex
#' @template w_exnex
#' @template n_mcmc
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates.
#' @export
#'
#' @examples
#' design <- setup_exnex(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_evaluation(design = design, n = c(20, 20, 20), r = c(10, 15, 5),
#'   lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
#'   lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
get_evaluation.exnex <- function(design, n, r, lambda, level = 0.95,
                              tau_scale, w_j, n_mcmc = 10000, ...) {

  # n must be passed in the correct form
  if((length(n) < design$k & length(n) != 1) | length(n) > design$k){
    stop("n must either have length 1 or k")
  }

  analyses <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = bhmbasket::createTrial(n_subjects = n, n_responders = r),
    evidence_levels = c(lambda, 1 - level),
    method_names = "exnex",
    prior_parameters_list = bhmbasket::setPriorParametersExNex(
      mu_mean = design$mu_mean,
      mu_sd = design$mu_sd,
      tau_scale = tau_scale,
      mu_j = rep(design$basket_mean, design$k),
      tau_j = rep(design$basket_sd, design$k),
      w_j = w_j
    ),
    n_mcmc_iterations = n_mcmc
  ))

  est <- bhmbasket::getEstimates(analyses, point_estimator = "mean",
                                 alpha_level = (1 - level))$exnex

  list(
    Estimates = unname(est$scenario_1[,"Mean"])
  )
}




#' Evaluate a Basket Trial with Fujikawa's Design
#'
#' @template design_fujikawa
#' @template n
#' @template r
#' @template lambda
#' @template level
#' @template tuning_fujikawa
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates and the
#' posterior probabilities that the estimated response rates are above a specified threshold p0.
#' @export
#'
#' @examples
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_evaluation(design = design, n = 20, r = c(10, 15, 5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25),
#'    r = c(10, 15, 17), lambda = 0.95, epsilon = 2,
#'    tau = 0, iter = 100)
get_evaluation.fujikawa <- function(design, n, r, lambda, level = 0.95,
                                 epsilon, tau, logbase = 2, ...) {
  # n must be passed in the correct form
  if((length(n) < design$k & length(n) != 1) | length(n) > design$k){
    stop("n must either have length 1 or k")
  }
  weights <- get_weights_jsd(design = design, n = n, epsilon = epsilon,
                             tau = tau, logbase = logbase)

  shape <- beta_borrow_fujikawa(design = design, n = n, r = r, weights = weights)
  posterior <- post_beta(shape, design$p0)
  estimates <- apply(shape, 2, function(x) x[1] / (x[1] + x[2]))


  list(
    Posterior_Probabilities = posterior,
    Estimates = estimates
  )
}




#' Evaluate a Basket Trial with the Calibrated Power Prior
#' Design
#'
#' @template design_cpp
#' @template n
#' @template r
#' @template lambda
#' @template level
#' @template tuning_cpp
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates and the
#' posterior probabilities that the estimated response rates are above a specified threshold p0.
#' @export
#'
#' @examples
#' design <- setup_cpp(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_evaluation(design = design, n = 20, r = c(10, 15, 5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
get_evaluation.cpp <- function(design, n, r, lambda, level = 0.95,
                            tune_a, tune_b, ...) {
  # n must be passed in the correct form
  if((length(n) < design$k & length(n) != 1) | length(n) > design$k){
    stop("n must either have length 1 or k")
  }

  weights <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  shape <- beta_borrow_pp(design = design, n = n, r = r, weights = weights)
  posterior <- post_beta(shape, design$p0)
  estimates <- apply(shape, 2, function(x) x[1] / (x[1] + x[2]))


  list(
    Posterior_Probabilities = posterior,
    Estimates = estimates
  )
}




#' Evaluate a Basket Trial with the Limited Calibrated Power
#' Prior Design
#'
#' @template design_cpplim
#' @template n
#' @template r
#' @template lambda
#' @template level
#' @template tuning_cpp
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates and the
#' posterior probabilities that the estimated response rates are above a specified threshold p0.
#' @export
#'
#' @examples
#' design <- setup_cpplim(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_evaluation(design = design, n = 20, r = c(10, 15, 5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
get_evaluation.cpplim <- function(design, n, r, lambda, level = 0.95,
                               tune_a, tune_b, ...) {

  # n must be passed in the correct form
  if((length(n) < design$k & length(n) != 1) | length(n) > design$k){
    stop("n must either have length 1 or k")
  }

  weights <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  alpha_0 <- get_alpha_0_app(design = design, n = n)

  shape <- beta_borrow_cpplim(design = design, n = n, r = r,
                              weights = weights, alpha_0 = alpha_0)
  posterior <- post_beta(shape, design$p0)
  estimates <- apply(shape, 2, function(x) x[1] / (x[1] + x[2]))


  list(
    Posterior_Probabilities = posterior,
    Estimates = estimates
  )

}





#' Evaluate a Basket Trial with the Adaptive Power Prior Design
#' for sequential clinical trials
#'
#' @template design_app
#' @template n
#' @template r
#' @template lambda
#' @template level
#' @template dotdotdot
#'
#' @return A list containing the point estimates of the basket-specific response rates and the
#' posterior probabilities that the estimated response rates are above a specified threshold p0.
#' @export
#'
#' @examples
#' design <- setup_app(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_evaluation(design = design, n = 20, r = c(10, 15, 5),
#'  lambda = 0.95, iter = 100)
#'
#' # Unequal sample sizes
#' get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
#'  lambda = 0.95, iter = 100)
#'
get_evaluation.app <- function(design, n, r, lambda, level = 0.95, ...) {

  # n must be passed in the correct form
  if((length(n) < design$k & length(n) != 1) | length(n) > design$k){
    stop("n must either have length 1 or k")
  }

  alpha_0 <- get_alpha_0_app(design = design, n = n)

  shape <- beta_borrow_app(design = design, n = n, r = r, alpha_0 = alpha_0)
  posterior <- post_beta(shape, design$p0)
  estimates <- apply(shape, 2, function(x) x[1] / (x[1] + x[2]))


  list(
    Posterior_Probabilities = posterior,
    Estimates = estimates
  )

}

#' Calculate the Expected Number of Correct Decisions for a Basket Trial Design
#'
#' @template design
#' @template n
#' @template p1
#' @template lambda
#' @template design_params
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A numeric value.
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' ecd(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, design_params = list(epsilon = 2, tau = 0),
#'   iter = 1000)
#'
#' # Unequal sample sizes
#' ecd(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, design_params = list(epsilon = 2, tau = 0),
#'   iter = 1000)
ecd <- function(design, n, p1, lambda, design_params = list(), iter = 1000,
                data = NULL, ...) {
  res <- do.call(get_results, args = c(design = list(design), n = list(n),
    p1 = list(p1), lambda = lambda, design_params, iter = iter,
    data = list(data), ...))
  targ <- design$p0 != p1
  mean(rowSums(t(apply(res, 1, function(x) x == targ))))
}

#' Calculate the Type 1 Error Rate for a Basket Trial Design
#'
#' @template design
#' @template n
#' @template p1_toer
#' @template lambda
#' @template design_params
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A numeric value.
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' toer(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, design_params = list(epsilon = 2, tau = 0),
#'   iter = 1000)
#'
#' # Unequal sample sizes
#' toer(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, design_params = list(epsilon = 2, tau = 0),
#'   iter = 1000)
toer <- function(design, n, p1 = NULL, lambda, design_params = list(),
                 iter = 1000, data = NULL, ...) {
  if (is.null(p1)) p1 <- rep(design$p0, design$k)
  res <- do.call(get_results, args = c(design = list(design), n = list(n),
                 p1 = list(p1), lambda = lambda, design_params,
                 iter = iter, data = list(data), ...))
  res_sel <- res[, p1 == design$p0, drop = FALSE]
  res_all <- apply(res_sel, 1, function(x) any(x == 1))
  mean(res_all)
}

test_that("toer works", {
  design_sim <- setup_fujikawa(k = 3, p0 = 0.2)
  set.seed(20230319)
  res1 <- toer(design = design_sim, n = 15, lambda = 0.99, design_params =
      list(epsilon = 2, tau = 0, logbase = exp(1)), iter = 5000)
  set.seed(20230319)
  res2 <- get_details(design = design_sim, n = 15, lambda = 0.99,
    epsilon = 2, tau = 0, logbase = exp(1), iter = 5000)

  # Compare results from toer and get_details
  expect_equal(res1, res2$FWER)

  # Compare with results from baskexact
  expect_true(abs(0.03845609 - res1) < 0.01)
})

test_that("ecd works", {
  set.seed(20230512)
  design <- setup_cpp(k = 3, p0 = 0.2)
  res1 <- ecd(design = design, n = 15, p1 = c(0.2, 0.2, 0.5), lambda = 0.95,
    design_params = list(tune_a = 1, tune_b = 1), iter = 5000)
  set.seed(20230512)
  res2 <- get_details(design = design, n = 15, p1 = c(0.2, 0.2, 0.5), lambda = 0.95,
    tune_a = 1, tune_b = 1, iter = 5000)

  # Compare results from ecd and get_details
  expect_equal(res1, res2$ECD)

  # Compare with results from baskexact
  expect_true(abs(2.338286 - res1) < 0.01)
})

# Quantity of information parameter
get_alpha_0_app <- function(design, n){

  k <- design$k
  if(length(n) == 1) n <- c(rep(n,k))
  alpha_0 <- matrix(nrow = design$k, ncol = design$k)

  # rows define the current basket (columns the compared ones)
  for(i in 1:k){
    n_cur <- n[i]

    for(j in 1:k){
      n_comp <- n[j]
      alpha_0[i,j] <- ifelse(n_cur/n_comp < 1, n_cur/n_comp, 1)
    }
  }
  alpha_0
}


# Commensurability parameter
get_gamma <- function(n_gamma = n_gamma, r_gamma = r_gamma){

  n_cur <- n_gamma[1]
  n_comp <- n_gamma[2]

  l_cur <- function(x) stats::dbinom(r_gamma[1], n_gamma[1], x)
  l_comp <- function(x) stats::dbinom(r_gamma[2], n_gamma[2], x)

  l_min_cur <- function(x) l_cur(x)^min(1,(n_comp/n_cur))
  l_min_comp <- function(x) l_comp(x)^min(1,(n_cur/n_comp))

  r_cur <- function(x) sqrt(l_min_cur(x)/ stats::integrate(l_min_cur, 0, 1)$value)
  r_comp <- function(x) sqrt(l_min_comp(x)/ stats::integrate(l_min_comp, 0, 1)$value)

  diff_sq <- function(x) (r_cur(x) - r_comp(x))^2

  d_squared <- 0.5*stats::integrate(diff_sq, 0, 1)$value

  sqrt(d_squared)

}




# Beta borrowing for APP Design
beta_borrow_app <- function(design = design, n = n, r = data[i, ],
                alpha_0 = alpha_0){

  k <- design$k
  if(length(n) == 1) n <- c(rep(n,k))

  shape_noprior <- matrix(c(r, n - r), nrow = 2, byrow = TRUE)

  gamma <- matrix(nrow = design$k, ncol = design$k)

  for(i in 1:k){
    for(j in 1:k){
      gamma[i,j] <- get_gamma(n_gamma = c(n[i], n[j]), r_gamma = c(r[i], r[j]))
    }
  }

  alpha <- alpha_0*(1-gamma)

  # Compute posterior shapes
  shape1post <- apply(alpha, 1, function(x) sum(shape_noprior[1, ] * x)) +
    design$shape1
  shape2post <- apply(alpha, 1, function(x) sum(shape_noprior[2, ] * x)) +
    design$shape2
  rbind(shape1post, shape2post)

}




# Analyzing Results for CPP Design
ana_app <- function(design, n, r, lambda, alpha_0) {
  shape_post <- beta_borrow_app(design = design, n = n, r = r,
                                alpha_0 = alpha_0)
  post_prob <- post_beta(shape = shape_post, p0 = design$p0)
  ifelse(post_prob >= lambda, 1, 0)
}

test_that("get_alpha_0_app works", {
  set.seed(20230319)
  design <- setup_app(k = 3, p0 = 0.2)

  expect_equal(get_alpha_0_app(design = design, n = c(5,10,15)),
               matrix(data = c(1,5/10, 5/15, 1, 1, 10/15, 1, 1, 1),
                      ncol = 3, byrow = TRUE))

  expect_equal(get_alpha_0_app(design = design, n = 10),
               matrix(data = c(rep(1,9)), ncol = 3, byrow = TRUE))


})

test_that("get_gamma works", {

  validate_gamma <- function(n, r) {
    l1 <- function(x) stats::dbinom(x = r[1], size = n[1], prob = x)^
      min(1, n[2] / n[1])
    l2 <- function(x) stats::dbinom(x = r[2], size = n[2], prob = x)^
      min(1, n[1] / n[2])
    l1_int <- integrate(l1, 0, 1)$value
    l2_int <- integrate(l2, 0, 1)$value
    l_all <- function(x) (sqrt(l1(x) / l1_int) - sqrt(l2(x) / l2_int))^2
    l_all_int <- integrate(l_all, 0, 1)$value
    sqrt(1 / 2 * l_all_int)
  }


  expect_equal(get_gamma(n_gamma = c(10,15), r_gamma = c(8,10)),
               validate_gamma(n = c(10,15), r = c(8,10)))


})


test_that("beta_borrow_app works", {
  # compare with CPP (w = 1, alpha_0 = 1, gamma = 0)
  design_app <- setup_app(k = 3, p0 = 0.2)
  design_cpp <- setup_cpp(k = 3, p0 = 0.2)

  n <- 15
  r <- rep(10,3)

  weights <- get_weights_cpp(n = n, tune_a = 1, tune_b = 1)
  alpha_0 <- get_alpha_0_app(design = design_app, n = n)

  expect_equal(beta_borrow_app(design = design_app, n = n, r = r, alpha_0 = alpha_0),
               beta_borrow_pp(design = design_cpp, n = n, r = r, weights = weights))


})

test_that("ana_app works", {
  # compare with CPP (w = 1, alpha_0 = 1, gamma = 0)
  design_app <- setup_app(k = 3, p0 = 0.2)
  design_cpp <- setup_cpp(k = 3, p0 = 0.2)

  n <- 15
  r <- rep(10,3)
  lambda <- 0.987

  weights <- get_weights_cpp(n = n, tune_a = 1, tune_b = 1)
  alpha_0 <- get_alpha_0_app(design = design_app, n = n)

  expect_equal(ana_app(design = design_app, n = n, r = r, lambda = lambda,
                       alpha_0 = alpha_0),
               ana_pp(design = design_cpp, n = n, r = r, lambda = lambda,
                       weights = weights))


})

# Beta Posterior Distribution
beta_post <- function(design, n, r) {
  shape <- matrix(c(design$shape1 + r, design$shape2 + n - r), nrow = 2,
                  byrow = TRUE)
  rownames(shape) <- c("shape1post", "shape2post")
  return(shape)
}

# weight matrix for jsd-weights
get_weight_mat_jsd <- function(design, n, r, weights){
  k <- design$k
  weight_mat <- matrix(data = NA, nrow = k, ncol = k)

  if(length(unique(n)) == 1 || length(n) == 1){

    # rows define the current basket (columns the compared ones)
    for(i in 1:k){
      for(j in 1:k){
        if(i == j){
          weight_mat[i,j] <- 1
          next
        }else{
          weight_mat[i,j] <- weights[r[i]+1, r[j]+1]
        }
      }
    }
  } else{
    # find all possible combinations
    unique_combs <- unique(t(apply(arrangements::combinations(k = 2, v = n),1,sort)))

    # rows define the current basket (columns the compared ones)
    for(i in 1:k){
      n_cur <- n[i]
      for(j in 1:k){
        if(i == j){
          weight_mat[i,j] <- 1
          next
        } else {
          n_comp <- n[j]

          # The same basket is used for both combinations
          index <- which((unique_combs[,1] == n_cur & unique_combs[,2] == n_comp) |
                           (unique_combs[,2] == n_cur & unique_combs[,1] == n_comp))
          matching_weights <- weights[[index]]

          # Ensure that the rows indicate the current basket
          if(nrow(matching_weights) != n_cur +1){
            matching_weights <- t(matching_weights)
          }
        }

        weight_mat[i,j] <- matching_weights[r[i]+1, r[j]+1]

      }
    }
  }
  weight_mat
}


# Beta Borrowing for Fujikawa's Design
beta_borrow_fujikawa <- function(design, n, r, weights) {

  shape <- matrix(c(design$shape1 + r, design$shape2 + n - r), nrow = 2,
                  byrow = TRUE)
  weight_mat <- get_weight_mat_jsd(design, n, r, weights)

  shape1post <- apply(weight_mat, 1, function(x) sum(shape[1, ] * x))
  shape2post <- apply(weight_mat, 1, function(x) sum(shape[2, ] * x))
  rbind(shape1post, shape2post)

}


# Analyzing Results for Fujikawa's Design
ana_fujikawa <- function(design, n, r, lambda, weights) {
  shape_post <- beta_borrow_fujikawa(design = design, n = n, r = r,
                                     weights = weights)
  post_prob <- post_beta(shape = shape_post, p0 = design$p0)
  ifelse(post_prob >= lambda, 1, 0)
}

test_that("beta_borrow_fujikawa works", {
  design <- setup_fujikawa(k = 4, p0 = 0.2)
  r <- c(9, 14, 1, 5)

  weight_mat <- get_weights_jsd(design, n = 20, epsilon = 1, tau = 0,
    logbase = 2)
  res1 <- beta_borrow_fujikawa(design = design, n = 20, r = r,
    weights = weight_mat)
  res2 <- val_borrow_fujikawa(design = design, n = 20, r = r, epsilon = 1,
    tau = 0, logbase = 2)
  res3 <- beta_borrow_fujikawa(design = design, n = 20, r = rev(r),
    weights = weight_mat)

  expect_equal(unname(res1), unname(res2))
  expect_equal(res1, res3[, 4:1])
})

test_that("get_weight_mat_jsd works", {
  design <- setup_fujikawa(k = 4, p0 = 0.2)

  n_int <- 5
  n_vec <- c(5,5,6,7)
  r <- c(4,5,5,3)

  weights_int <- get_weights_jsd(design = design, n = n_int, epsilon = 2,
                                 tau = 0, logbase = 2)
  weights_vec <- get_weights_jsd(design = design, n = n_vec, epsilon = 2,
                                 tau = 0, logbase = 2)

  expect_equal(get_weight_mat_jsd(design = design, n = n_int, r = r, weights = weights_int)[1:2,1:2],
               get_weight_mat_jsd(design = design, n = n_vec, r = r, weights = weights_vec)[1:2,1:2])

})

#' Set Up mml Design Object
#'
#' Creates an object of class \code{mml}.
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{mml} implements a modified version of the
#' empirical Bayes method by Gravestock & Held (2017) which was proposed for
#' borrowing strength from an external study. In their approach, the sharing
#' weight is found as the maximum of the marginal likelihood of the
#' weight, given the external data set. This leads, however, to
#' non-symmetric weights when applied to sharing in basket trials, i.e.
#' Basket i would not share the information from Basket j as the other way
#' round. Therefore, a symmetrised version is used, where the mean of the
#' two weights resulting from sharing in both directions is used.
#'
#' @references Gravestock, I., & Held, L. (2017). Adaptive power priors with
#' empirical Bayes for clinical trials. Pharmaceutical statistics, 16(5),
#' 349-360.
#'
#' @return An S3 object of class \code{mml}
#' @export
#'
#' @examples
#' design_mml <- setup_mml(k = 3, p0 = 0.2)
setup_mml <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "mml"
  ))
}

#' Set Up mmlglobal Design Object
#'
#' Creates an object of class \code{mmlglobal}.
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{mmlglobal} implements an empirical Bayes method
#' by Gravestock & Held (2019) which was proposed for borrowing strength
#' from multiple external studies.
#'
#' @references Gravestock, I., & Held, L. (2019). Power priors based on
#' multiple historical studies for binary outcomes. Biometrical Journal, 61(5),
#' 1201-1218.
#'
#' Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
#' Design Based on Power Priors. Statistics in Biopharmaceutical Research,
#' 17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
#'
#' @return An S3 object of class \code{mmlglobal}
#' @export
#'
#' @examples
#' design_mmlglobal <- setup_mmlglobal(k = 3, p0 = 0.2)
setup_mmlglobal <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "mmlglobal"
  ))
}


#' Set Up BHM Design Object
#'
#' @template k
#' @template p0
#' @template p_target
#' @template mu_bhm
#'
#' @details The class \code{bhm} implements the Bayesian Hierarchical Model
#' proposed by Berry et al. (2013). Methods for this class are
#' mostly wrappers for functions from the package \code{bhmbasket}.
#'
#' In the BHM the thetas of all baskets are modeled, where theta_i =
#' logit(p_i) - logit(p_target). These thetas are assumed to come from
#' a normal distribution with mean mu_mean and standard deviation mu_sd.
#' If \code{mu_mean = NULL} then mu_mean is determined as logit(p0) -
#' logit(p_target), hence the mean of the normal distribution corresponds
#' to the null hypothesis.
#'
#' @references Berry, S. M., Broglio, K. R., Groshen, S., & Berry, D. A. (2013).
#' Bayesian hierarchical modeling of patient subpopulations: efficient designs
#' of phase II oncology clinical trials. Clinical Trials, 10(5), 720-734.
#'
#' @return An S3 object of class \code{bhm}
#' @export
#'
#' @examples
#' design_bhm <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
setup_bhm <- function(k, p0, p_target, mu_mean = NULL, mu_sd = 100) {
  if (is.null(mu_mean)) {
    mu_mean <- bhmbasket::logit(p0) -
      bhmbasket::logit(p_target)
  }
  structure(
    list(k = k, p0 = p0, p_target = p_target, mu_mean = mu_mean, mu_sd = mu_sd),
    class = "bhm"
  )
}


#' Set Up EXNEX Design Object
#'
#' @template k
#' @template p0
#' @template params_exnex
#'
#' @details The class \code{exnex} implements the EXNEX model proposed by
#' Neuenschwander et al. (2016). Methods for this class are mostly wrappers
#' for functions from the package \code{bhmbasket}.
#'
#' In the EXNEX model the thetas of all baskets are modeled as a mixture
#' of individual models and a Bayesian Hierarchical Model with a fixed
#' mixture weight w. If \code{mu_mean} and \code{basket_mean} are \code{NULL}
#' then they are set to logit(p0).
#' Note that Neuenschwander et al. (2016) use different prior means and
#' standard deviations. The default values here are used for better comparison
#' with the BHM model (see \code{\link{setup_bhm}}).
#'
#' @references Neuenschwander, B., Wandel, S., Roychoudhury, S., & Bailey, S.
#' (2016). Robust exchangeability designs for early phase clinical trials with
#' multiple strata. Pharmaceutical statistics, 15(2), 123-134.
#'
#' @return An S3 object of class \code{exnex}
#' @export
#'
#' @examples
#' design_exnex <- setup_exnex(k = 3, p0 = 0.2)
setup_exnex <- function(
  k,
  p0,
  basket_mean = NULL,
  basket_sd = 100,
  mu_mean = NULL,
  mu_sd = 100
) {
  if (is.null(basket_mean)) {
    basket_mean <- bhmbasket::logit(p0)
  }
  if (is.null(mu_mean)) {
    mu_mean <- bhmbasket::logit(p0)
  }
  structure(
    list(
      k = k,
      p0 = p0,
      basket_mean = basket_mean,
      basket_sd = basket_sd,
      mu_mean = mu_mean,
      mu_sd = mu_sd
    ),
    class = "exnex"
  )
}


#' Set Up Fujikawa Design Object
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{fujikawa} implements a design by Fujikawa et al.
#' (2020) in which information is shared based on the pairwise similarity
#' between baskets which is quantified using the Jensen-Shannon divergence
#' between the individual posterior distributions between baskets.
#'
#' @references Fujikawa, K., Teramukai, S., Yokota, I., & Daimon, T. (2020).
#' A Bayesian basket trial design that borrows information across strata based
#' on the similarity between the posterior distributions of the response
#' probability. Biometrical Journal, 62(2), 330-338.
#'
#' @return An S3 object of class \code{fujikawa}
#' @export
#'
#' @examples
#' design_fujikawa <- setup_fujikawa(k = 3, p0 = 0.2)
setup_fujikawa <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "fujikawa"
  ))
}

#' Set Up Global JSD Design Object
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{jsdglobal} implements a version of the power prior
#' design, in which information is shared based on pairwise similarity
#' and overall heterogeneity between baskets. Both pairwise similarity and
#' overall heterogeneity are assessed using the Jensen-Shannon divergence.
#'
#' @references Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
#' Design Based on Power Priors. Statistics in Biopharmaceutical Research,
#' 17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
#'
#' @return An S3 object of class \code{jsdglobal}
#' @export
#'
#' @examples
#' design_jsdglobal <- setup_jsdglobal(k = 3, p0 = 0.2)
setup_jsdglobal <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "jsdglobal"
  ))
}

#' Set Up Calibrated Power Prior Design Object
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{cpp} implements a version of the power prior design,
#' in which the amount of information that is shared between baskets is
#' determined by the Kolmogorov-Smirnov test statistic between baskets (which
#' is equivalent to the absolut difference in response rates).
#'
#' @references Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
#' Design Based on Power Priors. Statistics in Biopharmaceutical Research,
#' 17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
#'
#' @return An S3 object of class \code{cpp}
#' @export
#'
#' @examples
#' design_cpp <- setup_cpp(k = 3, p0 = 0.2)
setup_cpp <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "cpp"
  ))
}

#' Set Up Global Calibrated Power Prior Design Object
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{cppglobal} implements a version of the power prior
#' design, in which the amount of information that is shared between baskets is
#' determined by the Kolmogorov-Smirnov test statistic between basekts and
#' a function based on response rate differences that quantifies the
#' overall heterogeneity.
#'
#' @references Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
#' Design Based on Power Priors. Statistics in Biopharmaceutical Research,
#' 17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
#'
#' @return An S3 object of class \code{cppglobal}
#' @export
#'
#' @examples
#' design_cppglobal <- setup_cppglobal(k = 3, p0 = 0.2)
setup_cppglobal <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "cppglobal"
  ))
}

#' Set Up Limited Calibrated Power Prior Design Object
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details
#' The class \code{cpplim} implements a combined version of the adaptive power
#' prior (\code{app}) and the calibrated power prior (\code{cpp}), where the
#' parameter limiting the amount of information to be borrowed in the adaptive
#' power prior design is included in the calibrated power prior design.
#'
#'
#' @return An S3 object of class \code{cpplim}
#' @export
#'
#' @references Ollier, A., Morita, S., Ursino, M., & Zohar, S. (2020). An
#' adaptive power prior for sequential clinical trials - Application to bridging
#' studies. Statistical methods in medical research, 29(8), 2282–2294.
#'
#' Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
#' Design Based on Power Priors. Statistics in Biopharmaceutical Research,
#' 17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
#'
#' @examples
#' design_cpplim <- setup_cpplim(k = 3, p0 = 0.2)
setup_cpplim <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "cpplim"
  ))
}


#' Set Up Adaptive Power Prior Design Object
#'
#' @template k
#' @template p0
#' @template shape_beta
#'
#' @details The class \code{app} implements the adaptive power prior design for
#' sequential clinical trials proposed by Ollier et al. (2020).
#'
#' @return An S3 object of class \code{app}
#' @export
#'
#' @references Ollier, A., Morita, S., Ursino, M., & Zohar, S. (2020). An
#' adaptive power prior for sequential clinical trials - Application to bridging
#' studies. Statistical methods in medical research, 29(8), 2282–2294.
#'
#' @examples
#' design_app <- setup_app(k = 3, p0 = 0.2)
setup_app <- function(k, p0, shape1 = 1, shape2 = 1) {
  validate_betabin(structure(
    list(k = k, p0 = p0, shape1 = shape1, shape2 = shape2),
    class = "app"
  ))
}


#' Set Up Frequentist Binomial Design Object
#'
#' @template k
#' @template p0
#' @template pool
#'
#' @details The class \code{binomial} implements a basket trial
#' design, in which each null hypothesis is tested using the
#' frequentist binomial test without multiplicity correction. All baskets are
#' either tested separately (the default) or pooled (not implemented yet).
#'
#' @return An S3 object of class \code{binomial}
#' @export
#'
#' @examples
#' design_binomial <- setup_binomial(k = 3, p0 = 0.2)
setup_binomial <- function(k, p0, pool = FALSE) {
  if (!is.logical(pool)) {
    stop("pool must be boolean.")
  }
  validate_basics(structure(
    list(k = k, p0 = p0, pool = pool),
    class = "binomial"
  ))
}

test_that("get_results works for mmlglobal", {
  design <- setup_mmlglobal(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
    lambda = 0.95, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, data = data,
    iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))
})

test_that("get_results works for bhm", {
  design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
  data <- bhmbasket::simulateScenarios(
    n_subjects_list = list(rep(10, 3)),
    response_rates_list = list(c(0.2, 0.5, 0.5)),
    n_trials = 15
  )

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
    lambda = 0.95, tau_scale = 1, iter = 10)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tau_scale = 1,
    data = data, iter = 15)

  expect_equal(dim(res1), c(10, 3))
  expect_equal(dim(res2), c(15, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))

  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 10)
  expect_error(get_results(design = design, n = 10, lambda = 0.95,
    tau_scale = 1, data = data))

  expect_error(get_results.bhm(design = design, n = c(10,20), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
  expect_error(get_results.bhm(design = design, n = c(10,20,30,40), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
})

test_that("get_results works for exnex", {
  design <- setup_exnex(k = 3, p0 = 0.2)
  data <- bhmbasket::simulateScenarios(
    n_subjects_list = list(rep(10, 3)),
    response_rates_list = list(c(0.2, 0.5, 0.5)),
    n_trials = 15
  )

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
    lambda = 0.95, tau_scale = 1, w = 0.5, iter = 10)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tau_scale = 1,
    w = 0.5, data = data, iter = 15)

  expect_equal(dim(res1), c(10, 3))
  expect_equal(dim(res2), c(15, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))

  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 10)
  expect_error(get_results(design = design, n = 10, lambda = 0.95,
    tau_scale = 1, w = 0.5, data = data))

  expect_error(get_results.exnex(design = design, n = c(10,20), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
  expect_error(get_results.exnex(design = design, n = c(10,20,30,40), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
})

test_that("get_results works for fujikawa", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
    lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, epsilon = 2,
    tau = 0, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))

  expect_error(get_results.fujikawa(design = design, n = c(10,20), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
  expect_error(get_results.fujikawa(design = design, n = c(10,20,30,40), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
})

test_that("get_results works for jsdglobal", {
  design <- setup_jsdglobal(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
    lambda = 0.95, eps_pair = 2, eps_all = 2, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, eps_pair = 2,
    eps_all = 2, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))
})

test_that("get_results works for cpp", {
  design <- setup_cpp(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
    lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tune_a = 1,
    tune_b = 1, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))

  expect_error(get_results.cpp(design = design, n = c(10,20), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
  expect_error(get_results.cpp(design = design, n = c(10,20,30,40), p1 = NULL,
                               lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
})

test_that("get_results works for cppglobal", {
  design <- setup_cppglobal(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
                      lambda = 0.95, tune_a = 1, tune_b = 1, epsilon = 2, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tune_a = 1,
                      tune_b = 1, epsilon = 2, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))
})

test_that("get_results works for mml", {
  design <- setup_mml(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
                      lambda = 0.95, tune_a = 1, tune_b = 1, epsilon = 2, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tune_a = 1,
                      tune_b = 1, epsilon = 2, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))
})

test_that("get_results works for cpplim", {
  design <- setup_cpplim(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
                      lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tune_a = 1,
                      tune_b = 1, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))

  expect_error(get_results.cpplim(design = design, n = c(10,20), p1 = NULL,
                                  lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
  expect_error(get_results.cpplim(design = design, n = c(10,20,30,40), p1 = NULL,
                                  lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
})

test_that("get_results works for app", {
  design <- setup_app(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 10, p = c(0.2, 0.5, 0.5), iter = 110)

  res1 <- get_results(design = design, n = 10, p1 = c(0.2, 0.5, 0.5),
                      lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
  res2 <- get_results(design = design, n = 10, lambda = 0.95, tune_a = 1,
                      tune_b = 1, data = data, iter = 110)

  expect_equal(dim(res1), c(100, 3))
  expect_equal(dim(res2), c(110, 3))
  expect_true(all(res1 %in% c(0, 1)))
  expect_true(all(res2 %in% c(0, 1)))

  expect_error(get_results.app(design = design, n = c(10,20), p1 = NULL,
                                  lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
  expect_error(get_results.app(design = design, n = c(10,20,30,40), p1 = NULL,
                                  lambda = 0.95, pmp0 = 1, data = NULL, iter = 110))
})

test_that("setup_fujikawa works", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)
  expect_s3_class(design, "fujikawa")
})

test_that("setup_app works", {
  design <- setup_app(k = 3, p0 = 0.2)
  expect_s3_class(design, "app")
})

test_that("setup_cpp works", {
  design <- setup_cpp(k = 3, p0 = 0.2)
  expect_s3_class(design, "cpp")
})

test_that("setup_cpplim works", {
  design <- setup_cpplim(k = 3, p0 = 0.2)
  expect_s3_class(design, "cpplim")
})

test_that("errors in setup functions work", {
  expect_error(setup_fujikawa(k = 0, p0 = 0.2))
  expect_error(setup_cpp(k = 2.5, p0 = 0.5))
  expect_error(setup_cpp(k = 2, p0 = 0))
  expect_error(setup_cpp(k = 2, p0 = 1))
  expect_error(setup_cpp(k = 2, p0 = 2))
  expect_error(setup_cpp(k = 3.5, p0 = 0.5))
  expect_error(setup_cpplim(k = 4, p0 = 0.2, shape1 = 0, shape2 = 0))
})

test_that("jsd_global works", {
  shape1 <- matrix(c(1, 1, 1, 10, 10, 10), ncol = 3, byrow = TRUE)
  jsdglobal1 <- jsd_global(shape1, 1)

  # (1 - JSD) is equal to 1 when all distributions are equal
  expect_equal(jsdglobal1, 1)

  shape2 <- matrix(c(1 + 4, 1 + 6, 1 + 2, 1 + 8), ncol = 2)
  jsdglobal2 <- jsd_global(shape2, 2)

  # Compare with results from baskexact
  expect_equal(jsdglobal2, 0.560588865)

  shape3 <- matrix(c(1, 100, 100, 1), ncol = 2, byrow = TRUE)
  jsdglobal3 <- jsd_global(shape3, 1)

  # (1 - JSD) is 0 when distributions are very different
  expect_equal(jsdglobal3, 0)

  # Compare with validator function jsd_global4
  r <- c(5, 1, 10, 7)
  shape4 <- rbind(1 + r, 1 + (20 - r))

  jsdglobal4 <- jsd_global(shape4, 2)
  jsdglobal5 <- jsd_global4(shape4, 2)

  expect_equal(jsdglobal4, jsdglobal5)

  # Compare with weights from get_weights_jsd
  design <- setup_jsdglobal(k = 4, p0 = 0.15)
  jsdmat <- get_weights_jsd(design = design, n = 20, epsilon = 1.5, tau = 0,
    logbase = 2)
  r2 <- c(5, 7)
  shape5 <- rbind(1 + r2, 1 + 20 - r2)
  jsdglobal6 <- jsd_global(shape5, 1.5)
  jsdglobal7 <- jsdmat[1 + 5, 1 + 7]

  expect_equal(jsdglobal6, jsdglobal7)
})

test_that("beta_borrow_jsdglobal works", {
  design1 <- setup_fujikawa(k = 3, p0 = 0.2)

  weights1 <- get_weights_jsd(design1, n = 20, epsilon = 2, tau = 1,
    logbase = 2)
  res1 <- beta_borrow_jsdglobal(design1, n = 20, r = c(1, 3, 5),
    weights_pair = weights1, eps_all = 0)
  res2 <- beta_borrow_fujikawa(design1, n = 20, r = c(1, 3, 5),
    weights = weights1)

  # Results are identical for fujikawa and jsd global when no information
  # is shared
  expect_equal(res1, res2)

  weights2 <- get_weights_jsd(design1, n = 20, epsilon = 2, tau = 0,
    logbase = 2)
  res3 <- beta_borrow_jsdglobal(design1, n = 20, r = c(3, 3, 3),
    weights_pair = weights2, eps_all = 0)
  res4 <- beta_borrow_fujikawa(design1, n = 20, r = c(3, 3, 3),
    weights = weights2)

  # Results differ only by the amount of prior information that is not shared
  expect_true(all(res3 + 2 == res4))

  # Sharing works correctly with 4 baskets
  design2 <- setup_jsdglobal(k = 4, p0 = 0.15)

  r1 <- c(12, 7, 3, 18)
  weights3 <- get_weights_jsd(design2, n = 20, epsilon = 1.5, tau = 0,
    logbase = 2)
  res5 <- beta_borrow_jsdglobal(design2, n = 20, r = r1,
    weights_pair = weights3, eps_all = 1.2)
  res6 <- val_borrow_jsdglobal(design2, n = 20, r = r1, epsilon = 1.5, tau = 0,
    logbase = 2, eps_all = 1.2)
  res7 <- beta_borrow_jsdglobal(design2, n = 20, r = rev(r1),
    weights_pair = weights3, eps_all = 1.2)

  expect_equal(unname(res5), unname(res6))
  expect_equal(res5, res7[, 4:1])

  # Compare with results from baskexact
  res8 <- beta_borrow_jsdglobal(design2, n = 20, r = r1,
    weights_pair = weights3, eps_all = 0)
  res9 <- rbind(c(17.23864, 12.83330, 6.842622, 20.380510),
    c(13.55264, 22.74386, 22.979734, 3.947204))

  expect_equal(unname(res8), res9, tolerance = 1e-7)
  r2 <- c(9, 9, 5, 17)
  res10 <- beta_borrow_jsdglobal(design2, n = 20, r = r2,
    weights_pair = weights3, eps_all = 1.2)

  s11 <- 1 + r2[1] +
    (jsd_global(rbind(1 + r2[1:2], 1 + 20 - r2[1:2]), eps = 1.5) * r2[2] +
    jsd_global(rbind(1 + r2[c(1, 3)], 1 + 20 - r2[c(1, 3)]), eps = 1.5) * r2[3] +
    jsd_global(rbind(1 + r2[c(1, 4)], 1 + 20 - r2[c(1, 4)]), eps = 1.5) * r2[4]) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)
  s21 <- 1 + r2[2] +
    (jsd_global(rbind(1 + r2[1:2], 1 + 20 - r2[1:2]), eps = 1.5) * r2[1] +
    jsd_global(rbind(1 + r2[2:3], 1 + 20 - r2[2:3]), eps = 1.5) * r2[3] +
    jsd_global(rbind(1 + r2[c(2, 4)], 1 + 20 - r2[c(2, 4)]), eps = 1.5) * r2[4]) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)
  s31 <- 1 + r2[3] +
    (jsd_global(rbind(1 + r2[c(1, 3)], 1 + 20 - r2[c(1, 3)]), eps = 1.5) * r2[1] +
    jsd_global(rbind(1 + r2[2:3], 1 + 20 - r2[2:3]), eps = 1.5) * r2[2] +
    jsd_global(rbind(1 + r2[3:4], 1 + 20 - r2[3:4]), eps = 1.5) * r2[4]) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)
  s41 <- 1 + r2[4] +
    (jsd_global(rbind(1 + r2[c(1, 4)], 1 + 20 - r2[c(1, 4)]), eps = 1.5) * r2[1] +
    jsd_global(rbind(1 + r2[c(2, 4)], 1 + 20 - r2[c(2, 4)]), eps = 1.5) * r2[2] +
    jsd_global(rbind(1 + r2[3:4], 1 + 20 - r2[3:4]), eps = 1.5) * r2[3]) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)

  s12 <- 1 + 20 - r2[1] +
    (jsd_global(rbind(1 + r2[1:2], 1 + 20 - r2[1:2]), eps = 1.5) *
        (20 - r2[2]) +
    jsd_global(rbind(1 + r2[c(1, 3)], 1 + 20 - r2[c(1, 3)]), eps = 1.5) *
        (20 - r2[3]) +
    jsd_global(rbind(1 + r2[c(1, 4)], 1 + 20 - r2[c(1, 4)]), eps = 1.5) *
        (20 - r2[4])) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)
  s22 <- 1 + 20 - r2[2] +
    (jsd_global(rbind(1 + r2[1:2], 1 + 20 - r2[1:2]), eps = 1.5) *
        (20 - r2[1]) +
    jsd_global(rbind(1 + r2[2:3], 1 + 20 - r2[2:3]), eps = 1.5) *
        (20 - r2[3]) +
    jsd_global(rbind(1 + r2[c(2, 4)], 1 + 20 - r2[c(2, 4)]), eps = 1.5) *
        (20 - r2[4])) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)
  s32 <- 1 + 20 - r2[3] +
    (jsd_global(rbind(1 + r2[c(1, 3)], 1 + 20 - r2[c(1, 3)]), eps = 1.5) *
        (20 - r2[1]) +
    jsd_global(rbind(1 + r2[2:3], 1 + 20 - r2[2:3]), eps = 1.5) *
        (20 - r2[2]) +
    jsd_global(rbind(1 + r2[3:4], 1 + 20 - r2[3:4]), eps = 1.5) *
        (20 - r2[4])) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)
  s42 <- 1 + 20 - r2[4] +
    (jsd_global(rbind(1 + r2[c(1, 4)], 1 + 20 - r2[c(1, 4)]), eps = 1.5) *
        (20 - r2[1]) +
    jsd_global(rbind(1 + r2[c(2, 4)], 1 + 20 - r2[c(2, 4)]), eps = 1.5) *
        (20 - r2[2]) +
    jsd_global(rbind(1 + r2[3:4], 1 + 20 - r2[3:4]), eps = 1.5) *
        (20 - r2[3])) *
    jsd_global(rbind(1 + r2, 1 + 20 - r2), eps = 1.2)

  res11 <- rbind(c(s11, s21, s31, s41), c(s12, s22, s32, s42))

  expect_equal(unname(res10), res11)
})

n_iter <- 100 # was 1000 before
test_that("adjust_lambda default method works", {
  design <- setup_cpp(k = 3, p0 = 0.2)

  # Without simulated data
  set.seed(20230319)
  res1 <- adjust_lambda(design = design, n = 15, alpha = 0.05,
    design_params = list(tune_a = 1, tune_b = 1), iter = n_iter)
  toer1 <- toer(design, n = 15, lambda = res1$lambda, design_params =
      list(tune_a = 1, tune_b = 1), iter = n_iter)

  expect_lte(res1$toer, 0.05)
  expect_equal(res1$toer, toer1, tolerance = 0.5)

  skip_on_cran()
  set.seed(123)
  res2 <- adjust_lambda(design = design, n = 12, alpha = 0.05,
    design_params = list(tune_a = 2, tune_b = 2), iter = n_iter)
  toer2 <- toer(design, n = 12, lambda = res2$lambda, design_params =
      list(tune_a = 2, tune_b = 2), iter = n_iter)

  expect_lte(res2$toer, 0.05)
  expect_equal(res2$toer, toer2, tolerance = 0.5)

  set.seed(456)
  res3 <- adjust_lambda(design = design, n = 13, alpha = 0.05,
    design_params = list(tune_a = 4, tune_b = 1.5), iter = n_iter)
  toer3 <- toer(design, n = 13, lambda = res3$lambda, design_params =
      list(tune_a = 4, tune_b = 1.5), iter = n_iter)

  expect_lte(res3$toer, 0.05)
  expect_equal(res3$toer, toer3, tolerance = 0.5)

  # With simulated data
  simdata <- get_data(k = 3, n = 15, p = 0.2, iter = n_iter)
  res4 <- adjust_lambda(design = design, n = 15, p1 = 0.2, alpha = 0.05,
    design_params = list(tune_a = 1, tune_b = 1), iter = n_iter, prec_digits = 3,
    data = simdata)
  res5 <- adjust_lambda(design = design, n = 15, p1 = 0.2, alpha = 0.05,
    design_params = list(tune_a = 1, tune_b = 1), iter = n_iter, prec_digits = 3,
    data = simdata)

  # Check if results are equal when adjust_lambda is called two times with
  # the same simulated data
  expect_equal(res4$lambda, res5$lambda)
  expect_equal(res4$toer, res5$toer)

  # Check if selected lambda controls the TOER
  toer4 <- toer(design, n = 15, p1 = 0.2, lambda = res4$lambda,
    design_params = list(tune_a = 1, tune_b = 1), iter = n_iter, data = simdata)
  toer5 <- toer(design, n = 15, p1 = 0.2, lambda = res4$lambda - 0.01,
    design_params = list(tune_a = 1, tune_b = 1), iter = n_iter, data = simdata)

  expect_equal(res4$toer, toer4)
  expect_gt(toer5, 0.05)
})

test_that("adjust_lambda works for exnex", {
  design <- setup_exnex(k = 3, p0 = 0.2)

  # Without simulated data
  set.seed(125)
  res1 <- adjust_lambda(design = design, n = 15,
    design_params = list(tau_scale = 1, w_j = 0.5), iter = n_iter, n_mcmc = 2500)

  set.seed(125)
  res2 <- toer(design = design, n = 15, lambda = res1$lambda,
               design_params = list(tau_scale = 1, w_j = 0.5), iter = n_iter, n_mcmc = 2500)

  expect_lte(res1$toer, 0.05)
  expect_equal(res1$toer, res2)

  skip_on_cran()
  # With simulated data
  set.seed(125)
  simdata <- get_data(k = 3, n = 15, p = 0.2, iter = n_iter, type = "bhmbasket")
  set.seed(126)
  res3 <- adjust_lambda(design = design, n = 15,
    design_params = list(tau_scale = 1, w_j = 0.5), iter = n_iter, n_mcmc = 2500,
    data = simdata)
  set.seed(126)
  res4 <- adjust_lambda(design = design, n = 15,
    design_params = list(tau_scale = 1, w_j = 0.5), iter = n_iter, n_mcmc = 2500,
    data = simdata)

  # Check if results are equal when adjust_lambda is called two times with
  # the same simulated data
  expect_equal(res3$lambda, res4$lambda)
  expect_equal(res3$toer, res4$toer)

  # Check if selected lambda controls the TOER
  set.seed(126)
  toer1 <- toer(design, n = 15, p1 = 0.2, lambda = res3$lambda,
    design_params = list(tau_scale = 1, w_j = 0.5), data = simdata, iter = n_iter,
    n_mcmc = 2500)
  toer2 <- toer(design, n = 15, p1 = 0.2, lambda = res3$lambda - 0.01,
    design_params = list(tau_scale = 1, w_j = 0.5), data = simdata, iter = n_iter,
    n_mcmc = 2500)

  expect_equal(toer1, 0.05)
  expect_gt(toer2, 0.05)
})

test_that("adjust_lambda works for bhm", {
  design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)

  # Without simulated data
  set.seed(125)
  res1 <- adjust_lambda(design = design, n = 15,
    design_params = list(tau_scale = 1), iter = n_iter, n_mcmc = 2500)

  set.seed(125)
  res2 <- toer(design = design, n = 15, lambda = res1$lambda,
    design_params = list(tau_scale = 1), iter = n_iter, n_mcmc = 2500)

  expect_lte(res1$toer, 0.05)
  expect_equal(res1$toer, res2)

  skip_on_cran()
  # With simulated data
  set.seed(125)
  simdata <- get_data(k = 3, n = 15, p = 0.2, iter = n_iter, type = "bhmbasket")
  set.seed(126)
  res3 <- adjust_lambda(design = design, n = 15,
    design_params = list(tau_scale = 1), iter = n_iter, n_mcmc = 2500,
    data = simdata)
  set.seed(126)
  res4 <- adjust_lambda(design = design, n = 15,
    design_params = list(tau_scale = 1), iter = n_iter, n_mcmc = 2500,
    data = simdata)

  # Check if results are equal when adjust_lambda is called two times with
  # the same simulated data
  expect_equal(res3$lambda, res4$lambda)
  expect_equal(res3$toer, res4$toer)

  # Check if selected lambda controls the TOER
  set.seed(126)
  toer1 <- toer(design, n = 15, p1 = 0.2, lambda = res1$lambda,
    design_params = list(tau_scale = 1), data = simdata, iter = n_iter,
    n_mcmc = 2500)
  toer2 <- toer(design, n = 15, p1 = 0.2, lambda = res1$lambda - 0.01,
    design_params = list(tau_scale = 1), data = simdata, iter = n_iter,
    n_mcmc = 2500)

  expect_equal(toer1, 0.05, tolerance = 0.1)
  expect_gt(toer2, 0.05)
})

test_that("get_details works for mmlglobal", {
  design <- setup_mmlglobal(k = 3, p0 = 0.2)
  set.seed(20230222)

  # Posterior means are close to p
  p1 <- c(0.5, 0.5, 0.5)
  res3 <- get_details(
    design = design,
    n = 20,
    p1 = p1,
    lambda = 0.95,
    pmp0 = 1,
    iter = 100
  )
  expect_equal(res3$Mean, p1, tolerance = 0.01)

  skip_on_cran()
  res1 <- get_details(
    design = design,
    n = 20,
    p1 = c(0.2, 0.4, 0.4),
    lambda = 0.95,
    pmp0 = 1,
    iter = 100
  )

  res2 <- get_details(
    design = design,
    n = 20,
    p1 = c(0.3, 0.5, 0.5),
    lambda = 0.95,
    pmp0 = 1,
    iter = 100
  )

  # Rejection probabilities are higher when p is higher
  expect_true(all(res2$Rejection_Probabilities > res1$Rejection_Probabilities))
})

test_that("get_details works for bhm", {
  set.seed(1)
  design <- setup_bhm(
    k = 3,
    p0 = 0.15,
    p_target = 0.5,
    mu_mean = -1.7346,
    mu_sd = 100
  )

  # Works without supplied data
  res3 <- get_details(
    design = design,
    n = 10,
    p1 = c(0.15, 0.5, 0.5),
    lambda = 0.9,
    tau_scale = 0.75,
    iter = 100
  )
  expect_equal(length(res3), 7)
  expect_equal(res3$Rejection_Probabilities[1], res3$FWER)

  skip_on_cran()
  scenarios <- bhmbasket::simulateScenarios(
    n_subjects_list = list(rep(10, 3)),
    response_rates_list = list(c(0.15, 0.5, 0.5)),
    n_trials = 100
  )

  set.seed(20230515)
  res1 <- get_details(
    design = design,
    n = 10,
    p1 = c(0.15, 0.5, 0.5),
    lambda = 0.9,
    tau_scale = 0.75,
    iter = 100,
    data = scenarios
  )

  # Results with bhmbasket
  set.seed(20230515)
  ana <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = scenarios,
    evidence_levels = 0.9,
    method_names = "berry",
    target_rates = rep(0.5, 3),
    prior_parameter_list <- bhmbasket::setPriorParametersBerry(
      mu_mean = -1.7346,
      mu_sd = 100,
      tau_scale = 0.75
    )
  ))

  godec <- bhmbasket::getGoDecisions(
    analyses_list = ana,
    cohort_names = paste("p", 1:3, sep = "_"),
    evidence_levels = rep(0.9, 3),
    boundary_rules = quote(c(x[1] > 0.15, x[2] > 0.15, x[3] > 0.15))
  )

  res2 <- bhmbasket::getGoProbabilities(godec)

  estim <- bhmbasket::getEstimates(
    analyses_list = ana,
    point_estimator = "mean",
    alpha_level = 0.05
  )

  # Results are equal with get_details and 
  expect_equal(res1$Rejection_Probabilities, unname(unlist(res2))[-1])
  expect_equal(res1$Mean, unname(estim$berry$scenario_1[, 1]))
  expect_equal(res1$MSE, unname(estim$berry$scenario_1[, 7]))
  expect_equal(res1$Lower_CL, unname(estim$berry$scenario_1[, 3]))
  expect_equal(res1$Upper_CL, unname(estim$berry$scenario_1[, 5]))

  # Error works
  data <- get_data(k = 3, n = 20, p = 0.5, iter = 100)
  expect_error(get_details(
    design,
    n = 20,
    p1 = c(0.2, 0.2, 0.5),
    lambda = 0.95,
    tau_scale = 1,
    iter = 100,
    data = data
  ))

  # If n is passed as a vector, it should have k entries
  expect_error(get_details.bhm(
    design = design,
    n = c(10, 20),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.bhm(
    design = design,
    n = c(10, 20, 30, 40),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))

  # Works without supplied p1
  expect_no_error(get_details(
    design = design,
    n = 10,
    p1 = NULL,
    lambda = 0.9,
    tau_scale = 0.75,
    iter = 100
  ))
})

test_that("get_details works for exnex", {
  scenarios <- bhmbasket::simulateScenarios(
    n_subjects_list = list(rep(10, 3)),
    response_rates_list = list(c(0.15, 0.5, 0.5)),
    n_trials = 100
  )

  design <- setup_exnex(k = 3, p0 = 0.15)

  set.seed(20230515)
  res1 <- get_details(
    design = design,
    n = 10,
    p1 = c(0.15, 0.5, 0.5),
    lambda = 0.9,
    tau_scale = 0.75,
    w = 0.5,
    iter = 100,
    data = scenarios
  )

  # Results with bhmbasket
  set.seed(20230515)
  ana <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = scenarios,
    evidence_levels = 0.9,
    method_names = "exnex",
    target_rates = NULL,
    prior_parameter_list <- bhmbasket::setPriorParametersExNex(
      mu_mean = bhmbasket:::logit(0.15),
      mu_sd = 100,
      tau_scale = 0.75,
      mu_j = rep(bhmbasket:::logit(0.15), 3),
      tau_j = rep(100, 3),
      w_j = 0.5
    )
  ))

  godec <- bhmbasket::getGoDecisions(
    analyses_list = ana,
    cohort_names = paste("p", 1:3, sep = "_"),
    evidence_levels = rep(0.9, 3),
    boundary_rules = quote(c(x[1] > 0.15, x[2] > 0.15, x[3] > 0.15))
  )

  res2 <- bhmbasket::getGoProbabilities(godec)

  estim <- bhmbasket::getEstimates(
    analyses_list = ana,
    point_estimator = "mean",
    alpha_level = 0.05
  )

  # Results are equal with get_details and bhmbasket
  expect_equal(res1$Rejection_Probabilities, unname(unlist(res2))[-1])
  expect_equal(res1$Mean, unname(estim$exnex$scenario_1[, 1]))
  expect_equal(res1$MSE, unname(estim$exnex$scenario_1[, 7]))
  expect_equal(res1$Lower_CL, unname(estim$exnex$scenario_1[, 3]))
  expect_equal(res1$Upper_CL, unname(estim$exnex$scenario_1[, 5]))

  skip_on_cran()
  # Works without supplied data
  res3 <- get_details(
    design = design,
    n = 10,
    p1 = c(0.15, 0.5, 0.5),
    lambda = 0.9,
    tau_scale = 0.75,
    w = 0.5,
    iter = 100
  )
  expect_equal(length(res3), 7)
  expect_equal(res3$Rejection_Probabilities[1], res3$FWER)

  # Errors work
  data <- get_data(k = 3, n = 20, p = 0.5, iter = 100)
  expect_error(get_details(
    design,
    n = 20,
    p1 = c(0.2, 0.2, 0.5),
    lambda = 0.95,
    tau_scale = 1,
    w = 0.5,
    iter = 100,
    data = data
  ))
  expect_error(get_details(
    design,
    n = 20,
    p1 = 0.2,
    lambda = 0.95,
    tau_scale = 1,
    w = 0.5,
    iter = 100,
    data = scenarios
  ))

  # If n is passed as a vector, it should have k entries
  expect_error(get_details.exnex(
    design = design,
    n = c(10, 20),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.exnex(
    design = design,
    n = c(10, 20, 30, 40),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))

  # Works without supplied p1
  expect_no_error(get_details(
    design = design,
    n = 10,
    p1 = NULL,
    lambda = 0.9,
    tau_scale = 0.75,
    w = 0.5,
    iter = 100
  ))
})


test_that("get_details works for fujikawa", {
  n_iter <- 100
  tol <- 0.15
  # With 3 baskets
  set.seed(20230319)
  design1 <- setup_fujikawa(k = 3, p0 = 0.2)
  res1 <- get_details(
    design = design1,
    n = 15,
    p1 = c(0.2, 0.2, 0.5),
    lambda = 0.99,
    epsilon = 2,
    logbase = exp(1),
    tau = 0,
    iter = n_iter
  )

  # Works without supplied p1
  expect_no_error(get_details(
    design = design1,
    n = 15,
    p1 = NULL,
    lambda = 0.99,
    epsilon = 2,
    logbase = exp(1),
    tau = 0,
    iter = 100
  ))

  # Compare with results from baskexact
  expect_equal(
    res1$Rejection_Probabilities,
    c(0.1003108, 0.1003108, 0.5965844),
    tolerance = tol
  )
  expect_equal(res1$Mean, c(0.2717930, 0.2717930, 0.4145559), tolerance = tol)
  expect_equal(res1$MSE, c(0.01043607, 0.01043607, 0.01674920), tolerance = tol)
  expect_equal(res1$EWP, 0.5965844, tolerance = tol)
  expect_equal(res1$FWER, 0.1480935, tolerance = tol)

  # If n is passed as a vector, it should have k entries
  expect_error(get_details.fujikawa(
    design = design1,
    n = c(10, 20),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.fujikawa(
    design = design1,
    n = c(10, 20, 30, 40),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))

  skip_on_cran()
  # With 4 baskets
  set.seed(20240308)
  n_iter <- 100 # was 5000 before
  design2 <- setup_fujikawa(k = 4, p0 = 0.15)
  res2 <- get_details(
    design = design2,
    n = 15,
    p1 = c(0.15, 0.3, 0.5, 0.15),
    lambda = 0.95,
    epsilon = 2,
    logbase = 2,
    tau = 0.1,
    iter = n_iter
  )

  # Compare with results from baskexact
  expect_equal(
    res2$Rejection_Probabilities,
    c(0.3133054, 0.7274739, 0.9793341, 0.3133054),
    tolerance = tol
  )
  expect_equal(res2$FWER, 0.4696476, tolerance = tol)
  expect_equal(res2$EWP, 0.9850177, tolerance = tol)
  expect_equal(
    res2$Mean,
    c(0.2145436, 0.3134261, 0.4571671, 0.2145436),
    tolerance = tol
  )
  expect_equal(
    res2$MSE,
    c(0.009882112, 0.009797917, 0.016591157, 0.009882112),
    tolerance = tol
  )
  expect_equal(res2$ECD, 3.080197, tolerance = tol)

  # Check MCSEs
  expect_equal(res2$FWER_SE, sqrt((res2$FWER) * (1 - res2$FWER) / n_iter))
  expect_equal(res2$EWP_SE, sqrt((res2$EWP) * (1 - res2$EWP) / n_iter))
  expect_equal(
    res2$Rejection_Probabilities_SE,
    sqrt(
      (res2$Rejection_Probabilities) *
        (1 - res2$Rejection_Probabilities) /
        n_iter
    )
  )
  expect_true(res2$ECD_SE < 1 / sqrt(n_iter))
  expect_true(res2$ECD_SE > 0.001 / sqrt(n_iter))
})

test_that("switching off parallelization does not change the results of
          get_details.fujikawa()", {
  skip_on_cran()
  # With 4 baskets
  design <- setup_fujikawa(k = 4, p0 = 0.15)
  set.seed(20240308)
  res1 <- get_details(
    design = design,
    n = 15,
    p1 = c(0.15, 0.3, 0.5, 0.15),
    lambda = 0.95,
    epsilon = 2,
    logbase = 2,
    tau = 0.1,
    iter = 100,
    use_future = TRUE
  )
  set.seed(20240308)
  res2 <- get_details(
    design = design,
    n = 15,
    p1 = c(0.15, 0.3, 0.5, 0.15),
    lambda = 0.95,
    epsilon = 2,
    logbase = 2,
    tau = 0.1,
    iter = 100,
    use_future = FALSE
  )

  # Compare with results from baskexact
  expect_equal(res1$Rejection_Probabilities, res2$Rejection_Probabilities)
  expect_equal(res1$FWER, res2$FWER)
  expect_equal(res1$EWP, res2$EWP)
  expect_equal(res1$Mean, res2$Mean)
  expect_equal(res1$MSE, res2$MSE)
  expect_equal(res1$ECD, res2$ECD)
})

test_that("get_details works for fujikawa with custom weights functions", {
  # With 3 baskets
  set.seed(20230319)
  weight_noshare <- function(design, n, epsilon, tau, logbase) {
    n_sum <- n + 1
    return(diag(n_sum))
  }
  design1 <- setup_fujikawa(k = 3, p0 = 0.2)
  n <- 15
  p1 <- c(0.2, 0.5, 0.5)
  lambda <- 0.99
  logbase <- exp(1)
  tau <- 0
  epsilon <- 2
  iter <- 100
  res <- get_details(
    design = design1,
    n = n,
    p1 = p1,
    lambda = lambda,
    epsilon = epsilon,
    logbase = logbase,
    tau = tau,
    iter = iter
  )
  rescustom <- get_details(
    design = design1,
    n = n,
    p1 = p1,
    lambda = lambda,
    epsilon = epsilon,
    logbase = logbase,
    tau = tau,
    iter = iter,
    weight_fun = weight_noshare
  )
  # Compare with results with sharing - sharing should increase power and TOER
  expect_true(all(
    rescustom$Rejection_Probabilities < res$Rejection_Probabilities
  ))
  expect_true(rescustom$FWER < res$FWER)
})

test_that("get_details works for jsdglobal", {
  n_iter <- 100 # was 1000 before
  tol <- 0.1
  # With 3 baskets
  set.seed(20230515)
  design1 <- setup_jsdglobal(k = 3, p0 = 0.2)
  res1 <- get_details(
    design = design1,
    n = 12,
    p1 = c(0.2, 0.5, 0.6),
    lambda = 0.95,
    eps_pair = 1.5,
    eps_all = 0,
    iter = n_iter
  )

  # Compare with results from baskexact
  expect_equal(
    res1$Rejection_Probabilities,
    c(0.3500560, 0.8917081, 0.9701317),
    tolerance = tol
  )
  expect_equal(res1$Mean, c(0.2976754, 0.4857321, 0.5432103), tolerance = tol)
  expect_equal(res1$MSE, c(0.02132311, 0.01401632, 0.01667987), tolerance = tol)

  skip_on_cran()
  # With 4 baskets
  set.seed(20240308)
  design2 <- setup_jsdglobal(k = 4, p0 = 0.15)
  res2 <- get_details(
    design = design2,
    n = 12,
    p1 = c(0.15, 0.4, 0.3, 0.5),
    lambda = 0.99,
    eps_pair = 1.5,
    tau = 0.2,
    eps_all = 0,
    iter = n_iter
  )

  # Compare with results from baskexact
  expect_equal(
    res2$Rejection_Probabilities,
    c(0.2070795, 0.7676506, 0.5765870, 0.8949090),
    tolerance = tol
  )
  expect_equal(
    res2$Mean,
    c(0.2374997, 0.3846106, 0.3261889, 0.4454463),
    tolerance = tol
  )
  expect_equal(
    res2$MSE,
    c(0.01581763, 0.01224553, 0.01111166, 0.01745138),
    tolerance = tol
  )
  expect_equal(res2$ECD, 3.032067, tolerance = tol)
})

tol <- 0.1
test_that("get_details works for cpp", {
  # With 3 baskets
  set.seed(20230319)
  design1 <- setup_cpp(k = 3, p0 = 0.2)
  res1 <- get_details(
    design = design1,
    n = 15,
    p1 = c(0.2, 0.2, 0.5),
    lambda = 0.99,
    tune_a = 2,
    tune_b = 2,
    iter = 100
  )

  # Works without supplied p1
  expect_no_error(get_details(
    design = design1,
    n = 15,
    p1 = NULL,
    lambda = 0.99,
    tune_a = 2,
    tune_b = 2,
    iter = 100
  ))

  # Compare with results from baskexact
  expect_equal(
    res1$Rejection_Probabilities,
    c(0.06643573, 0.06643573, 0.56254586),
    tolerance = tol
  )
  expect_equal(res1$Mean, c(0.2529584, 0.2529584, 0.4173126), tolerance = tol)
  expect_equal(
    res1$MSE,
    c(0.008506501, 0.008506501, 0.018349713),
    tolerance = tol
  )

  skip_on_cran()
  # With 4 baskets
  set.seed(20240308)
  design2 <- setup_cpp(k = 4, p0 = 0.15)
  res2 <- get_details(
    design = design2,
    n = 15,
    p1 = c(0.15, 0.15, 0.15, 0.4),
    lambda = 0.99,
    tune_a = 1.5,
    tune_b = 1.5,
    iter = 100
  )

  # Compare with results from baskexact
  expect_equal(
    res2$Rejection_Probabilities,
    c(0.05833261, 0.05833261, 0.05833261, 0.40165309),
    tolerance = tol
  )
  expect_equal(res2$FWER, 0.1142539, tolerance = tol)
  expect_equal(
    res2$Mean,
    c(0.1938748, 0.1938748, 0.1938748, 0.2966433),
    tolerance = tol
  )
  expect_equal(
    res2$MSE,
    c(0.005062454, 0.005062454, 0.005062454, 0.017710313),
    tolerance = tol
  )
  expect_equal(res2$ECD, 3.226655, tolerance = tol)
  # If n is passed as a vector, it should have k entries
  expect_error(get_details.cpp(
    design = design1,
    n = c(10, 20),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.cpp(
    design = design1,
    n = c(10, 20, 30, 40),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  # n cannot be a non-positive number or a non-integer rational number
  expect_error(get_details.cpp(
    design = design1,
    n = c(10, 20, -1),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.cpp(
    design = design1,
    n = c(10, 20, 10.5),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
})

test_that("get_details works for cppglobal", {
  # With 3 baskets
  set.seed(20230512)
  design1 <- setup_cppglobal(k = 3, p0 = 0.15)
  res1 <- get_details(
    design = design1,
    n = 15,
    p1 = c(0.2, 0.4, 0.5),
    lambda = 0.98,
    tune_a = 1.5,
    tune_b = 1.5,
    epsilon = 2.5,
    iter = 100
  )

  # Compare with results from baskexact
  expect_equal(
    res1$Rejection_Probabilities,
    c(0.3270933, 0.8168261, 0.9432173),
    tolerance = tol
  )
  expect_equal(res1$Mean, c(0.2764610, 0.3967091, 0.4596072), tolerance = tol)
  expect_equal(
    res1$MSE,
    c(0.013143002, 0.009512997, 0.013149404),
    tolerance = tol
  )

  skip_on_cran()
  # With 4 baskets
  set.seed(20240308)
  design2 <- setup_cppglobal(k = 4, p0 = 0.15)
  res2 <- get_details(
    design = design2,
    n = 20,
    p1 = c(0.15, 0.25, 0.35, 0.45),
    lambda = 0.97,
    tune_a = 1.5,
    tune_b = 1,
    epsilon = 2,
    iter = 100
  )

  # Compare with results from baskexact
  expect_equal(
    res2$Rejection_Probabilities,
    c(0.2270686, 0.5359827, 0.8157377, 0.9551109),
    tolerance = tol
  )
  expect_equal(
    res2$Mean,
    c(0.2232156, 0.2794798, 0.3387829, 0.4002807),
    tolerance = tol
  )
  expect_equal(
    res2$MSE,
    c(0.008987673, 0.005730799, 0.006345472, 0.010459698),
    tolerance = tol
  )
  expect_equal(res2$ECD, 3.079763, tolerance = tol)
})

test_that("get_details works for mml", {
  # With 3 baskets
  set.seed(20240320)
  design1 <- setup_mml(k = 3, p0 = 0.2)
  res1 <- get_details(
    design = design1,
    n = 15,
    p1 = c(0.2, 0.4, 0.6),
    lambda = 0.95,
    iter = 100
  )

  # Compare with results from baskexact
  expect_equal(
    res1$Rejection_Probabilities,
    c(0.2317285, 0.7058685, 0.9801984),
    tolerance = tol
  )
  expect_equal(res1$Mean, c(0.2606807, 0.4056437, 0.5567046), tolerance = tol)
  expect_equal(res1$ECD, 2.454338, tolerance = tol)

  skip_on_cran()
  # With 4 baskets
  set.seed(20240308)
  design2 <- setup_mml(k = 4, p0 = 0.15)
  res2 <- get_details(
    design = design2,
    n = 20,
    p1 = c(0.15, 0.15, 0.4, 0.4),
    lambda = 0.95,
    iter = 100
  ) # iter was 200 before

  # Compare with results from baskexact
  expect_equal(
    res2$Rejection_Probabilities,
    c(0.2614598, 0.2614598, 0.9254998, 0.9254998),
    tolerance = tol
  )
  expect_equal(res2$FWER, 0.3914886, tolerance = tol * 2)
  expect_equal(
    res2$Mean,
    c(0.1943315, 0.1943315, 0.3738412, 0.3738412),
    tolerance = tol
  )
  expect_equal(
    res2$MSE,
    c(0.006931833, 0.006931833, 0.010724183, 0.010724183),
    tolerance = tol
  )
  expect_equal(res2$ECD, 3.32808, tolerance = tol)
})


test_that("get_details works for cpplim", {
  # compare with CPP (w = 1, alpha_0 = 1, gamma = 0)
  design_cpplim <- setup_cpplim(k = 3, p0 = 0.2)
  design_cpp <- setup_cpp(k = 3, p0 = 0.2)

  n <- 20
  data <- matrix(data = 15, ncol = 3, nrow = 100)
  p1 <- c(0.2, 0.5, 0.5)
  lambda <- 0.987

  res_cpplim <- get_details.cpplim(
    design = design_cpplim,
    n = n,
    p1 = p1,
    lambda = lambda,
    tune_a = 1,
    tune_b = 1,
    iter = 100,
    data = data
  )
  expect_gte(res_cpplim$FWER, 0)
  expect_lte(res_cpplim$FWER, 1)

  # Compare
  skip_on_cran()
  res_cpp <- get_details.cpp(
    design = design_cpp,
    n = n,
    p1 = p1,
    lambda = lambda,
    tune_a = 1,
    tune_b = 1,
    iter = 100,
    data = data
  )

  expect_equal(
    res_cpplim$Rejection_Probabilities,
    res_cpp$Rejection_Probabilities
  )
  expect_equal(res_cpplim$FWER, res_cpp$FWER)
  expect_equal(res_cpplim$Mean, res_cpp$Mean)
  expect_equal(res_cpplim$MSE, res_cpp$MSE)
  expect_equal(res_cpplim$Lower_CL, res_cpp$Lower_CL)
  expect_equal(res_cpplim$Upper_CL, res_cpp$Upper_CL)

  # Works without supplied p1
  expect_no_error(get_details(
    design = design_cpplim,
    n = 15,
    p1 = NULL,
    lambda = 0.99,
    tune_a = 2,
    tune_b = 2,
    iter = 100
  ))

  # If n is passed as a vector, it should have k entries
  expect_error(get_details.cpplim(
    design = design_cpplim,
    n = c(10, 20),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.cpplim(
    design = design_cpplim,
    n = c(10, 20, 30, 40),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
})

test_that("CPP and LCPP work", {
  # Do we need this at all? Looks very similar to the tests above.
  skip_on_cran()
  design_cpp <- setup_cpp(k = 3, p0 = 0.15, shape1 = 1, shape2 = 1)
  design_cpplim <- setup_cpplim(k = 3, p0 = 0.15, shape1 = 1, shape2 = 1)

  n <- c(10, 20, 30)
  p <- c(rep(0.35, 3))

  set.seed(12042024)
  data <- get_data(k = 3, n = n, p = p, iter = 100)

  res_cpp <- get_details(
    design = design_cpp,
    n = n,
    p1 = p,
    lambda = 0.95,
    tune_a = 2,
    tune_b = 2,
    data = data,
    iter = 100
  )
  res_cpplim <- get_details(
    design = design_cpplim,
    n = n,
    p1 = p,
    lambda = 0.95,
    tune_a = 2,
    tune_b = 2,
    data = data,
    iter = 100
  )

  # Since basket 3 has the largest sample size, alpha_o = 1 and both models should
  # have the same power in the largest basket.
  expect_equal(
    res_cpp$Rejection_Probabilities[3],
    res_cpplim$Rejection_Probabilities[3]
  )
})

test_that("get_details works for app", {
  # compare with CPP (w = 1, alpha_0 = 1, gamma = 0)
  design_app <- setup_app(k = 3, p0 = 0.2)
  design_cpp <- setup_cpp(k = 3, p0 = 0.2)

  n <- 20
  data <- matrix(data = 15, ncol = 3, nrow = 100)
  p1 <- c(0.2, 0.5, 0.5)
  lambda <- 0.987

  res_app <- get_details.app(
    design = design_app,
    n = n,
    p1 = p1,
    lambda = lambda,
    iter = 100,
    data = data
  )
  res_cpp <- get_details.cpp(
    design = design_cpp,
    n = n,
    p1 = p1,
    lambda = lambda,
    tune_a = 1,
    tune_b = 1,
    iter = 100,
    data = data
  )

  expect_equal(res_app$Rejection_Probabilities, res_cpp$Rejection_Probabilities)
  expect_equal(res_app$FWER, res_cpp$FWER)
  expect_equal(res_app$Mean, res_cpp$Mean)
  expect_equal(res_app$MSE, res_cpp$MSE)
  expect_equal(res_app$Lower_CL, res_cpp$Lower_CL)
  expect_equal(res_app$Upper_CL, res_cpp$Upper_CL)

  # Works without supplied p1
  expect_no_error(get_details(
    design = design_app,
    n = 15,
    p1 = NULL,
    lambda = 0.99,
    tune_a = 2,
    tune_b = 2,
    iter = 100
  ))

  # If n is passed as a vector, it should have k entries
  expect_error(get_details.app(
    design = design_app,
    n = c(10, 20),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
  expect_error(get_details.app(
    design = design_app,
    n = c(10, 20, 30, 40),
    p1 = NULL,
    lambda = 0.95,
    pmp0 = 1,
    data = NULL,
    iter = 110
  ))
})

test_that("get_details works for binomial", {
  design <- setup_binomial(k = 4, p0 = 0.2)
  n <- c(10, 25, 50, 100)
  p1 <- c(0.2, 0.3, 0.5, 0.2)
  details <- get_details(design = design, n = n, p1 = p1)
  # Comparing with results calculated by PASS 2024, v24.0.7
  # PASS 2024 Power Analysis and Sample Size Software (2024). NCSS, LLC.
  # Kaysville, Utah, USA, ncss.com/software/pass.
  alpha_actual <- c(0.00637, 0.01733, 0.01444, 0.02002)
  power <- c(NA_real_, 0.18944, 0.99233, NA_real_)
  reject_h0_if_R_geq <- c(6, 10, 17, 29)
  expect_equal(
    details$Rejection_Probabilities,
    c(alpha_actual[1], power[2:3], alpha_actual[4]),
    tolerance = 0.0001
  )
  expect_equal(
    details$Critical_Values,
    reject_h0_if_R_geq - 1,
    tolerance = 0.0001
  )
  expect_equal(
    details$FWER,
    1 - (1 - alpha_actual[1]) * (1 - alpha_actual[4]),
    tolerance = 0.0001
  )
  expect_equal(
    details$FWER_nominal,
    1 - (1 - 0.025)^2,
    tolerance = 0.0001
  )
  expect_equal(
    details$EWP,
    1 - (1 - power[2]) * (1 - power[3]),
    tolerance = 0.0001
  )
  expect_equal(
    details$ECD,
    (1 - alpha_actual[1]) + (1 - alpha_actual[4]) + power[2] + power[3],
    tolerance = 0.00001
  )
  # Equal n and p1
  details_all_equal <- get_details(design = design, n = 50, p1 = 0.5)
  expect_equal(
    details_all_equal$Rejection_Probabilities,
    rep(power[3], 4),
    tolerance = 0.0001
  )
  expect_equal(details_all_equal$FWER, 0)
  # Error because pooled design is not yet implemented.
  expect_error(get_details(
    design = setup_binomial(k = 4, p0 = 0.2, pool = T),
    n = n,
    p1 = p1
  ))
  # Error for wrong alpha
  expect_error(get_details(
    design = design,
    n = n,
    p1 = p1,
    alpha = 1.3
  ))
})

test_that("weight_mmlglobal works", {
  design <- setup_mmlglobal(k = 4, p0 = 0.2)
  n <- 20
  r <- c(10, 12, 14, 16)
  res <- weight_mmlglobal(design = design, n = n, r = r)

  # Compare with the StudyPrior package (not avaible on cran)
  # val1 <- StudyPrior::binom.PP.EB(x = r[-1], n = rep(n, 3), X = r[1], N = n,
  #   p.prior.a = 1, p.prior.b = 1, mix = TRUE)
  # val2 <- StudyPrior::binom.PP.EB(x = r[-2], n = rep(n, 3), X = r[2], N = n,
  #   p.prior.a = 1, p.prior.b = 1, mix = TRUE)
  # val3 <- StudyPrior::binom.PP.EB(x = r[-3], n = rep(n, 3), X = r[3], N = n,
  #   p.prior.a = 1, p.prior.b = 1, mix = TRUE)
  # val4 <- StudyPrior::binom.PP.EB(x = r[-4], n = rep(n, 3), X = r[4], N = n,
  #   p.prior.a = 1, p.prior.b = 1, mix = TRUE)
  #
  # val_res <- cbind(t(attr(val1, "pars")), t(attr(val2, "pars")),
  #   t(attr(val3, "pars")), t(attr(val4, "pars")))
  # val_res[1, ] <- val_res[1, ] + r
  # val_res[2, ] <- val_res[2, ] + n - r

  val_res <- matrix(c(23, 39.87804, 43.87857, 31, 19, 25.71951, 19.87857, 11),
    nrow = 2, byrow = TRUE)

  expect_equal(res, val_res, tolerance = 1e-04)
})

test_that("opt_design works", {
  design1 <- setup_fujikawa(k = 3, p0 = 0.2)
  scenarios <- get_scenarios(design = design1, p1 = 0.5)


  set.seed(01022024)
  # Without simulated data
  res1 <- opt_design(design = design1, n = 10, alpha = 0.05,
    design_params = list(epsilon = c(1, 2), tau = 0, logbase = exp(1)),
    scenarios = scenarios, prec_digits = 3)

  ecdres1 <- ecd(design = design1, n = 10, p1 = c(0.2, 0.2, 0.5),
    lambda = res1[1, 4], design_params = list(epsilon = 2, tau = 0,
      logbase = exp(1)))

  expect_equal(mean(as.numeric(res1[1, 5:8])), unname(res1[1, 9]))
  expect_true(abs(res1[1, 6] - ecdres1) < 0.1)

  skip_on_cran()
  ## Optimize MML global
  design2 <- setup_mmlglobal(k = 3, p0 = 0.2)
  res2 <- opt_design(design = design2, n = 10, alpha = 0.05,
                     scenarios = scenarios, prec_digits = 3)

  ecdres2 <- ecd(design = design2, n = 10, p1 = c(0.2, 0.2, 0.5),
                 lambda = as.numeric(res2[1, 1]))

  expect_equal(mean(as.numeric(res2[1, 2:5])), unname(res2[1, 6]))
  expect_true(abs(res2[1, 3] - ecdres2) < 0.1)

  # Optimize APP
  design5 <- setup_app(k = 3, p0 = 0.2)
  res5 <- opt_design(design = design5, n = 10, alpha = 0.05,
                     scenarios = scenarios, prec_digits = 3)

  ecdres5 <- ecd(design = design5, n = 10, p1 = c(0.2, 0.2, 0.5),
                 lambda = as.numeric(res5[1, 1]))

  expect_equal(mean(as.numeric(res5[1, 2:5])), unname(res5[1, 6]))
  expect_true(abs(res5[1, 3] - ecdres5) < 0.1)


  # With simulated data
  set.seed(123456)
  scenario_list <- as.list(data.frame(scenarios))
  data_list <- lapply(scenario_list,
    function(x) get_data(k = 3, n = 15, p = x, iter = 100))
  res3 <- opt_design(design = design1, n = 15, alpha = 0.05,
    design_params = list(epsilon = c(1, 2), tau = 0), scenarios = scenarios,
    prec_digits = 3, data = data_list, iter = 100)
  res4 <- opt_design(design = design1, n = 15, alpha = 0.05,
    design_params = list(epsilon = c(1, 2), tau = 0), scenarios = scenarios,
    prec_digits = 3, data = data_list, iter = 100)

  # Check if results are equal when opt_design is called two times with
  # the same simulate data
  expect_true(all(res3 == res4))

  lambdares <- adjust_lambda(design = design1, n = 15, p1 = NULL, alpha = 0.05,
    design_params = list(epsilon = 2, tau = 0), iter = 100, prec_digits = 3,
    data = data_list[[1]])
  ecdres <- ecd(design = design1, n = 15, p1 = 0.2, lambda = lambdares$lambda,
    design_params = list(epsilon = 2, tau = 0), iter = 100,
    data = data_list[[1]])

  expect_equal(lambdares$lambda, res3$Lambda[1])
  expect_equal(res3[1, 4], ecdres)
})

test_that("get_evaluation works for bhm", {
  design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)

  # Works with equal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(20, 20, 20), r = c(10, 15, 5),
                                 lambda = 0.95, tau_scale = 1, iter = 100))

  # Works with unequal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
                                 lambda = 0.95, tau_scale = 1, iter = 100))

  # If n is passed as a vector, it should have k entries
  expect_error(get_evaluation(design = design, n = c(15, 20), r = c(10, 15, 17),
                              lambda = 0.95, tau_scale = 1, iter = 100))

  expect_error(get_evaluation(design = design, n = c(15, 20, 25, 30), r = c(10, 15, 17),
                              lambda = 0.95, tau_scale = 1, iter = 100))

})



test_that("get_evaluation works for exnex", {
  design <- setup_exnex(k = 3, p0 = 0.2)

  # Works with equal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(20, 20, 20), r = c(10, 15, 5),
                                 lambda = 0.95, tau_scale = 1, w = 0.5, iter = 100))

  # Works with unequal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
                                 lambda = 0.95, tau_scale = 1, w = 0.5, iter = 100))

  # If n is passed as a vector, it should have k entries
  expect_error(get_evaluation(design = design, n = c(15, 20), r = c(10, 15, 17),
                              lambda = 0.95, tau_scale = 1, w = 0.5, iter = 100))

  expect_error(get_evaluation(design = design, n = c(15, 20, 25, 30), r = c(10, 15, 17),
                              lambda = 0.95, tau_scale = 1, w = 0.5, iter = 100))

})



test_that("get_evaluation works for fujikawa", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)

  # Works with equal sample sizes
  expect_no_error(get_evaluation(design = design, n = 20, r = c(10, 15, 5), lambda = 0.95,
                                 epsilon = 2, tau = 0, iter = 100))

  # Works with unequal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
                                 lambda = 0.95, epsilon = 2, tau = 0, iter = 100))

  # If n is passed as a vector, it should have k entries
  expect_error(get_evaluation(design = design, n = c(15, 20), r = c(10, 15, 17),
                              lambda = 0.95, pmp0 = 1, iter = 100))

  expect_error(get_evaluation(design = design, n = c(15, 20, 25, 30), r = c(10, 15, 17),
                              lambda = 0.95, pmp0 = 1, iter = 100))

})



test_that("get_evaluation works for cpp", {
  design <- setup_cpp(k = 3, p0 = 0.2)

  # Works with equal sample sizes
  expect_no_error(get_evaluation(design = design, n = 20, r = c(10, 15, 5),
                                   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

  # Works with unequal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
                                 lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

  # If n is passed as a vector, it should have k entries
  expect_error(get_evaluation(design = design, n = c(15, 20), r = c(10, 15, 17),
                              lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

  expect_error(get_evaluation(design = design, n = c(15, 20, 25, 30), r = c(10, 15, 17),
                              lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

})



test_that("get_evaluation works for cpplim", {
  design <- setup_cpplim(k = 3, p0 = 0.2)

  # Works with equal sample sizes
  expect_no_error(get_evaluation(design = design, n = 20, r = c(10, 15, 5),
                                 lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

  # Works with unequal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
                                 lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

  # If n is passed as a vector, it should have k entries
  expect_error(get_evaluation(design = design, n = c(15, 20), r = c(10, 15, 17),
                              lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

  expect_error(get_evaluation(design = design, n = c(15, 20, 25, 30), r = c(10, 15, 17),
                              lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100))

})



test_that("get_evaluation works for app", {
  design <- setup_app(k = 3, p0 = 0.2)

  # Works with equal sample sizes
  expect_no_error(get_evaluation(design = design, n = 20, r = c(10, 15, 5),
                                 lambda = 0.95, iter = 100))

  # Works with unequal sample sizes
  expect_no_error(get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
                                 lambda = 0.95, iter = 100))

  # If n is passed as a vector, it should have k entries
  expect_error(get_evaluation(design = design, n = c(15, 20), r = c(10, 15, 17),
                              lambda = 0.95, iter = 100))

  expect_error(get_evaluation(design = design, n = c(15, 20, 25, 30), r = c(10, 15, 17),
                              lambda = 0.95, iter = 100))

})

test_that("setup_mml works", {
  design <- setup_mml(k = 3, p0 = 0.2)
  expect_s3_class(design, "mml")
})

test_that("setup_mmlglobal works", {
  design <- setup_mmlglobal(k = 3, p0 = 0.2)
  expect_s3_class(design, "mmlglobal")
})

test_that("setup_bhm works", {
  design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
  expect_s3_class(design, "bhm")
})

test_that("setup_exnex works", {
  design <- setup_exnex(k = 3, p0 = 0.2)
  expect_s3_class(design, "exnex")
})

test_that("setup_fujikawa works", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)
  expect_s3_class(design, "fujikawa")
})

test_that("setup_jsdglobal works", {
  design <- setup_jsdglobal(k = 3, p0 = 0.2)
  expect_s3_class(design, "jsdglobal")
})

test_that("setup_cpp works", {
  design <- setup_cpp(k = 3, p0 = 0.2)
  expect_s3_class(design, "cpp")
})

test_that("setup_cppglobal works", {
  design <- setup_cppglobal(k = 3, p0 = 0.2)
  expect_s3_class(design, "cppglobal")
})

test_that("setup_binomial works", {
  design <- setup_binomial(k = 3, p0 = 0.2)
  expect_s3_class(design, "binomial")
})

test_that("errors in setup functions work", {
  expect_error(setup_fujikawa(k = 0, p0 = 0.2))
  expect_error(setup_jsdglobal(k = 3, p0 = 1))
  expect_error(setup_cpp(k = 2.5, p0 = 0.5))
  expect_error(setup_cppglobal(k = 3, p0 = 0.2, shape1 = 0, shape2 = 0))
  expect_error(setup_binomial(k = 3, p0 = 0.2, pool = "not boolean"))
})

# Beta Borrowing for Global JSD Design
beta_borrow_jsdglobal <- function(design, n, r, weights_pair, eps_all) {
  shape_noprior <- matrix(c(r, n - r), nrow = 2, byrow = TRUE)
  shape <- matrix(c(design$shape1 + r, design$shape1 + n - r), nrow = 2,
    byrow = TRUE)
  # Compute global weight
  weight_all <- jsd_global(shape = shape, epsilon = eps_all)

  # Compute pairwise weights and multiply by global weight
  all_combs <- arrangements::combinations(r, 2) + 1
  weights_vec <- weights_pair[all_combs] * weight_all

  # Create weight k x k weight matrix
  weight_mat <- matrix(0, nrow = design$k, ncol = design$k)
  weight_mat[lower.tri(weight_mat)] <- weights_vec
  weight_mat <- weight_mat + t(weight_mat)
  diag(weight_mat) <- 1

  # Compute posterior shapes
  shape1post <- apply(weight_mat, 1, function(x) sum(shape_noprior[1, ] * x)) +
    design$shape1
  shape2post <- apply(weight_mat, 1, function(x) sum(shape_noprior[2, ] * x)) +
    design$shape2
  rbind(shape1post, shape2post)
}

# Compute global Jensen-Shannon divergence
jsd_global <- function(shape, epsilon) {
  k <- ncol(shape)
  bf <- function(x, i) stats::dbeta(x, shape[1, i], shape[2, i])
  bff <- Vectorize(function(x) sum(mapply(bf, i = 1:k,
    MoreArgs = list(x = x))) / k)
  klf <- function(x, i) bf(x, i) * log(bf(x, i) / bff(x), base = k)

  kl <- numeric(k)
  for (i in 1:k) {
    kl[i] <- stats::integrate(function(x) klf(x, i), 0, 1)$value
  }
  (1 - mean(kl))^epsilon
}

# Analyzing Results for Global JSD Design
ana_jsdglobal <- function(design, n, r, eps_all, lambda, weights_pair) {
  shape_post <- beta_borrow_jsdglobal(design = design, n = n, r = r,
    weights_pair = weights_pair, eps_all = eps_all)
  post_prob <- post_beta(shape = shape_post, p0 = design$p0)
  ifelse(post_prob >= lambda, 1, 0)
}

#' Optimize a Basket Trial Design
#'
#' @template design
#' @template n
#' @template alpha
#' @template design_params
#' @param scenarios A matrix of scenarios.
#' @template prec_digits
#' @template iter
#' @param data A list of data matrices generated with \code{get_data}. The
#'   list elements have to correspond to the columsn of \code{scenarios}.
#' @template dotdotdot
#'
#' @return A matrix with the expected number of correct decisions.
#' @export
#'
#' @examples
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#' scenarios <- get_scenarios(design, p1 = 0.5)
#'
#' ## Equal sample sizes
#' # Without simulated data
#' \donttest{
#' opt_design(design, n = 20, alpha = 0.05, design_params =
#'            list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
#'            prec_digits = 3)
#' }
#'
#' # With simulated data
#' scenario_list <- as.list(data.frame(scenarios))
#' data_list <- lapply(scenario_list,
#'   function(x) get_data(k = 3, n = 20, p = x, iter = 1000))
#' \donttest{
#' opt_design(design, n = 20, alpha = 0.05, design_params =
#'            list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
#'            prec_digits = 3, data = data_list)
#'
#'
#' ## Unequal sample sizes
#' # Without simulated data
#' opt_design(design, n = c(15, 20, 25), alpha = 0.05, design_params =
#'            list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
#'            prec_digits = 3)
#'
#' # With simulated data
#' scenario_list <- as.list(data.frame(scenarios))
#' data_list <- lapply(scenario_list,
#'                     function(x) get_data(k = 3, n = c(15, 20, 25),
#'                                          p = x, iter = 1000))
#' opt_design(design, n = c(15, 20, 25), alpha = 0.05, design_params =
#'            list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
#'            prec_digits = 3, data = data_list)
#' }
opt_design <- function(design, n, alpha, design_params = list(), scenarios,
                       prec_digits, iter = 1000, data = NULL, ...) {
  check_data_list(data = data, scenarios = scenarios)
  check_scenarios(scenarios = scenarios, design = design)
  grid <- expand.grid(design_params)
  if (length(design_params) == 0) {
    lgrid <- 1
  } else {
    lgrid <- nrow(grid)
  }
  p <- progressr::progressor(steps = lgrid)

  ecd_res <- matrix(nrow = lgrid, ncol = ncol(scenarios))
  colnames(ecd_res) <- colnames(scenarios)
  lambdas <- numeric(lgrid)
  null_scen <- which(apply(scenarios, 2, function(x) all(x == design$p0)))

  for (i in 1:lgrid) {
    params_loop <- lapply(as.list(grid), function(x) x[i])
    l <- do.call(adjust_lambda, args = list(design = design, n = n,
      p1 = NULL, alpha = alpha, design_params = params_loop, iter = iter,
      prec_digits = prec_digits, data = data[[null_scen]], ...))
    lambdas[i] <- l$lambda

    for (j in 1:ncol(scenarios)) {
      ecd_res[i, j] <- do.call(ecd, args = c(design = list(design), n = list(n),
        p1 = list(scenarios[, j]), lambda = l$lambda, params_loop,
        iter = iter, data = list(data[[j]]), ...))
    }
    p()
  }

  if (ncol(grid) == 0) {
    ecd_res <- cbind("Lambda" = lambdas, ecd_res,
      "Mean_ECD" = rowMeans(ecd_res))
  } else {
    ecd_res <- cbind(grid, "Lambda" = lambdas, ecd_res,
      "Mean_ECD" = rowMeans(ecd_res))
    ecd_res <- ecd_res[order(ecd_res[, ncol(ecd_res)], decreasing = TRUE), ]
  }
  ecd_res
}

#' Create a Scenario Matrix
#'
#' Creates a default scenario matrix.
#'
#' @template design
#' @param p1 Probability under the alternative hypothesis.
#'
#' @details \code{get_scenarios} creates a default scenario matrix
#' that can be used for \code{\link{opt_design}}. The function creates
#' \code{k + 1} scenarios, from a global null to a global alternative scenario.
#'
#' @return A matrix with \code{k} rows and \code{k + 1} columns.
#' @export
#'
#' @examples
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#' get_scenarios(design = design, p1 = 0.5)
get_scenarios <- function(design, p1) {
  scen_mat <- matrix(nrow = design$k, ncol = design$k + 1)
  for (i in 0:design$k) {
    scen_mat[, (i + 1)] <- c(rep(design$p0, design$k - i),
      rep(p1, i))
  }
  colnames(scen_mat) <- paste(0:design$k, "Active")
  scen_mat
}

# Check function for parameters k and n
validate_basics <- function(x) {
  if ((x$k %% 1) != 0) {
    stop("k must be an integer")
  }
  if (x$k <= 1) {
    stop("k must be greater than or equal to 2")
  }
  if (x$p0 <= 0 | x$p0 >= 1) {
    stop("p0 must be between 0 and 1")
  }
  x
}

# Check function for designs based on the beta-binomial distribution
validate_betabin <- function(x) {
  validate_basics(x)
  if (x$shape1 <= 0 | x$shape2 <= 0) {
    stop("shape1 and shape2 must be greater than 0")
  }
  x
}

# Calculate Posterior probabilities of a Beta Distribution
post_beta <- function(shape, p0) {
  stats::pbeta(p0, shape1 = shape[1, ], shape2 = shape[2, ], lower.tail = FALSE)
}

cfun1 <- function(x, y) {
  list(
    rbind(x[[1]], y[[1]]),
    rbind(x[[2]], y[[2]]),
    rbind(x[[3]], y[[3]]),
    rbind(x[[4]], y[[4]])
  )
}

check_scenarios <- function(scenarios, design) {
  if (!(is.matrix(scenarios) | is.data.frame(scenarios))) {
    stop("scenarios is not a matrix or a data.frame")
  }
  if (sum(duplicated(t(scenarios))) > 0) {
    stop("not all scenarios are distinct")
  }
  if (sum(apply(scenarios, 2, function(x) all(x == design$p0))) == 0) {
    stop("no null scenario")
  }
  if (nrow(scenarios) != design$k) {
    stop("scenarios doesn't have k rows")
  }
  if (!all(scenarios > 0) | !all(scenarios < 1)) {
    stop("probabilities have to be in (0, 1)")
  }
}

check_p1 <- function(design, p1, data) {
  if (is.null(p1) & is.null(data)) {
    p1 <- rep(design$p0, design$k)
  }
  if (any(p1 < design$p0)) {
    stop("all p1 must be greater than or equal to p0")
  }
  if ((length(p1) != design$k) & (length(p1) != 1) & (!is.null(p1))) {
    stop("p1 must either have length k or 1")
  }
  p1
}

check_params_differentn_frequentist <- function(design, n, alpha) {
  check_differentn(design, n)
  if (alpha <= 0 | alpha >= 1) {
    stop(paste0("alpha must be between 0 and 1. ", "alpha is ", alpha, "."))
  }
}

check_params_differentn <- function(design, n, lambda, iter) {
  check_differentn(design, n)
  check_lambda(lambda)
  check_iter(iter)
}

check_params <- function(n, lambda, iter) {
  if (length(n) != 1) {
    stop(paste0("n must have length 1. The current length is ", length(n), "."))
  }
  if ((n <= 0) | (n %% 1 != 0)) {
    stop(paste0("n must be a positive integer. ", "n is ", n, "."))
  }
  check_lambda(lambda)
  check_iter(iter)
}

check_differentn <- function(design, n) {
  # n must be passed in the correct form
  if ((length(n) < design$k & length(n) != 1) | length(n) > design$k) {
    stop("n must either have length 1 or k")
  }
  if (any((n <= 0)) | any((n %% 1 != 0))) {
    stop(paste0(
      "n must be a vector of positive integers. ",
      "n is c(",
      do.call(paste, c(as.list(n), sep = ", ")),
      ")."
    ))
  }
}

check_lambda <- function(lambda) {
  if (lambda <= 0 | lambda >= 1) {
    stop(paste0("lambda must be between 0 and 1. ", "lambda is ", lambda, "."))
  }
}

check_iter <- function(iter) {
  if ((iter <= 0) | (iter %% 1 != 0)) {
    stop(paste0("iter must be a positive integer. ", "iter is ", iter, "."))
  }
}

mcse_rate <- function(rate, iter) {
  return(sqrt(rate * (1 - rate) / iter))
}

test_that("check_scenarios works", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)
  scenarios <- get_scenarios(design = design, p1 = 0.5)

  expect_error(check_scenarios(scenarios = scenarios[, -1], design = design))
  expect_error(check_scenarios(scenarios = cbind(scenarios, scenarios[, 1]),
    design = design))
  expect_error(check_scenarios(scenarios = list(scenarios),
    design = design))
  expect_error(check_scenarios(scenarios = scenarios[-1, -4], design = design))
  expect_error(check_scenarios(scenarios = cbind(scenarios, c(0.2, 0.2, 1)),
    design = design))
})

test_that("check_p1 works", {
  design <- setup_mml(k = 3, p0 = 0.2)
  expect_equal(check_p1(design = design, p1 = NULL, data = NULL), c(0.2, 0.2, 0.2))
  expect_error(check_p1(design = design, p1 = c(0.1, 0.3, 0.4), data = NULL))
  expect_error(check_p1(design = design, p1 = c(0.2, 0.4), data = NULL))
})

test_that("check_params works", {
  expect_error(check_params(n = c(10, 20), lambda = 0.99, iter = 100))
  expect_error(check_params(n = -3, lambda = 0.99, iter = 100))
  expect_error(check_params(n = 10, lambda = 1, iter = 100))
  expect_error(check_params(n = 10, lambda = 0.99, iter = 40.5))
})

test_that("check_get_weights_cpp", {
  # Comparison of the two calculation methods. The case of equal sample sizes
  # has already been checked with baskexact

  expect_equal(get_weights_cpp(n = 5, tune_a = 1, tune_b = 1),
               get_weights_cpp(n = c(5,5,6,7), tune_a = 1, tune_b = 1)[[1]])

})



test_that("check_get_weights_jsd", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)

  # Comparison of the two calculation methods. The case of equal sample sizes
  # has already been checked with baskexact

  expect_equal(get_weights_jsd(design = design, n = 5, epsilon = 2, tau = 0,
                               logbase = 2),
               get_weights_jsd(design = design, n = c(5,5,6,7), epsilon = 2,
                               tau = 0, logbase = 2)[[1]])

})

# Beta Borrowing for Global CPP Design
beta_borrow_cppglobal <- function(design, n, r, weights_pair, epsilon) {
  shape_noprior <- matrix(c(r, n - r), nrow = 2, byrow = TRUE)
  weight_all <- diff_all(n = n, r = r, epsilon = epsilon)

  # Compute pairwise weights and multiply by global weight
  all_combs <- arrangements::combinations(r, 2) + 1
  weights_vec <- weights_pair[all_combs] * weight_all

  # Create weight k x k weight matrix
  weight_mat <- matrix(0, nrow = design$k, ncol = design$k)
  weight_mat[lower.tri(weight_mat)] <- weights_vec
  weight_mat <- weight_mat + t(weight_mat)
  diag(weight_mat) <- 1

  # Compute posterior shapes
  shape1post <- apply(weight_mat, 1, function(x) sum(shape_noprior[1, ] * x)) +
    design$shape1
  shape2post <- apply(weight_mat, 1, function(x) sum(shape_noprior[2, ] * x)) +
    design$shape2
  rbind(shape1post, shape2post)
}

# Global Weights for Global CPP Design
diff_all <- function(n, r, epsilon) {
  rr <- r / n
  rs <- sort(rr)
  d <- diff(rs)
  (1 - sum(d) * 10^(-sum((d - 1 / length(d))^2)))^epsilon
}

# Analyzing Results for CPP Design
ana_cppglobal <- function(design, n, r, lambda, weights_pair, epsilon) {
  shape_post <- beta_borrow_cppglobal(design = design, n = n, r = r,
    weights_pair = weights_pair, epsilon = epsilon)
  post_prob <- post_beta(shape = shape_post, p0 = design$p0)
  ifelse(post_prob >= lambda, 1, 0)
}

test_that("diff_all works", {
  res1 <- diff_all(n = 20, r = c(0, 10, 20), epsilon = 1)
  res2 <- diff_all(n = 20, r = c(5, 5, 5), epsilon = 1)

  expect_equal(res1, 0)
  expect_equal(res2, 1)
})

test_that("beta_borrow_cppglobal works", {
  design <- setup_cpp(k = 4, p0 = 0.2)
  r <- c(17, 15, 9, 5)

  weight_mat <- get_weights_cpp(n = 20, a = 1, b = 1)
  res1 <- beta_borrow_cppglobal(design = design, n = 20, r = r,
    weights_pair = weight_mat, epsilon = 1.5)
  res2 <- val_borrow_cpp(design = design, n = 20, r = r, a = 1, b = 1,
    globalweight_fun = diff_all, globalweight_params = list(epsilon = 1.5))
  res3 <- beta_borrow_cppglobal(design = design, n = 20, r = rev(r),
    weights_pair = weight_mat, epsilon = 1.5)

  expect_equal(unname(res1), unname(res2))
  expect_equal(res1, res3[, 4:1])
})

#' Simulate Data Based on a Binomial Distribution
#'
#' @template k
#' @template n
#' @param p Probabilities used to simulate the data
#' @template iter
#' @param type Type of output. Use \code{bhmbasket} for the BHM and EXNED
#'   design and \code{matrix} for everything else.
#'
#' @details For \code{type = "bhmbasket"} this is simply a wraper for
#' \code{bhmbasket::simulateScenarios}.
#'
#' @return If \code{type = "matrix"} then a matrix is returned, if
#' \code{type = "bhmbasket"} then an element with class \code{scenario_list}.
#' @export
#'
#' @examples
#' # Equal sample sizes
#' get_data(k = 3, n = 20, p = c(0.2, 0.2, 0.5), iter = 1000,
#'   type = "matrix")
#'
#' # Unequal sample sizes
#' get_data(k = 3, n = c(15, 20, 25), p = c(0.2, 0.2, 0.5),
#'   iter = 1000, type = "matrix")
get_data <- function(k, n, p, iter, type = c("matrix", "bhmbasket")) {
  type <- match.arg(type)
  if (length(p) == 1) {
    pvec <- rep(p, k)
  } else if (length(p) == k) {
    pvec <- p
  } else {
    stop("p1 must either have length 1 or k")
  }

  if(length(n) == 1)  n <- c(rep(n,k))

  if(length(n) < k || length(n) > k){
    stop("n must be either an integer or a vector of length k")
  }

  data <- matrix(data = NA, nrow = iter, ncol = k)

  if(type == "matrix"){
    for(j in 1:iter){
      data[j,] <- stats::rbinom(n = k, size = n, prob = pvec)
    }
    attr(data, "n") <- n
    attr(data, "p") <- p
    data
  }else {
    bhmbasket::simulateScenarios(
      n_subjects_list = n,
      response_rates_list = list(pvec),
      n_trials = iter
    )
  }
}

check_data_matrix <- function(data, design, n, p, iter) {
  if (is.null(data)) {
    get_data(k = design$k, n = n, p = p, iter = iter, type = "matrix")
  } else {
    if (!inherits(data, "matrix")) {
      stop("data are not a matrix")
    }
    if (ncol(data) != design$k) {
      stop("data do not have k columns")
    }
    if (!all(attr(data, "n") == n)) {
      stop("data were not generated with the specified n")
    }
    if (!all(attr(data, "p") == p)) {
      stop("data were not generated with the specified p1")
    }
    if (nrow(data) != iter) {
      message("data do not have iter rows - argument iter is ignored")
    }
    data
  }
}

check_data_bhmbasket <- function(data, design, n, p, iter) {
  if (is.null(data)) {
    get_data(k = design$k, n = n, p = p, iter = iter, type = "bhmbasket")
  } else {
    if (!inherits(data, "scenario_list")) {
      stop("data are not of class scenario_list")
    }
    if (ncol(data$scenario_1$n_responders) != design$k) {
      stop("data do not have k columns")
    }
    if (!all(data$scenario_1$n_subjects == n)) {
      stop("data were not generated with the specified n")
    }
    if (!all(data$scenario_1$response_rates == p) & !is.null(p)) {
      stop("data were not generated with the specified p1")
    }
    if (nrow(data$scenario_1$n_responders) != iter) {
      message("data do not have iter rows - argument iter is ignored")
    }
    data
  }
}

check_data_list <- function(data, scenarios) {
  if (!is.null(data)) {
    if (!is.list(data)) {
      stop("data are not a list")
    }
    if (length(data) != ncol(scenarios)) {
      stop("data do not have an element for each scenario")
    }
  }
}

test_that("get_data works", {
  set.seed(123)
  data1 <- get_data(k = 3, n = 20, p = 0.2, iter = 100)

  expect_true(all(dim(data1) == c(100, 3)))
  expect_equal(attr(data1, "n"), c(20,20,20))
  expect_equal(attr(data1, "p"), 0.2)
  expect_error(get_data(k = 3, n = 20, p = c(0.2, 0.5), iter = 100))
  expect_error(get_data(k = 3, n = c(12,15), p = 0.4, iter = 100))


  set.seed(123)
  data2 <- get_data(k = 3, n = 20, p = 0.2, iter = 100, type = "bhmbasket")

  expect_s3_class(data2, "scenario_list")
  expect_true(all(data1 == data2$scenario_1$n_responders))
})

test_that("check_data_matrix works", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)
  data1 <- get_data(k = 3, n = 20, p = 0.2, iter = 100)
  data2 <- get_data(k = 4, n = 20, p = 0.2, iter = 100)

  expect_error(check_data_matrix(data = data.frame(data1), design = design,
    n = 20, p = NULL, iter = 100))
  expect_error(check_data_matrix(data = data2, design = design, n = 20,
    p = NULL, iter = 100))
  expect_error(check_data_matrix(data = data1, design = design, n = 19,
    p = 0.2, iter = 100))
  expect_error(check_data_matrix(data = data1, design = design, n = 20,
    p = 0.5, iter = 100))
  expect_message(check_data_matrix(data = data1, design = design, n = 20,
    p = NULL, iter = 101))
  expect_equal(data1, check_data_matrix(data = data1, design = design,
    n = 20, p = NULL, iter = 100))
})

test_that("check_data_bhmbasket works", {
  design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
  data1 <- get_data(k = 3, n = 20, p = 0.2, iter = 100, type = "bhmbasket")
  data2 <- get_data(k = 3, n = 20, p = 0.2, iter = 100)
  data3 <- get_data(k = 4, n = 20, p = 0.2, iter = 100, type = "bhmbasket")
  data4 <- get_data(k = 3, n = c(15,20,25), p = 0.2, iter = 100,
                    type = "bhmbasket")

  expect_error(check_data_bhmbasket(data = data2, design = design,
    n = 20, p = NULL, iter = 100))
  expect_error(check_data_bhmbasket(data = data3, design = design, n = 20,
    p = NULL, iter = 100))
  expect_error(check_data_bhmbasket(data = data1, design = design, n = 19,
    p = 0.2, iter = 100))
  expect_error(check_data_bhmbasket(data = data1, design = design, n = 20,
    p = 0.5, iter = 100))
  expect_error(check_data_bhmbasket(data = data4, design = design, n = 20,
    p = 0.2, iter = 100))
  expect_error(check_data_bhmbasket(data = data4, design = design,
    n = c(20,20,25), p = 0.2, iter = 100))
  expect_message(check_data_bhmbasket(data = data1, design = design, n = 20,
    p = NULL, iter = 101))
  expect_equal(data1, check_data_bhmbasket(data = data1, design = design, n = 20,
    p = NULL, iter = 100))
})

test_that("check_data_list works", {
  design <- setup_fujikawa(k = 3, p0 = 0.2)
  data <- get_data(k = 3, n = 20, p = 0.2, iter = 100)
  scenarios <- get_scenarios(design, p1 = 0.5)
  scenario_list <- as.list(data.frame(scenarios))
  data_list <- lapply(scenario_list,
    function(x) get_data(k = 3, n = 20, p = x, iter = 1000))

  expect_error(check_data_list(data = data, scenarios = scenarios))
  expect_error(check_data_list(data = data_list[1:3], scenarios = scenarios))
})

#' Get Results for Simulation of Basket Trial Designs
#'
#' @template design
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
get_results <- function(design, ...) {
  UseMethod("get_results", design)
}


#' Get Results for Simulation of a Basket Trial with the MML Design
#'
#' @template design_mml
#' @template n
#' @template p1
#' @template lambda
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_mml(k = 3, p0 = 0.2)
#' get_results(design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   iter = 100)
get_results.mml <- function(design, n, p1 = NULL, lambda, iter = 1000,
                            data = NULL, ...) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
    iter = iter)
  weights <- get_weights_mml(design, n, ...)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_pp(design = design, n = n, r = data[i, ], lambda = lambda,
      weights = weights)
  }
}

#' Get Results for Simulation of a Basket Trial with the Global MML Design
#'
#' @template design_mmlglobal
#' @template n
#' @template p1
#' @template lambda
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_mmlglobal(k = 3, p0 = 0.2)
#' get_results(design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   iter = 100)
get_results.mmlglobal <- function(design, n, p1 = NULL, lambda, iter = 1000,
                                  data = NULL, ...) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
    iter = iter)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind',
                   .options.future = list(seed = TRUE)) %dofuture% {
    ana_mmlglobal(design = design, n = n, r = data[i, ], lambda = lambda)
  }
}

#' Get Results for Simulation of a Basket Trial with the BHM Design
#'
#' @template design_bhm
#' @template n
#' @template p1
#' @template lambda
#' @template tau_bhm
#' @template iter
#' @template n_mcmc
#' @template data_bhm
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' \donttest{
#' design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
#'
#' # Equal sample sizes
#' get_results(design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, iter = 100)
#' }
get_results.bhm <- function(design, n, p1 = NULL, lambda, tau_scale,
                            iter = 1000, n_mcmc = 10000, data = NULL, ...) {

  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  data <- check_data_bhmbasket(data = data, design = design, n = n, p = p1,
    iter = iter)

  analyses <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = data,
    evidence_levels = lambda,
    method_names = "berry",
    target_rates = rep(design$p_target, design$k),
    prior_parameters_list = bhmbasket::setPriorParametersBerry(
      mu_mean = design$mu_mean,
      mu_sd = design$mu_sd,
      tau_scale = tau_scale
    ) ,
    n_mcmc_iterations = n_mcmc
  ))

  br <- paste0("c(", paste0("x[", 1:design$k, "] > ", design$p0,
    collapse = ", "), ")")
  res <- bhmbasket::getGoDecisions(
    analyses_list = analyses,
    cohort_names = paste("p", 1:design$k, sep = "_"),
    evidence_levels = rep(lambda, design$k),
    boundary_rules = str2lang(br),
    ...
  )

  ifelse(res$scenario_1$decisions_list$berry[, -1] == "TRUE", 1, 0)
}

#' Get Results for Simulation of a Basket Trial with the EXNEX Design
#'
#' @template design_exnex
#' @template n
#' @template p1
#' @template lambda
#' @template tau_exnex
#' @template w_exnex
#' @template iter
#' @template n_mcmc
#' @template data_bhm
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_exnex(k = 3, p0 = 0.2)
#' \donttest{
#' # Equal sample sizes
#' get_results(design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
#' }
get_results.exnex <- function(design, n, p1 = NULL, lambda, tau_scale, w_j,
                              iter = 1000, n_mcmc = 10000, data = NULL, ...) {

  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  data <- check_data_bhmbasket(data = data, design = design, n = n, p = p1,
    iter = iter)

  analyses <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = data,
    evidence_levels = lambda,
    method_names = "exnex",
    prior_parameters_list = bhmbasket::setPriorParametersExNex(
      mu_mean = design$mu_mean,
      mu_sd = design$mu_sd,
      tau_scale = tau_scale,
      mu_j = rep(design$basket_mean, design$k),
      tau_j = rep(design$basket_sd, design$k),
      w_j = w_j
    ),
    n_mcmc_iterations = n_mcmc
  ))

  br <- paste0("c(", paste0("x[", 1:design$k, "] > ", design$p0,
    collapse = ", "), ")")
  res <- bhmbasket::getGoDecisions(
    analyses_list = analyses,
    cohort_names = paste("p", 1:design$k, sep = "_"),
    evidence_levels = rep(lambda, design$k),
    boundary_rules = str2lang(br)
  )

  ifelse(res$scenario_1$decisions_list$exnex[, -1] == "TRUE", 1, 0)
}

#' Get Results for Simulation of a Basket Trial with Fujikawa's Design
#'
#' @template design_fujikawa
#' @template n
#' @template p1
#' @template lambda
#' @template tuning_fujikawa
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
get_results.fujikawa <- function(design, n, p1 = NULL, lambda, epsilon, tau,
                                 logbase = 2, iter = 1000, data = NULL,
                                 ...) {

  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
    iter = iter)
  weights <- get_weights_jsd(design = design, n = n, epsilon = epsilon,
    tau = tau, logbase = logbase)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_fujikawa(design = design, n = n, r = data[i, ], lambda = lambda,
      weights = weights)
  }
}

#' Get Results for Simulation of a Basket Trial with the Power Prior Design
#' Based on Global JSD Weights
#'
#' @template design_jsdglobal
#' @template n
#' @template p1
#' @template lambda
#' @template tuning_jsdglobal
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_jsdglobal(k = 3, p0 = 0.2)
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   eps_pair = 2, eps_all = 2, iter = 100)
get_results.jsdglobal <- function(design, n, p1 = NULL, lambda, eps_pair,
                                  tau = 0, eps_all, logbase = 2, iter = 1000,
                                  data = NULL, ...) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
    iter = iter)
  weights_pair <- get_weights_jsd(design = design, n = n, epsilon = eps_pair,
    tau = tau, logbase = logbase)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_jsdglobal(design = design, n = n, r = data[i, ], eps_all = eps_all,
      lambda = lambda, weights_pair = weights_pair)
  }
}

#' Get Results for Simulation of a Basket Trial with a Calibrated Power Prior
#' Design
#'
#' @template design_cpp
#' @template n
#' @template p1
#' @template lambda
#' @template tuning_cpp
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_cpp(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
get_results.cpp <- function(design, n, p1 = NULL, lambda, tune_a, tune_b,
                            iter = 1000, data = NULL, ...) {

  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
    iter = iter)
  weights <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_pp(design = design, n = n, r = data[i, ], lambda = lambda,
      weights = weights)
  }
}

#' Get Results for Simulation of a Basket Trial with a Global Calibrated
#' Power Prior Design
#'
#' @template design_cppglobal
#' @template n
#' @template p1
#' @template lambda
#' @template tuning_cppglobal
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_cppglobal(k = 3, p0 = 0.2)
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   tune_a = 1, tune_b = 1, epsilon = 2, iter = 100)
get_results.cppglobal <- function(design, n, p1 = NULL, lambda, tune_a, tune_b,
                                  epsilon, iter = 1000, data = NULL, ...) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
    iter = iter)
  weights_pair <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_cppglobal(design = design, n = n, r = data[i, ], lambda = lambda,
      weights_pair = weights_pair, epsilon = epsilon)
  }
}



#' Get Results for Simulation of a Basket Trial with a Limited Calibrated Power
#' Prior Design
#'
#' @template design_cpplim
#' @template n
#' @template p1
#' @template lambda
#' @template tuning_cpp
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.

#' @export
#'
#' @examples
#' design <- setup_cpplim(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
get_results.cpplim <- function(design, n, p1 = NULL, lambda, tune_a, tune_b,
                               iter = 1000, data = NULL, ...) {

  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)

  weights <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  alpha_0 <- get_alpha_0_app(design = design, n = n)

  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
                            iter = iter)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_cpplim(design = design, n = n, r = data[i, ], lambda = lambda,
            weights = weights, alpha_0 = alpha_0)
  }


}



#' Get Results for Simulation of a Basket Trial with Adaptive Power Prior Design
#'
#' @template design_app
#' @template n
#' @template p1
#' @template lambda
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A matrix of results with \code{iter} rows. A 0 means, that the
#' null hypothesis that the response probability exceeds \code{p0} was not
#' rejected, a 1 means, that the null hypothesis was rejected.
#' @export
#'
#' @examples
#' design <- setup_app(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, iter = 100)
#'
#' # Unequal sample sizes
#' get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, iter = 100)
get_results.app <- function(design, n, p1 = NULL, lambda,
                            iter = 1000, data = NULL, ...) {

  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(data = data, design = design, n = n, p = p1,
                            iter = iter)

  alpha_0 <- get_alpha_0_app(design = design, n = n)

  foreach::foreach(i = 1:nrow(data), .combine = 'rbind') %dofuture% {
    ana_app(design = design, n = n, r = data[i, ], lambda = lambda,
            alpha_0 = alpha_0)
  }



}

#' Plot a Bayesian basket trial's prior distribution
#'
#' @template design
#' @template dotdotdot_geom
#'
#' @return A list of ggplot layers of type `geom_function`.
#' @inherit geom_prior.fujikawa examples
#'
#' @export
geom_prior <- function(design, ...) {
  UseMethod("geom_prior")
}
#' Plot a Fujikawa basket trial's prior distribution
#'
#' @template dotdotdot_geom
#' @template design
#' @inherit geom_prior return
#'
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#' # One facet per basket
#' library(ggplot2)
#' ggplot() +
#'     geom_prior(design) +
#'     facet_wrap(vars(basket))
#' # Colour different baskets
#' ggplot() +
#'     geom_prior(design, aes(colour = basket))
geom_prior.fujikawa <- function(design, ...) {
  purrr::pmap(data.frame(basket = (1:design$k)),
              function(basket) {
                ggplot2::geom_function(
                  data = data.frame(basket = basket),
                  fun = stats::dbeta,
                  args = list(shape1 = design$shape1,
                              shape2 = design$shape2),
                  ...
                )
              })
}
#' Plot a Bayesian basket trial's posterior distribution
#'
#' @template design
#' @template dotdotdot_geom
#'
#' @return A list of ggplot layers of type `geom_function`.
#' @inherit geom_posterior.fujikawa examples
#'
#' @export
geom_posterior <- function(design, ...) {
  UseMethod("geom_posterior")
}
#' Plot a Fujikawa basket trial's posterior distribution
#'
#' @template dotdotdot_geom
#' @template design
#' @template n
#' @template r
#'
#' @inherit geom_posterior return
#'
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#' n <- 20
#' r <- c(4, 5, 2)
#' # One facet per basket
#' library(ggplot2)
#' ggplot() +
#'     geom_posterior(design, n, r) +
#'     facet_wrap(vars(basket))
#' # Colour different baskets
#' ggplot() +
#'     geom_posterior(design, n, r, aes(colour = basket))
geom_posterior.fujikawa <- function(design, n, r, ...) {
  purrr::pmap(cbind(t(beta_post(design, n, r)),
                    data.frame(basket = (1:design$k))),
              function(shape1post, shape2post, basket) {
                ggplot2::geom_function(
                  data = data.frame(basket = factor(basket, levels = (1:design$k))),
                  fun = stats::dbeta,
                  args = list(shape1 = shape1post,
                              shape2 = shape2post),
                  ...
                )
              })
}
#' Plot a Bayesian basket trial's posterior distribution after borrowing
#'
#' @inherit geom_posterior params return
#' @inherit geom_borrow.fujikawa examples
#'
#' @export
geom_borrow <- function(design, ...) {
  UseMethod("geom_borrow")
}
#' Plot a Fujikawa basket trial's posterior distribution after borrowing
#'
#' @template dotdotdot_geom
#' @template design
#' @template n
#' @template r
#' @template tuning_fujikawa
#' @inherit geom_borrow  return
#'
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#' n <- 20
#' r <- c(4, 5, 2)
#' epsilon <- 2
#' tau <- 0.5
#' # One facet per basket
#' library(ggplot2)
#' ggplot() +
#'     geom_borrow(design, n, r, epsilon, tau, logbase = exp(1)) +
#'     facet_wrap(vars(basket))
#' # Colour different baskets
#' ggplot() +
#'     geom_borrow(design, n, r, epsilon, tau,
#'                 logbase = exp(1), aes(colour = basket))
geom_borrow.fujikawa <-
  function(design, n, r, epsilon, tau, logbase, ...) {
    weights <-
      get_weights_jsd(
        design = design,
        n = n,
        epsilon = epsilon,
        tau = tau,
        logbase = logbase
      )
    purrr::pmap(cbind(t(beta_borrow_fujikawa(design, n, r, weights)),
                      data.frame(basket = (1:design$k))),
                function(shape1post, shape2post, basket) {
                  ggplot2::geom_function(
                    data = data.frame(basket = factor(basket, levels = (1:design$k))),
                    fun = stats::dbeta,
                    args = list(shape1 = shape1post,
                                shape2 = shape2post),
                    ...
                  )
    })
}

design <- setup_fujikawa(k = 3, p0 = 0.2)
n <- 20
r <- c(4, 5, 2)
test_that("geom_prior() works", {
  plot1 <- ggplot2::ggplot() +
    geom_prior(design) +
    ggplot2::facet_wrap(ggplot2::vars(basket))
  expect_s3_class(plot1$layers[[1]]$geom, "GeomFunction")
  expect_equal(plot1$layers[[1]]$data, data.frame(basket = 1))
})

test_that("geom_posterior() works", {
  plot1 <- ggplot2::ggplot() +
    geom_posterior(design,n, r) +
    ggplot2::facet_wrap(ggplot2::vars(basket))
  expect_s3_class(plot1$layers[[1]]$geom, "GeomFunction")
  expect_equal(plot1$layers[[1]]$data,
               data.frame(basket = factor(1, level = c("1", "2", "3"))))
})

test_that("geom_borrow() works", {
  plot1 <- ggplot2::ggplot() +
    geom_borrow(design, n, r,
                epsilon = 2, tau = 0.5, logbase = exp(1)) +
    ggplot2::facet_wrap(ggplot2::vars(basket))
  expect_s3_class(plot1$layers[[1]]$geom, "GeomFunction")
  expect_equal(plot1$layers[[1]]$data,
               data.frame(basket = factor(1, level = c("1", "2", "3"))))
})

val_borrow_cpp <- function(design, n, r, a, b,
                           globalweight_fun = NULL,
                           globalweight_params = list()) {
  cpp_fun <- function(x, n, a, b) {
    1 / (1 + exp(a + b * log(n^(1 / 4) * abs(x[1]/n - x[2]/n))))
  }

  shape1 <- shape2 <- numeric(length(r))
  if (!is.null(globalweight_fun)) {
    gw <- do.call(globalweight_fun, args = c(globalweight_params, n = n,
      r = list(r)))
  }

  for (i in 1:length(r)) {
    w <- sapply((1:length(r))[-i],
      function(x) cpp_fun(x = r[c(i, x)], n = n, a = a, b = b))

    if (!is.null(globalweight_fun)) {
      w <- w * gw
    }

    shape1[i] <- design$shape1 + r[i] + sum(r[-i] * w)
    shape2[i] <- design$shape2 + (n - r[i]) + sum((n - r[-i]) * w)
  }
  rbind(shape1, shape2)
}

val_borrow_jsdglobal <- function(design, n, r, epsilon, tau, logbase,
                              eps_all) {
  kl_fun <- function(x, y) {
    f <- function(z) x(z) * log(x(z) / y(z), base = logbase)
    stats::integrate(f, lower = 0, upper = 1)$value
  }

  jsd_fun <- function(sp1, sp2, n, epsilon, tau, logbase) {
    j1 <- function(x) stats::dbeta(x, shape1 = sp1[1], shape2 = sp2[1])
    j2 <- function(x) stats::dbeta(x, shape1 = sp1[2], shape2 = sp2[2])
    m <- function(x) (1 / 2) * (j1(x) + j2(x))
    jsd <- (1 / 2) * kl_fun(j1, m) + (1 / 2) * kl_fun(j2, m)
    w <- (1 - jsd)^epsilon
    ifelse(w <= tau, 0, w)
  }

  shape1 <- shape2 <- numeric(length(r))
  shape_prior1 <- design$shape1 + r
  shape_prior2 <- design$shape2 + (n - r)
  for (i in 1:length(r)) {
    w <- sapply((1:length(r))[-i],
      function(x) jsd_fun(sp1 = shape_prior1[c(i, x)],
        sp2 = shape_prior2[c(i, x)], n = n, epsilon = epsilon, tau = tau,
        logbase = logbase))
    w <- w * jsd_global4(rbind(shape_prior1, shape_prior2), epsilon = eps_all)
    shape1[i] <- shape_prior1[i] + sum(r[-i] * w)
    shape2[i] <- shape_prior2[i] + sum((n - r[-i]) * w)
  }
  rbind(shape1, shape2)
}

val_borrow_fujikawa <- function(design, n, r, epsilon, tau, logbase) {
  kl_fun <- function(x, y) {
    f <- function(z) x(z) * log(x(z) / y(z), base = logbase)
    stats::integrate(f, lower = 0, upper = 1)$value
  }

  jsd_fun <- function(sp1, sp2, n, epsilon, tau, logbase) {
    j1 <- function(x) stats::dbeta(x, shape1 = sp1[1], shape2 = sp2[1])
    j2 <- function(x) stats::dbeta(x, shape1 = sp1[2], shape2 = sp2[2])
    m <- function(x) (1 / 2) * (j1(x) + j2(x))
    jsd <- (1 / 2) * kl_fun(j1, m) + (1 / 2) * kl_fun(j2, m)
    w <- (1 - jsd)^epsilon
    ifelse(w <= tau, 0, w)
  }

  shape1 <- shape2 <- numeric(length(r))
  shape_prior1 <- design$shape1 + r
  shape_prior2 <- design$shape2 + (n - r)
  for (i in 1:length(r)) {
    w <- sapply((1:length(r))[-i],
      function(x) jsd_fun(sp1 = shape_prior1[c(i, x)],
        sp2 = shape_prior2[c(i, x)], n = n, epsilon = epsilon, tau = tau,
        logbase = logbase))
    shape1[i] <- shape_prior1[i] + sum(shape_prior1[-i] * w)
    shape2[i] <- shape_prior2[i] + sum(shape_prior2[-i] * w)
  }
  rbind(shape1, shape2)
}

jsd_global4 <- function(shape, epsilon) {
  f1 <- function(x) stats::dbeta(x, shape1 = shape[1, 1], shape2 = shape[2, 1])
  f2 <- function(x) stats::dbeta(x, shape1 = shape[1, 2], shape2 = shape[2, 2])
  f3 <- function(x) stats::dbeta(x, shape1 = shape[1, 3], shape2 = shape[2, 3])
  f4 <- function(x) stats::dbeta(x, shape1 = shape[1, 4], shape2 = shape[2, 4])
  m <- function(x) (f1(x) + f2(x) + f3(x) + f4(x)) / 4

  kl_fun <- function(x, y) {
    f <- function(z) x(z) * log(x(z) / y(z), base = 4)
    stats::integrate(f, lower = 0, upper = 1)$value
  }

  j1 <- kl_fun(f1, m)
  j2 <- kl_fun(f2, m)
  j3 <- kl_fun(f3, m)
  j4 <- kl_fun(f4, m)

  (1 - (j1 + j2 + j3 + j4) / 4)^epsilon
}

#' Get Details of a Basket Trial Simulation
#'
#' @template design
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means, mean
#' squared errors of all baskets and the family-wise error rate. For some
#' methods the mean limits of HDI intervals are also returned.
#' @export
#'
#' @examples
#' # Example for a basket trial with Fujikawa's Design
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25),
#'    p1 = c(0.2, 0.5, 0.5), lambda = 0.95, epsilon = 2,
#'    tau = 0, iter = 100)
#'
get_details <- function(design, ...) {
  UseMethod("get_details", design)
}

#' Get Details of a Basket Trial Simulation with the MML Design
#'
#' @template design_cpp
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_mml(k = 3, p0 = 0.2)
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   tune_a = 1, tune_b = 1, iter = 100)
get_details.mml <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  iter = 1000,
  data = NULL,
  ...
) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )

  targ <- design$p0 == p1
  weights <- get_weights_mml(design, n = n, ...)

  res <- foreach::foreach(i = 1:nrow(data), .combine = 'cfun1') %dofuture%
    {
      shape_loop <- beta_borrow_pp(
        design = design,
        n = n,
        r = data[i, ],
        weights = weights
      )
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }
  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]]),
    ECD = mean(rowSums(t(apply(res[[1]], 1, function(x) x != targ))))
  )
}

#' Get Details of a Basket Trial Simulation with the Global MML Design
#'
#' @template design_mmlglobal
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_mmlglobal(k = 3, p0 = 0.2)
#' get_details(design = design, n = 20, p1 = 0.5, lambda = 0.95, iter = 100)
get_details.mmlglobal <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  iter = 1000,
  data = NULL,
  ...
) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )
  targ <- design$p0 == p1

  res <- foreach::foreach(
    i = 1:nrow(data),
    .combine = 'cfun1',
    .options.future = list(seed = TRUE)
  ) %dofuture%
    {
      shape_loop <- weight_mmlglobal(design = design, n = n, r = data[i, ])
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }

  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]]),
    ECD = mean(rowSums(t(apply(res[[1]], 1, function(x) x != targ))))
  )
}

#' Get Details of a BHM Basket Trial Simulation
#'
#' @template design_bhm
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tau_bhm
#' @template iter
#' @template n_mcmc
#' @template data_bhm
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
#' \donttest{
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, iter = 100)
#' }
get_details.bhm <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  tau_scale,
  iter = 1000,
  n_mcmc = 10000,
  data = NULL,
  ...
) {
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  p1 <- check_p1(design = design, p1 = p1, data = data)

  data <- check_data_bhmbasket(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )
  targ <- design$p0 == p1

  analyses <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = data,
    evidence_levels = c(lambda, 1 - level),
    method_names = "berry",
    target_rates = rep(design$p_target, design$k),
    prior_parameters_list = bhmbasket::setPriorParametersBerry(
      mu_mean = design$mu_mean,
      mu_sd = design$mu_sd,
      tau_scale = tau_scale
    ),
    n_mcmc_iterations = n_mcmc
  ))

  br <- paste0(
    "c(",
    paste0("x[", 1:design$k, "] > ", design$p0, collapse = ", "),
    ")"
  )
  res <- bhmbasket::getGoDecisions(
    analyses_list = analyses,
    cohort_names = paste("p", 1:design$k, sep = "_"),
    evidence_levels = rep(lambda, design$k),
    boundary_rules = str2lang(br)
  )$scenario_1$decisions_list$berry[, -1]

  est <- bhmbasket::getEstimates(
    analyses,
    point_estimator = "mean",
    alpha_level = (1 - level)
  )$berry$scenario_1
  list(
    Rejection_Probabilities = unname(colMeans(res)),
    FWER = mean(apply(res, 1, function(x) any(x[targ] == 1))),
    Mean = unname(est[, "Mean"]),
    MSE = unname(est[, "MSE"]),
    Lower_CL = unname(est[, "2.5%"]),
    Upper_CL = unname(est[, "97.5%"]),
    ECD = mean(rowSums(t(apply(res, 1, function(x) x != targ))))
  )
}

#' Get Details of a Basket Trial Simulation with the EXNEX Design
#'
#' @template design_exnex
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tau_exnex
#' @template w_exnex
#' @template iter
#' @template n_mcmc
#' @template data_bhm
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' \donttest{
#' design <- setup_exnex(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
#' }
get_details.exnex <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  tau_scale,
  w_j,
  iter = 1000,
  n_mcmc = 10000,
  data = NULL,
  ...
) {
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  p1 <- check_p1(design = design, p1 = p1, data = data)

  data <- check_data_bhmbasket(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )
  targ <- design$p0 == p1

  analyses <- suppressMessages(bhmbasket::performAnalyses(
    scenario_list = data,
    evidence_levels = c(lambda, 1 - level),
    method_names = "exnex",
    prior_parameters_list = bhmbasket::setPriorParametersExNex(
      mu_mean = design$mu_mean,
      mu_sd = design$mu_sd,
      tau_scale = tau_scale,
      mu_j = rep(design$basket_mean, design$k),
      tau_j = rep(design$basket_sd, design$k),
      w_j = w_j
    ),
    n_mcmc_iterations = n_mcmc
  ))

  br <- paste0(
    "c(",
    paste0("x[", 1:design$k, "] > ", design$p0, collapse = ", "),
    ")"
  )
  res <- bhmbasket::getGoDecisions(
    analyses_list = analyses,
    cohort_names = paste("p", 1:design$k, sep = "_"),
    evidence_levels = rep(lambda, design$k),
    boundary_rules = str2lang(br)
  )$scenario_1$decisions_list$exnex[, -1]

  est <- bhmbasket::getEstimates(
    analyses,
    point_estimator = "mean",
    alpha_level = (1 - level)
  )$exnex$scenario_1

  list(
    Rejection_Probabilities = unname(colMeans(res)),
    FWER = mean(apply(res, 1, function(x) any(x[targ] == 1))),
    Mean = unname(est[, "Mean"]),
    MSE = unname(est[, "MSE"]),
    Lower_CL = unname(est[, "2.5%"]),
    Upper_CL = unname(est[, "97.5%"]),
    ECD = mean(rowSums(t(apply(res, 1, function(x) x != targ))))
  )
}

#' Get Details of a Basket Trial Simulation with Fujikawa's Design
#'
#' @template design_fujikawa
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tuning_fujikawa
#' @template iter
#' @template data
#' @template dotdotdot
#' @template use_future
#' @template weight_fun
#' @template weight_params
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate and the experiment-wise power.
#' @export
#'
#' @examples
#' design <- setup_fujikawa(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
#'
#' # A custom weight function can be defined, e.g.
#' weight_noshare <- function(design, n, epsilon, tau, logbase){
#'   n_sum <- n + 1
#'   return(diag(n_sum))
#' }
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'            epsilon = 2, tau = 0, iter = 1000, weight_fun = weight_noshare)
get_details.fujikawa <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  epsilon,
  tau,
  logbase = 2,
  iter = 1000,
  data = NULL,
  use_future = FALSE,
  weight_fun = NULL,
  weight_params = list(epsilon = epsilon, tau = tau, logbase = logbase),
  ...
) {
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  p1 <- check_p1(design = design, p1 = p1, data = data)

  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )

  targ <- design$p0 == p1
  not_targ <- design$p0 != p1
  weights <- NULL
  if (is.null(weight_fun)) {
    weights <- get_weights_jsd(
      design = design,
      n = n,
      epsilon = epsilon,
      tau = tau,
      logbase = logbase
    )
  } else {
    weights <- do.call(
      weight_fun,
      args = c(design = list(design), n = n, weight_params)
    )
  }

  if (use_future) {
    # message("Use of %dofuture% in get_details() is switched on.")
    do_fun <- doFuture::`%dofuture%`
  } else {
    # message("Use of %dofuture% in get_details() is switched off.")
    do_fun <- foreach::`%do%`
  }
  res <- do_fun(foreach::foreach(i = 1:nrow(data), .combine = 'cfun1'), {
    shape_loop <- beta_borrow_fujikawa(
      design = design,
      n = n,
      r = data[i, ],
      weights = weights
    )
    res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
    mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
    hdi_loop <- apply(shape_loop, 2, function(x) {
      HDInterval::hdi(
        stats::qbeta,
        shape1 = x[1],
        shape2 = x[2],
        credMass = level
      )
    })
    list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
  })

  res_list <- list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    EWP = mean(apply(res[[1]], 1, function(x) any(x[not_targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]]),
    ECD = mean(rowSums(t(apply(res[[1]], 1, function(x) x != targ)))),
    Rejection_Probabilities_SE = NA_real_,
    FWER_SE = NA_real_,
    EWP_SE = NA_real_,
    ECD_SE = stats::sd(rowSums(t(apply(res[[1]], 1, function(x) x != targ)))) /
      sqrt(iter)
  )
  res_list$FWER_SE <- mcse_rate(res_list$FWER, iter)
  res_list$EWP_SE <- mcse_rate(res_list$EWP, iter)
  res_list$Rejection_Probabilities_SE <-
    mcse_rate(res_list$Rejection_Probabilities, iter)
  return(res_list)
}


#' Get Details of a Basket Trial Simulation with the Power Prior Design
#' Based on Global JSD Weights
#'
#' @template design_jsdglobal
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tuning_jsdglobal
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_jsdglobal(k = 3, p0 = 0.2)
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   eps_pair = 2, eps_all = 2, iter = 100)
get_details.jsdglobal <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  eps_pair,
  tau = 0,
  eps_all,
  logbase = 2,
  iter = 1000,
  data = NULL,
  ...
) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )

  weights_pair <- get_weights_jsd(
    design = design,
    n = n,
    epsilon = eps_pair,
    tau = tau,
    logbase = logbase
  )
  targ <- design$p0 == p1

  res <- foreach::foreach(i = 1:nrow(data), .combine = 'cfun1') %dofuture%
    {
      shape_loop <- beta_borrow_jsdglobal(
        design = design,
        n = n,
        r = data[i, ],
        weights_pair = weights_pair,
        eps_all = eps_all
      )
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }

  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]]),
    ECD = mean(rowSums(t(apply(res[[1]], 1, function(x) x != targ))))
  )
}

#' Get Details of a Basket Trial Simulation with the Calibrated Power Prior
#' Design
#'
#' @template design_cpp
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tuning_cpp
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_cpp(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
get_details.cpp <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  tune_a,
  tune_b,
  iter = 1000,
  data = NULL,
  ...
) {
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  p1 <- check_p1(design = design, p1 = p1, data = data)
  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )

  targ <- design$p0 == p1
  weights <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  res <- foreach::foreach(i = 1:nrow(data), .combine = 'cfun1') %dofuture%
    {
      shape_loop <- beta_borrow_pp(
        design = design,
        n = n,
        r = data[i, ],
        weights = weights
      )
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }
  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]]),
    ECD = mean(rowSums(t(apply(res[[1]], 1, function(x) x != targ))))
  )
}

#' Get Details of a Basket Trial Simulation with the Global Calibrated
#' Power Prior Design
#'
#' @template design_cppglobal
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tuning_cppglobal
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_cppglobal(k = 3, p0 = 0.2)
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
#'   tune_a = 1, tune_b = 1, epsilon = 2, iter = 100)
get_details.cppglobal <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  tune_a,
  tune_b,
  epsilon,
  iter = 1000,
  data = NULL,
  ...
) {
  p1 <- check_p1(design = design, p1 = p1, data = data)
  check_params(n = n, lambda = lambda, iter = iter)
  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )

  targ <- design$p0 == p1
  weights_pair <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  res <- foreach::foreach(i = 1:nrow(data), .combine = 'cfun1') %dofuture%
    {
      shape_loop <- beta_borrow_cppglobal(
        design = design,
        n = n,
        r = data[i, ],
        weights_pair = weights_pair,
        epsilon = epsilon
      )
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }
  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]]),
    ECD = mean(rowSums(t(apply(res[[1]], 1, function(x) x != targ))))
  )
}


#' Get Details of a Basket Trial Simulation with the Limited Calibrated Power
#' Prior Design
#'
#' @template design_cpplim
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template tuning_cpp
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_cpplim(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'   lambda = 0.95, tune_a = 1, tune_b = 1, iter = 100)
get_details.cpplim <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  tune_a,
  tune_b,
  iter = 1000,
  data = NULL,
  ...
) {
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  p1 <- check_p1(design = design, p1 = p1, data = data)
  targ <- design$p0 == p1

  weights <- get_weights_cpp(n = n, tune_a = tune_a, tune_b = tune_b)

  alpha_0 <- get_alpha_0_app(design = design, n = n)

  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )

  res <- foreach::foreach(i = 1:nrow(data), .combine = 'cfun1') %dofuture%
    {
      shape_loop <- beta_borrow_cpplim(
        design = design,
        n = n,
        r = data[i, ],
        weights = weights,
        alpha_0 = alpha_0
      )
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }
  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]])
  )
}

#' Get Details of a Basket Trial Simulation with the Adaptive Power Prior Design
#' for sequential clinical trials
#'
#' @template design_app
#' @template n
#' @template p1
#' @template lambda
#' @template level
#' @template iter
#' @template data
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, posterior means,
#' mean squared errors and mean limits of HDI intervals for all baskets as well
#' as the family-wise error rate.
#' @export
#'
#' @examples
#' design <- setup_app(k = 3, p0 = 0.2)
#'
#' # Equal sample sizes
#' get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
#'  lambda = 0.95, iter = 100)
#'
#' # Unequal sample sizes
#' get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
#'  lambda = 0.95, iter = 100)
get_details.app <- function(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  iter = 1000,
  data = NULL,
  ...
) {
  check_params_differentn(design = design, n = n, lambda = lambda, iter = iter)
  p1 <- check_p1(design = design, p1 = p1, data = data)
  data <- check_data_matrix(
    data = data,
    design = design,
    n = n,
    p = p1,
    iter = iter
  )
  targ <- design$p0 == p1

  alpha_0 <- get_alpha_0_app(design = design, n = n)

  res <- foreach::foreach(i = 1:nrow(data), .combine = 'cfun1') %dofuture%
    {
      shape_loop <- beta_borrow_app(
        design = design,
        n = n,
        r = data[i, ],
        alpha_0 = alpha_0
      )
      res_loop <- ifelse(post_beta(shape_loop, design$p0) >= lambda, 1, 0)
      mean_loop <- apply(shape_loop, 2, function(x) x[1] / (x[1] + x[2]))
      hdi_loop <- apply(shape_loop, 2, function(x) {
        HDInterval::hdi(
          stats::qbeta,
          shape1 = x[1],
          shape2 = x[2],
          credMass = level
        )
      })
      list(res_loop, mean_loop, hdi_loop[1, ], hdi_loop[2, ])
    }
  list(
    Rejection_Probabilities = colMeans(res[[1]]),
    FWER = mean(apply(res[[1]], 1, function(x) any(x[targ] == 1))),
    Mean = colMeans(res[[2]]),
    MSE = colMeans(t(t(res[[2]]) - p1)^2),
    Lower_CL = colMeans(res[[3]]),
    Upper_CL = colMeans(res[[4]])
  )
}

#' Get Details of a Basket Trial with the Frequentist Binomial Design
#'
#' This basic frequentist design conducts a separate binomial test for each
#' basket. All tests are one-sided, and the alternative is greater than the
#' null hypothesis. These details are calculated exactly, not simulated.
#'
#' @template design_binomial
#' @template n
#' @template p1
#' @template alpha
#' @template dotdotdot
#'
#' @return A list containing the rejection probabilities, critical values
#' and expected number of correct decisions. Critical values \eqn{c} are defined
#' so that the null hypothesis is rejected if the observed number of responses
#' \eqn{r} is greater than \eqn{c}, i.e. \eqn{r > c} rejects \eqn{H_0}.
#' The nominal FWER is the (theoretical) FWER of a multiple testing problem
#' with k hypothesis tests at significance level \eqn{\alpha}. The actual FWER is
#' usually lower than the nominal FWER, as the binomial test does not exhaust
#' its significance level.
#' @export
#'
#' @examples
#' design <- setup_binomial(k = 3, p0 = 0.2)
#' p1 <- c(0.2, 0.5, 0.5)
#' get_details(design = design, n = 20, p1 = p1)
get_details.binomial <- function(
  design,
  n,
  p1 = NULL,
  alpha = 0.025,
  ...
) {
  p1 <- check_p1(design = design, p1 = p1, data = NULL)
  check_params_differentn_frequentist(
    design = design,
    n = n,
    alpha = alpha
  )
  if (length(n) == 1) {
    n <- c(rep(n, design$k))
  }
  if (design$pool) {
    stop("The pool design is not yet implemented.")
  } else {
    Critical_Values <- stats::qbinom(
      p = alpha,
      size = n,
      prob = design$p0,
      lower.tail = F
    )
    Rejection_Probabilities <- mapply(
      FUN = function(q, n, p1) {
        stats::pbinom(
          q = q,
          size = n,
          prob = p1,
          lower.tail = F
        )
      },
      q = Critical_Values,
      n = n,
      p1 = p1
    )

    targ <- design$p0 == p1

    res_list <- list(
      Rejection_Probabilities = Rejection_Probabilities,
      Critical_Values = Critical_Values,
      FWER = 1 - prod(1 - Rejection_Probabilities[targ]),
      FWER_nominal = 1 - (1 - alpha)^sum(targ),
      EWP = 1 - prod(1 - Rejection_Probabilities[!targ]),
      ECD = sum(1 - Rejection_Probabilities[targ]) +
        sum(Rejection_Probabilities[!targ])
    )
  }
  return(res_list)
}

ana_mmlglobal <- function(design, n, r, lambda) {
  shape_post <- weight_mmlglobal(design = design, n = n, r = r)
  post_prob <- post_beta(shape = shape_post, p0 = design$p0)
  ifelse(post_prob >= lambda, 1, 0)
}

weight_mmlglobal <- function(design, n, r) {
  shape1_post <- shape2_post <- numeric(design$k)

  for (i in 1:design$k) {
    f <- function(delta) -extraDistr::dbbinom(
      x = r[i],
      size = n,
      alpha = design$shape1 + sum(delta * r[-i]),
      beta = design$shape2 + sum(delta * (n - r[-i]))
    )

    l <- stats::optim(rep(0.5, design$k - 1), fn = f,
      lower = rep(0, design$k - 1), upper = rep(1, design$k - 1),
      method = "L-BFGS-B")$par

    shape1_post[i] <- design$shape1 + r[i] + sum(l * r[-i])
    shape2_post[i] <- design$shape2 + (n - r[i]) + sum(l * (n - r[-i]))
  }

  matrix(c(shape1_post, shape2_post), nrow = 2, byrow = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_results.R
\name{get_results.app}
\alias{get_results.app}
\title{Get Results for Simulation of a Basket Trial with Adaptive Power Prior Design}
\usage{
\method{get_results}{app}(design, n, p1 = NULL, lambda, iter = 1000, data = NULL, ...)
}
\arguments{
\item{design}{An object of class \code{app}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities used for the simulation. If \code{NULL}
then all probabilities are set to p0.}

\item{lambda}{The posterior probability threshold.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{data}{A data matrix with k column with the number of responses for each
basket. Has to be generated with \code{get_data}. If \code{data} is used,
then \code{iter} is ignored.}

\item{...}{Further arguments.}
}
\value{
A matrix of results with \code{iter} rows. A 0 means, that the
null hypothesis that the response probability exceeds \code{p0} was not
rejected, a 1 means, that the null hypothesis was rejected.
}
\description{
Get Results for Simulation of a Basket Trial with Adaptive Power Prior Design
}
\examples{
design <- setup_app(k = 3, p0 = 0.2)

# Equal sample sizes
get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, iter = 100)

# Unequal sample sizes
get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, iter = 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/operating_characteristics.R
\name{toer}
\alias{toer}
\title{Calculate the Type 1 Error Rate for a Basket Trial Design}
\usage{
toer(
  design,
  n,
  p1 = NULL,
  lambda,
  design_params = list(),
  iter = 1000,
  data = NULL,
  ...
)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities under the alternative hypothesis. If \code{NULL}
then the type 1 error rate under the global null hypothesis is
calculated.}

\item{lambda}{The posterior probability threshold.}

\item{design_params}{A list of params that is specific to the class of
\code{design}.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{data}{A data matrix with k column with the number of responses for each
basket. Has to be generated with \code{get_data}. If \code{data} is used,
then \code{iter} is ignored.}

\item{...}{Further arguments.}
}
\value{
A numeric value.
}
\description{
Calculate the Type 1 Error Rate for a Basket Trial Design
}
\examples{
# Example for a basket trial with Fujikawa's Design
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
toer(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, design_params = list(epsilon = 2, tau = 0),
  iter = 1000)

# Unequal sample sizes
toer(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, design_params = list(epsilon = 2, tau = 0),
  iter = 1000)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_lambda.R
\name{adjust_lambda.default}
\alias{adjust_lambda.default}
\title{Adjust Lambda}
\usage{
\method{adjust_lambda}{default}(
  design,
  n,
  p1 = NULL,
  alpha = 0.05,
  design_params = list(),
  iter = 1000,
  prec_digits = 3,
  data = NULL,
  ...
)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities under the alternative hypothesis. If \code{NULL}
then the type 1 error rate under the global null hypothesis is
calculated.}

\item{alpha}{The one-sided significance level.}

\item{design_params}{A list of params that is specific to the class of
\code{design}.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{prec_digits}{Number of decimal places that are considered when
adjusting lambda.}

\item{data}{A data matrix with k column with the number of responses for each
basket. Has to be generated with \code{get_data}. If \code{data} is used,
then \code{iter} is ignored.}

\item{...}{Further arguments.}
}
\value{
A list containing the greatest estimated value for \code{lambda} with
\code{prec_digits} decimal places which controls the family wise error rate
at level \code{alpha} (one-sided) and the estimated family wise error rate
for the estimated \code{lambda}.
}
\description{
Adjust Lambda
}
\details{
It is recommended to use \code{data} and then use the same simulated
data set for all further calculations. If \code{data = NULL} then
new data are generated in each step of the algorithm, so \code{lambda} doesn't
necessarily protect the family wise error rate for different simulated data
due to Monte Carlo simulation error.
}
\examples{
\donttest{
# Example for a basket trial with Fujikawa's Design
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
adjust_lambda(design = design, n = 20, alpha = 0.05,
  design_params = list(epsilon = 2, tau = 0), iter = 1000)

# Unequal sample sizes
adjust_lambda(design = design, n = c(15, 20, 25), alpha = 0.05,
  design_params = list(epsilon = 2, tau = 0), iter = 1000)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basksim-package.R
\docType{package}
\name{basksim-package}
\alias{basksim}
\alias{basksim-package}
\title{basksim: Simulation-Based Calculation of Basket Trial Operating Characteristics}
\description{
Provides a unified syntax for the simulation-based comparison of different single-stage basket trial designs with a binary endpoint and equal sample sizes in all baskets. Methods include the designs by Baumann et al. (2025) \doi{10.1080/19466315.2024.2402275}, Schmitt and Baumann (2025) \doi{10.1080/19466315.2025.2486231}, Fujikawa et al. (2020) \doi{10.1002/bimj.201800404}, Berry et al. (2020) \doi{10.1177/1740774513497539}, and Neuenschwander et al. (2016) \doi{10.1002/pst.1730}. For the latter two designs, the functions are mostly wrappers for functions provided by the package 'bhmbasket'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/lbau7/basksim}
  \item Report bugs at \url{https://github.com/lbau7/basksim/issues}
}

}
\author{
\strong{Maintainer}: Lukas Baumann \email{baumann@imbi.uni-heidelberg.de} (\href{https://orcid.org/0000-0001-7931-7470}{ORCID})

Authors:
\itemize{
  \item Lukas D Sauer \email{sauer@imbi.uni-heidelberg.de} (\href{https://orcid.org/0000-0002-1340-9994}{ORCID})
  \item Sabrina Schmitt \email{sabrina.schmitt@uni-wuerzburg.de} (\href{https://orcid.org/0009-0008-2719-6513}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt_design.R
\name{opt_design}
\alias{opt_design}
\title{Optimize a Basket Trial Design}
\usage{
opt_design(
  design,
  n,
  alpha,
  design_params = list(),
  scenarios,
  prec_digits,
  iter = 1000,
  data = NULL,
  ...
)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{alpha}{The one-sided significance level.}

\item{design_params}{A list of params that is specific to the class of
\code{design}.}

\item{scenarios}{A matrix of scenarios.}

\item{prec_digits}{Number of decimal places that are considered when
adjusting lambda.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{data}{A list of data matrices generated with \code{get_data}. The
list elements have to correspond to the columsn of \code{scenarios}.}

\item{...}{Further arguments.}
}
\value{
A matrix with the expected number of correct decisions.
}
\description{
Optimize a Basket Trial Design
}
\examples{
design <- setup_fujikawa(k = 3, p0 = 0.2)
scenarios <- get_scenarios(design, p1 = 0.5)

## Equal sample sizes
# Without simulated data
\donttest{
opt_design(design, n = 20, alpha = 0.05, design_params =
           list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
           prec_digits = 3)
}

# With simulated data
scenario_list <- as.list(data.frame(scenarios))
data_list <- lapply(scenario_list,
  function(x) get_data(k = 3, n = 20, p = x, iter = 1000))
\donttest{
opt_design(design, n = 20, alpha = 0.05, design_params =
           list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
           prec_digits = 3, data = data_list)


## Unequal sample sizes
# Without simulated data
opt_design(design, n = c(15, 20, 25), alpha = 0.05, design_params =
           list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
           prec_digits = 3)

# With simulated data
scenario_list <- as.list(data.frame(scenarios))
data_list <- lapply(scenario_list,
                    function(x) get_data(k = 3, n = c(15, 20, 25),
                                         p = x, iter = 1000))
opt_design(design, n = c(15, 20, 25), alpha = 0.05, design_params =
           list(epsilon = c(1, 2), tau = c(0, 0.5)), scenarios = scenarios,
           prec_digits = 3, data = data_list)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_app}
\alias{setup_app}
\title{Set Up Adaptive Power Prior Design Object}
\usage{
setup_app(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{app}
}
\description{
Set Up Adaptive Power Prior Design Object
}
\details{
The class \code{app} implements the adaptive power prior design for
sequential clinical trials proposed by Ollier et al. (2020).
}
\examples{
design_app <- setup_app(k = 3, p0 = 0.2)
}
\references{
Ollier, A., Morita, S., Ursino, M., & Zohar, S. (2020). An
adaptive power prior for sequential clinical trials - Application to bridging
studies. Statistical methods in medical research, 29(8), 2282–2294.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_data.R
\name{get_data}
\alias{get_data}
\title{Simulate Data Based on a Binomial Distribution}
\usage{
get_data(k, n, p, iter, type = c("matrix", "bhmbasket"))
}
\arguments{
\item{k}{The number of baskets.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p}{Probabilities used to simulate the data}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{type}{Type of output. Use \code{bhmbasket} for the BHM and EXNED
design and \code{matrix} for everything else.}
}
\value{
If \code{type = "matrix"} then a matrix is returned, if
\code{type = "bhmbasket"} then an element with class \code{scenario_list}.
}
\description{
Simulate Data Based on a Binomial Distribution
}
\details{
For \code{type = "bhmbasket"} this is simply a wraper for
\code{bhmbasket::simulateScenarios}.
}
\examples{
# Equal sample sizes
get_data(k = 3, n = 20, p = c(0.2, 0.2, 0.5), iter = 1000,
  type = "matrix")

# Unequal sample sizes
get_data(k = 3, n = c(15, 20, 25), p = c(0.2, 0.2, 0.5),
  iter = 1000, type = "matrix")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_mmlglobal}
\alias{setup_mmlglobal}
\title{Set Up mmlglobal Design Object}
\usage{
setup_mmlglobal(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{mmlglobal}
}
\description{
Creates an object of class \code{mmlglobal}.
}
\details{
The class \code{mmlglobal} implements an empirical Bayes method
by Gravestock & Held (2019) which was proposed for borrowing strength
from multiple external studies.
}
\examples{
design_mmlglobal <- setup_mmlglobal(k = 3, p0 = 0.2)
}
\references{
Gravestock, I., & Held, L. (2019). Power priors based on
multiple historical studies for binary outcomes. Biometrical Journal, 61(5),
1201-1218.

Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
Design Based on Power Priors. Statistics in Biopharmaceutical Research,
17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_cppglobal}
\alias{setup_cppglobal}
\title{Set Up Global Calibrated Power Prior Design Object}
\usage{
setup_cppglobal(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{cppglobal}
}
\description{
Set Up Global Calibrated Power Prior Design Object
}
\details{
The class \code{cppglobal} implements a version of the power prior
design, in which the amount of information that is shared between baskets is
determined by the Kolmogorov-Smirnov test statistic between basekts and
a function based on response rate differences that quantifies the
overall heterogeneity.
}
\examples{
design_cppglobal <- setup_cppglobal(k = 3, p0 = 0.2)
}
\references{
Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
Design Based on Power Priors. Statistics in Biopharmaceutical Research,
17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_mml}
\alias{setup_mml}
\title{Set Up mml Design Object}
\usage{
setup_mml(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{mml}
}
\description{
Creates an object of class \code{mml}.
}
\details{
The class \code{mml} implements a modified version of the
empirical Bayes method by Gravestock & Held (2017) which was proposed for
borrowing strength from an external study. In their approach, the sharing
weight is found as the maximum of the marginal likelihood of the
weight, given the external data set. This leads, however, to
non-symmetric weights when applied to sharing in basket trials, i.e.
Basket i would not share the information from Basket j as the other way
round. Therefore, a symmetrised version is used, where the mean of the
two weights resulting from sharing in both directions is used.
}
\examples{
design_mml <- setup_mml(k = 3, p0 = 0.2)
}
\references{
Gravestock, I., & Held, L. (2017). Adaptive power priors with
empirical Bayes for clinical trials. Pharmaceutical statistics, 16(5),
349-360.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_binomial}
\alias{setup_binomial}
\title{Set Up Frequentist Binomial Design Object}
\usage{
setup_binomial(k, p0, pool = FALSE)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{pool}{A logical, should data from all baskets be pooled? Default is
\code{FALSE}. Currently \code{FALSE} is the only working option, reserved for
future implementation.}
}
\value{
An S3 object of class \code{binomial}
}
\description{
Set Up Frequentist Binomial Design Object
}
\details{
The class \code{binomial} implements a basket trial
design, in which each null hypothesis is tested using the
frequentist binomial test without multiplicity correction. All baskets are
either tested separately (the default) or pooled (not implemented yet).
}
\examples{
design_binomial <- setup_binomial(k = 3, p0 = 0.2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_evaluation.R
\name{get_evaluation.fujikawa}
\alias{get_evaluation.fujikawa}
\title{Evaluate a Basket Trial with Fujikawa's Design}
\usage{
\method{get_evaluation}{fujikawa}(
  design,
  n,
  r,
  lambda,
  level = 0.95,
  epsilon,
  tau,
  logbase = 2,
  ...
)
}
\arguments{
\item{design}{An object of class \code{fujikawa}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{r}{Vector of responses.}

\item{lambda}{The posterior probability threshold.}

\item{level}{Level of the credibility intervals.}

\item{epsilon}{Tuning parameter that determines the amount of borrowing.
See \code{\link{setup_fujikawa}}).}

\item{tau}{Tuning parameter that determines how similar the baskets
have to be that information is shared. See \code{\link{setup_fujikawa}}).}

\item{logbase}{Tuning parameter. The base of the logarithm that is used to
calculate the Jensen-Shannon divergence.}

\item{...}{Further arguments.}
}
\value{
A list containing the point estimates of the basket-specific response rates and the
posterior probabilities that the estimated response rates are above a specified threshold p0.
}
\description{
Evaluate a Basket Trial with Fujikawa's Design
}
\examples{
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
get_evaluation(design = design, n = 20, r = c(10, 15, 5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)

# Unequal sample sizes
get_evaluation(design = design, n = c(15, 20, 25),
   r = c(10, 15, 17), lambda = 0.95, epsilon = 2,
   tau = 0, iter = 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_fujikawa}
\alias{setup_fujikawa}
\title{Set Up Fujikawa Design Object}
\usage{
setup_fujikawa(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{fujikawa}
}
\description{
Set Up Fujikawa Design Object
}
\details{
The class \code{fujikawa} implements a design by Fujikawa et al.
(2020) in which information is shared based on the pairwise similarity
between baskets which is quantified using the Jensen-Shannon divergence
between the individual posterior distributions between baskets.
}
\examples{
design_fujikawa <- setup_fujikawa(k = 3, p0 = 0.2)
}
\references{
Fujikawa, K., Teramukai, S., Yokota, I., & Daimon, T. (2020).
A Bayesian basket trial design that borrows information across strata based
on the similarity between the posterior distributions of the response
probability. Biometrical Journal, 62(2), 330-338.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_results.R
\name{get_results.bhm}
\alias{get_results.bhm}
\title{Get Results for Simulation of a Basket Trial with the BHM Design}
\usage{
\method{get_results}{bhm}(
  design,
  n,
  p1 = NULL,
  lambda,
  tau_scale,
  iter = 1000,
  n_mcmc = 10000,
  data = NULL,
  ...
)
}
\arguments{
\item{design}{An object of class \code{bhm}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities used for the simulation. If \code{NULL}
then all probabilities are set to p0.}

\item{lambda}{The posterior probability threshold.}

\item{tau_scale}{Standard deviation of the half normal prior distribution
for the variance of the thetas.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{n_mcmc}{Number of MCMC samples.}

\item{data}{An object of class \code{scenario_list} as returned by the
function \code{bhmbasket::simulateScenarios}.}

\item{...}{Further arguments.}
}
\value{
A matrix of results with \code{iter} rows. A 0 means, that the
null hypothesis that the response probability exceeds \code{p0} was not
rejected, a 1 means, that the null hypothesis was rejected.
}
\description{
Get Results for Simulation of a Basket Trial with the BHM Design
}
\examples{
\donttest{
design <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)

# Equal sample sizes
get_results(design, n = 20, p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, tau_scale = 1, iter = 100)

# Unequal sample sizes
get_results(design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, tau_scale = 1, iter = 100)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_evaluation.R
\name{get_evaluation}
\alias{get_evaluation}
\title{Evaluate a Basket Trial}
\usage{
get_evaluation(design, ...)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{...}{Further arguments.}
}
\value{
A list containing the point estimates of the basket-specific response rates and,
for some methods, the posterior probabilities that the estimated response rates are above
a specified threshold p0.
}
\description{
Evaluate a Basket Trial
}
\examples{
# Example for a basket trial with Fujikawa's Design
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
get_evaluation(design = design, n = 20, r = c(10, 15, 5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)

# Unequal sample sizes
get_evaluation(design = design, n = c(15, 20, 25),
   r = c(10, 15, 17), lambda = 0.95, epsilon = 2,
   tau = 0, iter = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_details.R
\name{get_details}
\alias{get_details}
\title{Get Details of a Basket Trial Simulation}
\usage{
get_details(design, ...)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{...}{Further arguments.}
}
\value{
A list containing the rejection probabilities, posterior means, mean
squared errors of all baskets and the family-wise error rate. For some
methods the mean limits of HDI intervals are also returned.
}
\description{
Get Details of a Basket Trial Simulation
}
\examples{
# Example for a basket trial with Fujikawa's Design
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)

# Unequal sample sizes
get_details(design = design, n = c(15, 20, 25),
   p1 = c(0.2, 0.5, 0.5), lambda = 0.95, epsilon = 2,
   tau = 0, iter = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_cpp}
\alias{setup_cpp}
\title{Set Up Calibrated Power Prior Design Object}
\usage{
setup_cpp(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{cpp}
}
\description{
Set Up Calibrated Power Prior Design Object
}
\details{
The class \code{cpp} implements a version of the power prior design,
in which the amount of information that is shared between baskets is
determined by the Kolmogorov-Smirnov test statistic between baskets (which
is equivalent to the absolut difference in response rates).
}
\examples{
design_cpp <- setup_cpp(k = 3, p0 = 0.2)
}
\references{
Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
Design Based on Power Priors. Statistics in Biopharmaceutical Research,
17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geoms.R
\name{geom_borrow}
\alias{geom_borrow}
\title{Plot a Bayesian basket trial's posterior distribution after borrowing}
\usage{
geom_borrow(design, ...)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{...}{Further arguments to be passed to `geom_function`.}
}
\value{
A list of ggplot layers of type `geom_function`.
}
\description{
Plot a Bayesian basket trial's posterior distribution after borrowing
}
\examples{
# Example for a basket trial with Fujikawa's Design
design <- setup_fujikawa(k = 3, p0 = 0.2)
n <- 20
r <- c(4, 5, 2)
epsilon <- 2
tau <- 0.5
# One facet per basket
library(ggplot2)
ggplot() +
    geom_borrow(design, n, r, epsilon, tau, logbase = exp(1)) +
    facet_wrap(vars(basket))
# Colour different baskets
ggplot() +
    geom_borrow(design, n, r, epsilon, tau,
                logbase = exp(1), aes(colour = basket))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_evaluation.R
\name{get_evaluation.app}
\alias{get_evaluation.app}
\title{Evaluate a Basket Trial with the Adaptive Power Prior Design
for sequential clinical trials}
\usage{
\method{get_evaluation}{app}(design, n, r, lambda, level = 0.95, ...)
}
\arguments{
\item{design}{An object of class \code{app}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{r}{Vector of responses.}

\item{lambda}{The posterior probability threshold.}

\item{level}{Level of the credibility intervals.}

\item{...}{Further arguments.}
}
\value{
A list containing the point estimates of the basket-specific response rates and the
posterior probabilities that the estimated response rates are above a specified threshold p0.
}
\description{
Evaluate a Basket Trial with the Adaptive Power Prior Design
for sequential clinical trials
}
\examples{
design <- setup_app(k = 3, p0 = 0.2)

# Equal sample sizes
get_evaluation(design = design, n = 20, r = c(10, 15, 5),
 lambda = 0.95, iter = 100)

# Unequal sample sizes
get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
 lambda = 0.95, iter = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_details.R
\name{get_details.jsdglobal}
\alias{get_details.jsdglobal}
\title{Get Details of a Basket Trial Simulation with the Power Prior Design
Based on Global JSD Weights}
\usage{
\method{get_details}{jsdglobal}(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  eps_pair,
  tau = 0,
  eps_all,
  logbase = 2,
  iter = 1000,
  data = NULL,
  ...
)
}
\arguments{
\item{design}{An object of class \code{jsdgen}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities used for the simulation. If \code{NULL}
then all probabilities are set to p0.}

\item{lambda}{The posterior probability threshold.}

\item{level}{Level of the credibility intervals.}

\item{eps_pair}{Tuning parameter that determines the amount of borrowing
based on pairwise similarity.}

\item{tau}{Tuning parameter that determines how similar the baskets
have to be that information is shared.}

\item{eps_all}{Tuning parameter that determines the amount of borrowing
based on overall heterogeneity.}

\item{logbase}{Tuning parameter. The base of the logarithm that is used to
calculate the Jensen-Shannon divergence.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{data}{A data matrix with k column with the number of responses for each
basket. Has to be generated with \code{get_data}. If \code{data} is used,
then \code{iter} is ignored.}

\item{...}{Further arguments.}
}
\value{
A list containing the rejection probabilities, posterior means,
mean squared errors and mean limits of HDI intervals for all baskets as well
as the family-wise error rate.
}
\description{
Get Details of a Basket Trial Simulation with the Power Prior Design
Based on Global JSD Weights
}
\examples{
design <- setup_jsdglobal(k = 3, p0 = 0.2)
get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
  eps_pair = 2, eps_all = 2, iter = 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjust_lambda.R
\name{adjust_lambda}
\alias{adjust_lambda}
\title{Adjust Lambda}
\usage{
adjust_lambda(design, ...)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{...}{Further arguments.}
}
\value{
A list containing the greatest estimated value for \code{lambda} with
\code{prec_digits} decimal places which controls the family wise error rate
at level \code{alpha} (one-sided) and the estimated family wise error rate
for the estimated \code{lambda}.
}
\description{
Adjust Lambda
}
\details{
The default method for \code{adjust_lambda} uses a combination
of \code{\link{uniroot}} and grid search and calls \code{\link{toer}}
in every iteration. For methods implemented in the \code{bhmbasket} package
there are separate methods that are computationally more efficient.
}
\examples{
\donttest{
design <- setup_cpp(k = 3, p0 = 0.2)

# Equal sample sizes
adjust_lambda(design = design, n = 20, alpha = 0.05,
  design_params = list(tune_a = 1, tune_b = 1), iter = 1000)

# Unequal sample sizes
adjust_lambda(design = design, n = c(15, 20, 25), alpha = 0.05,
   design_params = list(tune_a = 1, tune_b = 1), iter = 1000)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_exnex}
\alias{setup_exnex}
\title{Set Up EXNEX Design Object}
\usage{
setup_exnex(
  k,
  p0,
  basket_mean = NULL,
  basket_sd = 100,
  mu_mean = NULL,
  mu_sd = 100
)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{basket_mean}{Mean of the normal prior distribution of the individual
thetas (NEX part). See details.}

\item{basket_sd}{Standard deviation of the normal prior distribution of the
individual thetas (NEX part).}

\item{mu_mean}{Mean of the normal prior exchangeability distribution for
the mean of the thetas (EX part). See details.}

\item{mu_sd}{Standard deviation of the normal prior exchangeability
distribution for the mean of the thetas (EX part).}
}
\value{
An S3 object of class \code{exnex}
}
\description{
Set Up EXNEX Design Object
}
\details{
The class \code{exnex} implements the EXNEX model proposed by
Neuenschwander et al. (2016). Methods for this class are mostly wrappers
for functions from the package \code{bhmbasket}.

In the EXNEX model the thetas of all baskets are modeled as a mixture
of individual models and a Bayesian Hierarchical Model with a fixed
mixture weight w. If \code{mu_mean} and \code{basket_mean} are \code{NULL}
then they are set to logit(p0).
Note that Neuenschwander et al. (2016) use different prior means and
standard deviations. The default values here are used for better comparison
with the BHM model (see \code{\link{setup_bhm}}).
}
\examples{
design_exnex <- setup_exnex(k = 3, p0 = 0.2)
}
\references{
Neuenschwander, B., Wandel, S., Roychoudhury, S., & Bailey, S.
(2016). Robust exchangeability designs for early phase clinical trials with
multiple strata. Pharmaceutical statistics, 15(2), 123-134.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_evaluation.R
\name{get_evaluation.exnex}
\alias{get_evaluation.exnex}
\title{Evaluate a Basket Trial with the EXNEX Design}
\usage{
\method{get_evaluation}{exnex}(
  design,
  n,
  r,
  lambda,
  level = 0.95,
  tau_scale,
  w_j,
  n_mcmc = 10000,
  ...
)
}
\arguments{
\item{design}{An object of class \code{exnex}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{r}{Vector of responses.}

\item{lambda}{The posterior probability threshold.}

\item{level}{Level of the credibility intervals.}

\item{tau_scale}{Standard deviation of the half normal prior exchangeability
distribution for the variance of the thetas.}

\item{w_j}{Fixed prior weight for the exchangeability part of the model.}

\item{n_mcmc}{Number of MCMC samples.}

\item{...}{Further arguments.}
}
\value{
A list containing the point estimates of the basket-specific response rates.
}
\description{
Evaluate a Basket Trial with the EXNEX Design
}
\examples{
design <- setup_exnex(k = 3, p0 = 0.2)

# Equal sample sizes
get_evaluation(design = design, n = c(20, 20, 20), r = c(10, 15, 5),
  lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)

# Unequal sample sizes
get_evaluation(design = design, n = c(15, 20, 25), r = c(10, 15, 17),
  lambda = 0.95, tau_scale = 1, w_j = 0.5, iter = 100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_cpplim}
\alias{setup_cpplim}
\title{Set Up Limited Calibrated Power Prior Design Object}
\usage{
setup_cpplim(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{cpplim}
}
\description{
Set Up Limited Calibrated Power Prior Design Object
}
\details{
The class \code{cpplim} implements a combined version of the adaptive power
prior (\code{app}) and the calibrated power prior (\code{cpp}), where the
parameter limiting the amount of information to be borrowed in the adaptive
power prior design is included in the calibrated power prior design.
}
\examples{
design_cpplim <- setup_cpplim(k = 3, p0 = 0.2)
}
\references{
Ollier, A., Morita, S., Ursino, M., & Zohar, S. (2020). An
adaptive power prior for sequential clinical trials - Application to bridging
studies. Statistical methods in medical research, 29(8), 2282–2294.

Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
Design Based on Power Priors. Statistics in Biopharmaceutical Research,
17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_jsdglobal}
\alias{setup_jsdglobal}
\title{Set Up Global JSD Design Object}
\usage{
setup_jsdglobal(k, p0, shape1 = 1, shape2 = 1)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{shape1}{First common shape parameter of the beta prior.}

\item{shape2}{Second common shape parameter of the beta prior.}
}
\value{
An S3 object of class \code{jsdglobal}
}
\description{
Set Up Global JSD Design Object
}
\details{
The class \code{jsdglobal} implements a version of the power prior
design, in which information is shared based on pairwise similarity
and overall heterogeneity between baskets. Both pairwise similarity and
overall heterogeneity are assessed using the Jensen-Shannon divergence.
}
\examples{
design_jsdglobal <- setup_jsdglobal(k = 3, p0 = 0.2)
}
\references{
Baumann, L., Sauer, L. D., & Kieser, M. (2025). A Basket Trial
Design Based on Power Priors. Statistics in Biopharmaceutical Research,
17(3), 446–456. https://doi.org/10.1080/19466315.2024.2402275
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_details.R
\name{get_details.binomial}
\alias{get_details.binomial}
\title{Get Details of a Basket Trial with the Frequentist Binomial Design}
\usage{
\method{get_details}{binomial}(design, n, p1 = NULL, alpha = 0.025, ...)
}
\arguments{
\item{design}{An object of class \code{binomial}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities used for the simulation. If \code{NULL}
then all probabilities are set to p0.}

\item{alpha}{The one-sided significance level.}

\item{...}{Further arguments.}
}
\value{
A list containing the rejection probabilities, critical values
and expected number of correct decisions. Critical values \eqn{c} are defined
so that the null hypothesis is rejected if the observed number of responses
\eqn{r} is greater than \eqn{c}, i.e. \eqn{r > c} rejects \eqn{H_0}.
The nominal FWER is the (theoretical) FWER of a multiple testing problem
with k hypothesis tests at significance level \eqn{\alpha}. The actual FWER is
usually lower than the nominal FWER, as the binomial test does not exhaust
its significance level.
}
\description{
This basic frequentist design conducts a separate binomial test for each
basket. All tests are one-sided, and the alternative is greater than the
null hypothesis. These details are calculated exactly, not simulated.
}
\examples{
design <- setup_binomial(k = 3, p0 = 0.2)
p1 <- c(0.2, 0.5, 0.5)
get_details(design = design, n = 20, p1 = p1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_details.R
\name{get_details.fujikawa}
\alias{get_details.fujikawa}
\title{Get Details of a Basket Trial Simulation with Fujikawa's Design}
\usage{
\method{get_details}{fujikawa}(
  design,
  n,
  p1 = NULL,
  lambda,
  level = 0.95,
  epsilon,
  tau,
  logbase = 2,
  iter = 1000,
  data = NULL,
  use_future = FALSE,
  weight_fun = NULL,
  weight_params = list(epsilon = epsilon, tau = tau, logbase = logbase),
  ...
)
}
\arguments{
\item{design}{An object of class \code{fujikawa}.}

\item{n}{The sample sizes of the baskets. A vector must be used for varying sample sizes.}

\item{p1}{Probabilities used for the simulation. If \code{NULL}
then all probabilities are set to p0.}

\item{lambda}{The posterior probability threshold.}

\item{level}{Level of the credibility intervals.}

\item{epsilon}{Tuning parameter that determines the amount of borrowing.
See \code{\link{setup_fujikawa}}).}

\item{tau}{Tuning parameter that determines how similar the baskets
have to be that information is shared. See \code{\link{setup_fujikawa}}).}

\item{logbase}{Tuning parameter. The base of the logarithm that is used to
calculate the Jensen-Shannon divergence.}

\item{iter}{The number of iterations in the simulation. Is ignored if
\code{data} is specified.}

\item{data}{A data matrix with k column with the number of responses for each
basket. Has to be generated with \code{get_data}. If \code{data} is used,
then \code{iter} is ignored.}

\item{use_future}{A logical, should \code{\%dofuture\%} or \code{\%do\%} be used
for the call to \code{foreach}. Default is \code{FALSE} which means that
\code{\%dofuture\%} is not used. \code{\%dofuture\%}
is needed for parallelization. Note that for actually using parallelized
calculations, one needs to activate a \code{future} backend.}

\item{weight_fun}{A function of the form \code{function(design, n, ...)} that
additionally takes the arguments given in \code{weight_params}. If \code{NULL}, the
original weights suggested by Fujikawa are used (based on the Jensen-Shannon
divergence).}

\item{weight_params}{A named list of input parameters (additional to \code{design}
and \code{n}) for the function \code{weight_fun}.}

\item{...}{Further arguments.}
}
\value{
A list containing the rejection probabilities, posterior means,
mean squared errors and mean limits of HDI intervals for all baskets as well
as the family-wise error rate and the experiment-wise power.
}
\description{
Get Details of a Basket Trial Simulation with Fujikawa's Design
}
\examples{
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)

# Unequal sample sizes
get_details(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)

# A custom weight function can be defined, e.g.
weight_noshare <- function(design, n, epsilon, tau, logbase){
  n_sum <- n + 1
  return(diag(n_sum))
}
get_details(design = design, n = 20, p1 = c(0.2, 0.5, 0.5), lambda = 0.95,
           epsilon = 2, tau = 0, iter = 1000, weight_fun = weight_noshare)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt_design.R
\name{get_scenarios}
\alias{get_scenarios}
\title{Create a Scenario Matrix}
\usage{
get_scenarios(design, p1)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{p1}{Probability under the alternative hypothesis.}
}
\value{
A matrix with \code{k} rows and \code{k + 1} columns.
}
\description{
Creates a default scenario matrix.
}
\details{
\code{get_scenarios} creates a default scenario matrix
that can be used for \code{\link{opt_design}}. The function creates
\code{k + 1} scenarios, from a global null to a global alternative scenario.
}
\examples{
design <- setup_fujikawa(k = 3, p0 = 0.2)
get_scenarios(design = design, p1 = 0.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class.R
\name{setup_bhm}
\alias{setup_bhm}
\title{Set Up BHM Design Object}
\usage{
setup_bhm(k, p0, p_target, mu_mean = NULL, mu_sd = 100)
}
\arguments{
\item{k}{The number of baskets.}

\item{p0}{A common probability under the null hypothesis.}

\item{p_target}{The response rate of interest. See details.}

\item{mu_mean}{Mean of the normal prior distribution for the mean of the
thetas. See details.}

\item{mu_sd}{Standard deviation of the normal prior distribution for the
mean of the thetas.}
}
\value{
An S3 object of class \code{bhm}
}
\description{
Set Up BHM Design Object
}
\details{
The class \code{bhm} implements the Bayesian Hierarchical Model
proposed by Berry et al. (2013). Methods for this class are
mostly wrappers for functions from the package \code{bhmbasket}.

In the BHM the thetas of all baskets are modeled, where theta_i =
logit(p_i) - logit(p_target). These thetas are assumed to come from
a normal distribution with mean mu_mean and standard deviation mu_sd.
If \code{mu_mean = NULL} then mu_mean is determined as logit(p0) -
logit(p_target), hence the mean of the normal distribution corresponds
to the null hypothesis.
}
\examples{
design_bhm <- setup_bhm(k = 3, p0 = 0.2, p_target = 0.5)
}
\references{
Berry, S. M., Broglio, K. R., Groshen, S., & Berry, D. A. (2013).
Bayesian hierarchical modeling of patient subpopulations: efficient designs
of phase II oncology clinical trials. Clinical Trials, 10(5), 720-734.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_results.R
\name{get_results}
\alias{get_results}
\title{Get Results for Simulation of Basket Trial Designs}
\usage{
get_results(design, ...)
}
\arguments{
\item{design}{An object created with one of the \code{setup} functions.}

\item{...}{Further arguments.}
}
\value{
A matrix of results with \code{iter} rows. A 0 means, that the
null hypothesis that the response probability exceeds \code{p0} was not
rejected, a 1 means, that the null hypothesis was rejected.
}
\description{
Get Results for Simulation of Basket Trial Designs
}
\examples{
# Example for a basket trial with Fujikawa's Design
design <- setup_fujikawa(k = 3, p0 = 0.2)

# Equal sample sizes
get_results(design = design, n = 20, p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)

# Unequal sample sizes
get_results(design = design, n = c(15, 20, 25), p1 = c(0.2, 0.5, 0.5),
  lambda = 0.95, epsilon = 2, tau = 0, iter = 100)
}
